#ifndef AXES_H
#define AXES_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CAxes K-3D object, which provides a configurable set of axes to help in visualizing the 3D workspace
		\author Tim Shead (tshead@k-3d.com)
*/

// Forward class declarations ...
class CAxes;

#include <k3ddialog.h>
#include <k3dpersistobject.h>
#include <k3deditorrender.h>
#include <k3ditarget.h>
#include <k3dvariable.h>

/////////////////////////////////////////////////////////////////////////////
// CAxesDialog

class CAxesDialog : public k3dDialog
{
public:
	CAxesDialog();

	bool Create(CAxes* Object);

protected:
	CAxes* m_Object;

	// Event handling ...
	void OnEvent(sdpGtkEvent* Event);

	void OnAxes();
	void OnXYPlane();
	void OnYZPlane();
	void OnXZPlane();

	// Update controls ...
	void UpdateControls();
};


/////////////////////////////////////////////////////
// CAxes

class CAxes :	public k3dPersistObject,
							public k3dEditorRender,
							public k3dITarget
{
friend class CAxesDialog;
public:
	CAxes();
	~CAxes();

	CAxesDialog m_UserInterface;

	k3dVariable<bool> m_Axes;
	k3dVariable<bool> m_XYPlane;
	k3dVariable<bool> m_YZPlane;
	k3dVariable<bool> m_XZPlane;
	k3dVariable<gdouble> m_Size;
	k3dVariable<gulong> m_GridSize;
	k3dVariable<sdpVector3> m_Translation;
	k3dVariable<sdpVector3> m_Rotation;
	k3dVariable<gdouble> m_Sensitivity;

	static double m_XAxisColor[4];
	static double m_YAxisColor[4];
	static double m_ZAxisColor[4];
	static double m_GridColor[4];

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIObject overrides
	sdpResult EditObject();
	sdpResult ObjectRenamed(SDPCSTRING NewName);
	sdpResult ObjectDeleted();

	// k3dIEditorRender overrides
	sdpResult EditorPushRender(k3dIEditorEngine* Engine);
	sdpResult EditorRender(k3dIEditorEngine* Engine);

	// k3dITarget overrides
	sdpResult GetTargetCoordinates(double Coords[3]);

	// k3dIPersist overrides
};


#endif // AXES_H
