#ifndef EDITORENGINE_H
#define EDITORENGINE_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CEditorEngine class, which is responsible for rendering editor-mode graphics into a k3dIView
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpgtkopengldrawingarea.h>
#include <k3direnderengine.h>
#include <k3dieditorengine.h>

// Forward class declarations ...
class CCamera;

//////////////////////////////////////////////////////////////////////////
// CEditorEngine

class CEditorEngine : public k3dIRenderEngine,
									public k3dIEditorEngine
{
friend class CCamera;
public:
	CEditorEngine();
	virtual ~CEditorEngine();

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface) { return SDP_E_FAIL; }
	gulong AddRef() { return 1; }
	gulong Release() { return 0; }

	// k3dIRenderEngine overrides ...
	sdpResult SetCamera(k3dICamera* Camera);
	sdpResult Redraw();
	sdpResult StartJob(k3dIRenderJob* Job);
	sdpResult QueueFrame(k3dIRenderFrame* Frame);
	sdpResult FinishJob(k3dIRenderJob* Job);

	// k3dIEditorEngine overrides ...
	k3dICamera* Camera();
	sdpResult EditorQuerySolid();
	sdpResult EditorQueryShaded();
	sdpResult EditorQueryTextured();
	sdpResult EditorQueryOrientation();
	sdpResult EditorQueryNormals();
	gulong EditorQueryWireframePatchDivisions();
	gulong EditorQuerySolidPatchDivisions();

protected:
	// Owner camera ...
	CCamera* m_Camera;

	// Helper functions ...
	void SetupView(long JitterArrayIndex, long Jitter);
	void SetupViewVolume(GLdouble PixelDX, GLdouble PixelDY, GLdouble EyeDX, GLdouble EyeDY);

	void EditorRender();
	void RenderOptions();
	void RenderWireframeOptions();
	void RenderSolidOptions();

	void PreRender();
	void LightSetup();
	void Render();
	void PostRender();
};



#endif // EDITORENGINE_H
