#include "stdafx.h"

#include "olbtarget.h"
#include "objectlib.h"




CTarget::CTarget() : k3dTransformObject()
{
	SetNumLocations(1);

	m_Sheet = NULL;
}


CTarget::~CTarget()
{
	if(m_Sheet)
		m_Sheet->DestroyWindow();
}


///////////////////////////////////////////////////////////////
// k3dIUnknown overrides

sdpResult CTarget::QueryInterface(const sdpIID& InterfaceID, void** Interface)
{
	if(sdpSupportInterface<sdpIUnknown>(static_cast<k3dIObject*>(this), InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIObject>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIDocumentEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIUserEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIAnimate>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dITimeServer>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIPersist>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIOpenGLRender>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIRiRender>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dILocationServer>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dITransform>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dITarget>(this, InterfaceID, Interface)) return SDP_S_OK;

	return SDP_E_NO_INTERFACE;
}


gulong CTarget::AddRef()
{
	return sdpComAddRef(m_ReferenceCount);
}


gulong CTarget::Release()
{
	return sdpComRelease(this, m_ReferenceCount);
}





///////////////////////////////////////////////////////////////
// k3dIObject overrides

sdpResult CTarget::EditObject()
{
	if(!KOpenDialog(m_Sheet, this)) KFAIL(SDP_E_FAIL);
	return SDP_S_OK;
}


///////////////////////////////////////////////////////////////


sdpResult CTarget::ObjectRenamed(SDPCSTRING NewName)
{
	if(m_Sheet)
		{
			m_Sheet->RenameObject(NewName);
		}

	return SDP_S_OK;
}





/////////////////////////////////////////////////////////////////
// k3dIOpenGLRender overrides ...

sdpResult CTarget::OpenGLRender(k3dICamera* Camera, k3dIOpenGLEngine* Engine)
{
	// Don't draw ourselves in preview mode ...
	if(k3dIOpenGLEngine::MODE_PREVIEW == Engine->OpenGLQueryMode()) return SDP_S_FALSE;

	glDisable(GL_LIGHTING);
	glEnable(GL_POINT_SMOOTH);
	glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);

	// Outer ring
	glColor3d(0.0, 0.0, 0.0);
	glPointSize(10.0f);

	glBegin(GL_POINTS);
	glVertex3d(m_Locations[0].m_AnimXYZ[0], m_Locations[0].m_AnimXYZ[1], m_Locations[0].m_AnimXYZ[2]);
	glEnd();

	// Inner ring
	glColor3d(1.0, 1.0, 1.0);
	glPointSize(6.0f);

	glBegin(GL_POINTS);
	glVertex3d(m_Locations[0].m_AnimXYZ[0], m_Locations[0].m_AnimXYZ[1], m_Locations[0].m_AnimXYZ[2]);
	glEnd();

	// Outer ring
	glColor3d(0.0, 0.0, 0.0);
	glPointSize(2.0f);

	glBegin(GL_POINTS);
	glVertex3d(m_Locations[0].m_AnimXYZ[0], m_Locations[0].m_AnimXYZ[1], m_Locations[0].m_AnimXYZ[2]);
	glEnd();

	return SDP_S_OK;
}



///////////////////////////////////////////////////////////////
// k3dIRiRender overrides ...


///////////////////////////////////////////////////////////////
// k3dILocationServer overrides


///////////////////////////////////////////////////////////////
// k3dITarget overrides


///////////////////////////////////////////////////////////////
// k3dIPersist overrides

sdpResult CTarget::Save(sdpXmlIElement* Element, k3dIDependencies* Dependencies)
{
	if(sdpFailed(k3dTransformObject::Save(Stream, Dependencies)))
		KFAIL(SDP_E_FAIL);

	if(sdpFailed(k3dLocationServer::Save(Stream, Dependencies)))
		KFAIL(SDP_E_FAIL);

	return SDP_S_OK;
}


sdpResult CTarget::Load(sdpXmlIElement* Element)
{
	if(sdpFailed(k3dTransformObject::Load(Element)))
		KFAIL(SDP_E_FAIL);

	if(sdpFailed(k3dLocationServer::Load(Element)))
		KFAIL(SDP_E_FAIL);

	return SDP_S_OK;
}



/////////////////////////////////////////////////////////////////////////////
// CTargetSheet

CTargetSheet::CTargetSheet(CTarget* Target)
 : KSheet((k3dPersistObject*)Target, theApp.m_hInstance, "Target - Directional control for cameras, lights, etc.", VERTICAL, "Target")
{
	m_Object = Target;

	m_TargetTypePage = NULL;
}



CTargetSheet::~CTargetSheet()
{
	m_Object->m_Sheet = NULL;
}



bool CTargetSheet::Create()
{
	m_TargetTypePage = new CTargetTypePage(this);

	if(m_TargetTypePage)
		{
			AddPage(m_TargetTypePage);
			Create(IDD);

			return TRUE;
		}

	KFAIL(FALSE);
}



void CTargetSheet::OnHelp()
{
	m_Object->StandardHelp("objects", "target.html");
}


BEGIN_MESSAGE_MAP(CTargetSheet, KSheet)
	//{{AFX_MSG_MAP(CTargetSheet)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CTargetSheet message handlers


BOOL CTargetSheet::OnInitDialog()
{
	KSheet::OnInitDialog();

	RenameObject(m_Object->m_ObjectContext->Name());


	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}



void CTargetSheet::OnTypeChange()
{
}





/////////////////////////////////////////////////////////////////////////////
// CTargetTypePage property page

CTargetTypePage::CTargetTypePage(CTargetSheet* Parent)
  : KPage(IDD, Parent, "Target")
{
	//{{AFX_DATA_INIT(CTargetTypePage)
	//}}AFX_DATA_INIT

	m_Sheet = Parent;
}




BEGIN_MESSAGE_MAP(CTargetTypePage, KPage)
	//{{AFX_MSG_MAP(CTargetTypePage)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CTargetTypePage message handlers

BOOL CTargetTypePage::OnInitDialog()
{
	KPage::OnInitDialog();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}




