#ifndef OPENGLENGINE_H
#define OPENGLENGINE_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtkopengldrawingarea.h>
#include <k3direnderengine.h>
#include <k3diopenglengine.h>

// Forward class declarations ...
class CCamera;

#define NUM_JITTER_SETS				8
#define NUM_JITTER_SAMPLES_1	1
#define NUM_JITTER_SAMPLES_2	2
#define NUM_JITTER_SAMPLES_3	3
#define NUM_JITTER_SAMPLES_4	4
#define NUM_JITTER_SAMPLES_5	8
#define NUM_JITTER_SAMPLES_6	15
#define NUM_JITTER_SAMPLES_7	24
#define NUM_JITTER_SAMPLES_8	66

class CJitterPoint
{
public:
	GLfloat m_x;
	GLfloat m_y;
};

//////////////////////////////////////////////////////////////////////////
// COpenGLEngine

class COpenGLEngine : public k3dIRenderEngine,
											public k3dIOpenGLEngine
{
friend class CCamera;
public:
	COpenGLEngine();
	virtual ~COpenGLEngine();

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface) { return SDP_E_FAIL; }
	gulong AddRef() { return 1; }
	gulong Release() { return 0; }

	// k3dIRenderEngine overrides ...
	sdpResult SetCamera(k3dICamera* Camera);
	sdpResult OnSize(gulong Width, gulong Height);
	sdpResult RefreshView();
	sdpResult RedrawView(SDPCSTRING ProcessName, bool StartImmediately, bool Synchronous);
	sdpResult RenderView(SDPCSTRING ProcessName, bool StartImmediately, bool Synchronous);
	sdpResult RenderViewToBitmap(SDPCSTRING ProcessName, bool StartImmediately, bool Synchronous, k3dIBitmap* Bitmap);
	sdpResult RenderViewToFile(SDPCSTRING ProcessName, bool StartImmediately, bool Synchronous, SDPCSTRING FileName, k3dIBitmapWriteFormat* Format);
	sdpResult SetupViewToVideo(k3dIVideoWriteFormat* Format);

	// k3dIOpenGLEngine overrides ...
	sdpResult OpenGLQuerySolid();
	sdpResult OpenGLQueryShaded();
	sdpResult OpenGLQueryTextured();
	gulong OpenGLQueryWireframePatchDivisions();
	gulong OpenGLQuerySolidPatchDivisions();

protected:
	// Owner camera ...
	CCamera* m_Camera;

	// Current dimensions of the view ...
	gulong m_Width;
	gulong m_Height;

	// Helper functions ...
//	void UpdateBuffers(bool ForceUpdate);
	void SetupView(long JitterArrayIndex, long Jitter);
	void SetupViewVolume(GLdouble PixelDX, GLdouble PixelDY, GLdouble EyeDX, GLdouble EyeDY);

	void CallPreRender();
	void CallLightSetup();
	void CallRender();
	void CallPostRender();

	void PreviewRender();
	void PreviewOptions();
	void PreviewFieldSetup(long Field, double& FieldTime, double& TotalFrames);
	void PreviewBackgroundSetup(long Field);
	void PreviewFog();
	void PreviewLighting();

	// Jitter constants
	static long m_JitterSamples[NUM_JITTER_SETS];
	static CJitterPoint* m_JitterSetArray[NUM_JITTER_SETS];
	static CJitterPoint m_JitterSet1[NUM_JITTER_SAMPLES_1];
	static CJitterPoint m_JitterSet2[NUM_JITTER_SAMPLES_2];
	static CJitterPoint m_JitterSet3[NUM_JITTER_SAMPLES_3];
	static CJitterPoint m_JitterSet4[NUM_JITTER_SAMPLES_4];
	static CJitterPoint m_JitterSet5[NUM_JITTER_SAMPLES_5];
	static CJitterPoint m_JitterSet6[NUM_JITTER_SAMPLES_6];
	static CJitterPoint m_JitterSet7[NUM_JITTER_SAMPLES_7];
	static CJitterPoint m_JitterSet8[NUM_JITTER_SAMPLES_8];

	// Field-rendering constants
	static GLubyte m_EvenFieldStencil[128];
};



#endif // OPENGLENGINE_H
