// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the functions necessary to export K-3D objects from the experimentallib module
		\author Timothy M. Shead (tshead@k-3d.com)
*/
		
#include <gmodule.h>

#include <strstream>
#include <sdpxml.h>

#include <k3dcom.h>
#include <k3dmisc.h>

#include "softobject.h"

#include "../../k3d_version.h"

extern "C" {

/// Stores the XML document that describes this module and its exported objects
static std::strstream ClassInfo;

/// Called to create an XML stream object that describes this module and its exported objects
static void BuildClassInfo()
{
	// Generate an XML document ...
	sdpXmlDocument document;
	sdpXmlIElement* object;
	sdpXmlIElement*  library = k3dStartLibraryInfo(&document, "experimentallib", VERSION_STRING);

	// SoftObject ...
	object = k3dAddObject(library, sdpSoftObject, "SoftObject");
	k3dAddBookmark(object, "Objects/SoftObject");
	
	ClassInfo << document << std::ends;
}


/// Called when the module is dynamically loaded
G_MODULE_EXPORT const gchar* g_module_check_init(GModule *module)
{
	// Sanity checks ...
	g_assert(module);

	// Build our XML sdpComClassInfo document ...
	BuildClassInfo();

	return 0;
}

/// Called when the module is about to be unloaded
G_MODULE_EXPORT void g_module_unload(GModule *module)
{
	// Sanity checks ...
	g_assert(module);

	// Unlock memory on our XML sdpComClassInfo document ...
	ClassInfo.rdbuf()->freeze(false);
}


/// Called to return an XML document describing this module and its exported objects
G_MODULE_EXPORT SDPCSTRING sdpComLibraryInfo()
{
	return ClassInfo.str();
}


/// Called to create an object of the given type, returning an interface of the given type
G_MODULE_EXPORT sdpResult sdpComClassProcedure(const sdpCLSID& ClassID, const sdpIID& InterfaceID, void** Interface)
{
	if(ClassID == sdpSoftObject)
		return sdpCreateClassInstance<CSoftObject>(InterfaceID, Interface) ? SDP_S_OK : SDP_E_FAIL;

	return SDP_E_UNKNOWN_OBJECT;
}

}	// extern "C"

