#include "stdafx.h"
#include "geometryformatlwob.h"
#include "geometryfilter.h"



///////////////////////////////////////////////////////////////
// k3dIUnknown overrides

sdpResult CGeometryFormatLWOB::QueryInterface(const sdpIID& InterfaceID, void** Interface)
{
	if(sdpSupportInterface<sdpIUnknown>(static_cast<k3dIObject*>(this), InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIObject>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIGeometryReadFormat>(this, InterfaceID, Interface)) return SDP_S_OK;

	return SDP_E_NO_INTERFACE;
}


gulong CGeometryFormatLWOB::AddRef()
{
	return sdpCOMAddRef(m_ReferenceCount);
}


gulong CGeometryFormatLWOB::Release()
{
	return sdpCOMRelease(this, m_ReferenceCount);
}





///////////////////////////////////////////////////////////////
// k3dIGeometryFormat overrides

SDPCSTRING CGeometryFormatLWOB::GetReadFilterString()
{
	return "LightWave-3D Object File (*.lwo)|*.lwo";
}


sdpResult CGeometryFormatLWOB::QueryCanRead(SDPCSTRING FilePath)
{
	sdpString filepath = FilePath;

	filepath.tolower();
	if("lwo" == filepath.right(3)) return SDP_S_OK;

	return SDP_S_FALSE;
}



sdpResult CGeometryFormatLWOB::ReadFile(SDPCSTRING FilePath)
{
#pragma SDPPOSTIT(*** CGeometryFormatLWOB::ReadFile())
/*
	sdpResult hr;

	CFile file;
	gulong chunktype, filesize, chunksize, chunksize2, subchunktype;
	USHORT subchunksize;

	CArray<KHANDLE, KHANDLE> pointarray;
	CArray<CGeometryFormatLWOBSurface, CGeometryFormatLWOBSurface> surfacearray;

	KHANDLE handle;
	CGeometryFormatLWOBSurface surface;
	char* surfacebuffer;

	float coordinate;
	double coords[3];

	unsigned char basecolor[4];
	double color[3];

	gulong i, numpoints;
	USHORT numvertices, vertex;
	short surfaceindex;

	CGeometryFormatLWOBNameDlg dlg;
	k3dIObject* object;

	sdpString s;
	char c;

	dlg.m_Name = "New LightWave Object";

	if(IDOK == dlg.DoModal())
		{
			if(file.Open(FilePath, CFile::modeRead | CFile::shareExclusive))
				{
					m_GroupList->Create();
					m_Group->ObjectContext()->SetName(dlg.m_Name);

					file.Read(&chunktype, 4);
					ByteSwap(&chunktype);
					if('FORM' == chunktype)
						{
							file.Read(&filesize, 4);
							ByteSwap(&filesize);

							file.Read(&chunktype, 4);
							ByteSwap(&chunktype);

							filesize -= 4;

							if(('LWOB' == chunktype) || ('LWLO' == chunktype))
								{
									while(filesize > 0)
										{
											file.Read(&chunktype, 4);
											ByteSwap(&chunktype);
											file.Read(&chunksize, 4);
											ByteSwap(&chunksize);

											filesize -= 8 + chunksize;

											if('PNTS' == chunktype)
												{
													numpoints = chunksize / 12;
													pointarray.SetSize(numpoints, -1);

													for(i = 0; i < numpoints; i++)
														{
															hr = m_GroupPointList->Create();
															if(SDP_S_OK == hr)
																{
																	pointarray.SetAt(i, m_GroupPointList->GetHandle());

																	file.Read(&coordinate, 4);
																	ByteSwap(&coordinate);
																	coords[0] = coordinate;

																	file.Read(&coordinate, 4);
																	ByteSwap(&coordinate);
																	coords[1] = coordinate;

																	file.Read(&coordinate, 4);
																	ByteSwap(&coordinate);
																	coords[2] = coordinate;

																	m_GroupPointLocation->SetOldXYZ(coords);
																	m_GroupPointLocation->SetLocalXYZ(coords);
																}
														}
												}
											else if('SRFS' == chunktype)
												{
													surfacebuffer = (char*)malloc(chunksize);
													if(surfacebuffer)
														{
															file.Read(surfacebuffer, chunksize);

															i = 0;

															while(i < chunksize)
																{
																	surface.m_Name.Empty();
																	surface.m_Name = &surfacebuffer[i];
																	m_Document->CreateObject(sdpMaterial, IID_k3dIObject, FALSE, TRUE, TRUE, (void**)&object);
																	hr = object->QueryInterface(IID_k3dIMaterial, (void**)&surface.m_k3dIMaterial);
																	s.Format("%s %s Material", dlg.m_Name, surface.m_Name);
																	object->ObjectContext()->SetName(s);
																	object->Release();

																	surfacearray.Add(surface);

																	while(surfacebuffer[i] != '\0')
																		{
																			i++;
																		}

																	while(surfacebuffer[i] == '\0')
																		{
																			i++;
																		}
																}

															free(surfacebuffer);
														}
												}
											else if('POLS' == chunktype)
												{
													chunksize2 = chunksize;

													while(chunksize2 > 0)
														{
															file.Read(&numvertices, 2);
															ByteSwap(&numvertices);
															chunksize2 -= 2;

															hr = m_GroupPolygonList->Create();
															if(SDP_S_OK == hr)
																{
																	for(i = 0; i < numvertices; i++)
																		{
																			file.Read(&vertex, 2);
																			ByteSwap(&vertex);
																			chunksize2 -= 2;

																			handle = pointarray.GetAt(vertex);

																			hr = m_GroupPolygon->AttachPointAfter(handle);
																		}
																}

															file.Read(&surfaceindex, 2);
															ByteSwap(&surfaceindex);
															chunksize2 -= 2;

															m_GroupPolygonMaterialServer->SetMaterial(surfacearray[surfaceindex - 1].m_k3dIMaterial);
														}
												}
											else if('CRVS' == chunktype)
												{
													file.Seek(chunksize, CFile::current);
												}
											else if('PCHS' == chunktype)
												{
													file.Seek(chunksize, CFile::current);
												}
											else if('SURF' == chunktype)
												{
													chunksize2 = chunksize;

													// Get surface name
													s.Empty();
													file.Read(&c, 1);
													chunksize2--;
													while(c != '\0')
														{
															s += c;
															file.Read(&c, 1);
															chunksize2--;
														}

													while(c == '\0')
														{
															file.Read(&c, 1);
															chunksize2--;
														}

													chunksize2++;
													file.Seek(-1, CFile::current);

													// Get matching surface from array
													for(surfaceindex = 0; surfaceindex < surfacearray.GetSize(); surfaceindex++)
														{
															if(s == surfacearray[surfaceindex].m_Name) break;
														}

													while(chunksize2 > 0)
														{
															file.Read(&subchunktype, 4);
															ByteSwap(&subchunktype);
															file.Read(&subchunksize, 2);
															ByteSwap(&subchunksize);

															chunksize2 -= 6 + subchunksize;

															if('COLR' == subchunktype)
																{
																	file.Read(&basecolor, 4);

																	color[0] = color[1] = color[2] = 0.0;
																	surfacearray[surfaceindex].m_k3dIMaterial->SetAmbient(color);
																	surfacearray[surfaceindex].m_k3dIMaterial->SetEmissive(color);

																	color[0] = (double)basecolor[0] / 255.0;
																	color[1] = (double)basecolor[1] / 255.0;
																	color[2] = (double)basecolor[2] / 255.0;
																	surfacearray[surfaceindex].m_k3dIMaterial->SetDiffuse(color);

																	color[0] = color[1] = color[2] = 1.0;
																	surfacearray[surfaceindex].m_k3dIMaterial->SetSpecular(color);
																}
															else if('SPEC' == subchunktype)
																{
																	if(4 == subchunksize)
																		{
																			subchunksize = 2;
																			chunksize2 += 2;
																		}

																	file.Seek(subchunksize, CFile::current);
																}
															else if('REFL' == subchunktype)
																{
																	if(4 == subchunksize)
																		{
																			subchunksize = 2;
																			chunksize2 += 2;
																		}

																	file.Seek(subchunksize, CFile::current);
																}
															else if('GLOS' == subchunktype)
																{
																	if(4 == subchunksize)
																		{
																			subchunksize = 2;
																			chunksize2 += 2;
																		}

																	file.Seek(subchunksize, CFile::current);
																}
															else // Unknown Surface chunk type
																{
																	file.Seek(subchunksize, CFile::current);
																}
														}
												}
											else	// Unknown chunk type
												{
													try
														{
															file.Seek(chunksize, CFile::current);
														}
													catch(CException* exception)
														{
															exception->Delete();
															break;
														}
												}

										}
								}
							else
								{
									m_IO->Error("File isn't a LightWave-3D Object File", "Load .lwo file:");
									KFAIL(SDP_E_FAIL);
								}
						}
					else
						{
							m_IO->Error("File isn't a LightWave-3D Object File", "Load .lwo file:");
							KFAIL(SDP_E_FAIL);
						}
				}
			else
				{
					m_IO->Error("Error opening LightWave-3D Object File", "Load .lwo file:");
					KFAIL(SDP_E_FAIL);
				}
		}
*/
	return SDP_S_OK;
}



void CGeometryFormatLWOB::ByteSwap(unsigned long *Arg)
{
	char temp;
	char* arg = (char*)Arg;

	temp = arg[3];
	arg[3] = arg[0];
	arg[0] = temp;

	temp = arg[1];
	arg[1] = arg[2];
	arg[2] = temp;
}



void CGeometryFormatLWOB::ByteSwap(float *Arg)
{
	char temp;
	char* arg = (char*)Arg;

	temp = arg[3];
	arg[3] = arg[0];
	arg[0] = temp;

	temp = arg[1];
	arg[1] = arg[2];
	arg[2] = temp;
}



void CGeometryFormatLWOB::ByteSwap(unsigned short *Arg)
{
	char temp;
	char* arg = (char*)Arg;

	temp = arg[1];
	arg[1] = arg[0];
	arg[0] = temp;
}



void CGeometryFormatLWOB::ByteSwap(short *Arg)
{
	char temp;
	char* arg = (char*)Arg;

	temp = arg[1];
	arg[1] = arg[0];
	arg[0] = temp;
}




/////////////////////////////////////////////////////////////////////////////
// CGeometryFormatLWOBNameDlg dialog


CGeometryFormatLWOBNameDlg::CGeometryFormatLWOBNameDlg(CWnd* pParent)
	: CDialog(CGeometryFormatLWOBNameDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CGeometryFormatLWOBNameDlg)
	m_Name = _T("");
	//}}AFX_DATA_INIT

	AfxSetResourceHandle(theApp.m_hInstance);
}


void CGeometryFormatLWOBNameDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CGeometryFormatLWOBNameDlg)
	DDX_Text(pDX, IDC_NEWNAME, m_Name);
	DDV_MaxChars(pDX, m_Name, 80);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CGeometryFormatLWOBNameDlg, CDialog)
	//{{AFX_MSG_MAP(CGeometryFormatLWOBNameDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGeometryFormatLWOBNameDlg message handlers

