#ifndef JAVASCRIPTENGINE_H
#define JAVASCRIPTENGINE_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CJavaScriptEngine and related classes, which allow K-3D scripts to be written in the JavaScript language
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3discriptengine.h>
#include "../../../libjs/jsapi.h"

// Forward class declarations ...
class k3dIApplication;
class k3dIDocument;
class k3dIRenderJob;
class k3dIRenderFrame;
class k3dICamera;
class k3dIObject;
class k3dIMesh;
class k3dIPoint;
class k3dIPolygon;
class k3dIPolyPointServer;
class k3dILocation;

///////////////////////////////////////////////////////////////////////////
// CJavaScriptLocation

class CJavaScriptLocation
{
public:
	static JSObject* Create(k3dILocation* Location, JSContext* Context, JSObject* ParentObject);
	static k3dILocation* Location(JSContext* Context, JSObject* Object);

protected:
	// Standard JS object data ...
	typedef enum
	{
		PROPERTY_OLD,
		PROPERTY_LOCAL,
		PROPERTY_ANIMATED,
		PROPERTY_WORLD,
		PROPERTY_CAMERA,
		PROPERTY_SCREEN,
		PROPERTY_NORMAL,
		PROPERTY_TEXTURE,
		PROPERTY_IMPLICITTEXTURE,
	} PropertyID;

	static JSClass Class;
	static JSPropertySpec Properties[];
	static JSFunctionSpec Methods[];

	// K-3D Location properties ...
	static JSBool GetOld(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetOld(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetLocal(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetLocal(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetAnimated(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetAnimated(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetWorld(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetWorld(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetCamera(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetCamera(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetScreen(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetScreen(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetNormal(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetNormal(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetTexture(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetTexture(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetImplicitTexture(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetImplicitTexture(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	

	// K-3D Location methods ...
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptPoint

class CJavaScriptPoint
{
public:
	static JSObject* Create(k3dIPoint* Point, JSContext* Context, JSObject* MeshObject);
	static k3dIPoint* Point(JSContext* Context, JSObject* Object);

protected:
	// Standard JS object data ...
	typedef enum
	{
		PROPERTY_MESH,
		PROPERTY_LOCATION,
	} PropertyID;

	static JSClass Class;
	static JSPropertySpec Properties[];
	static JSFunctionSpec Methods[];

	// K-3D Point properties ...
	static JSBool GetMesh(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetLocation(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	

	// K-3D Point methods ...
//	static JSBool CreatePoint(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptPolyPointServer

class CJavaScriptPolyPointServer
{
public:
	static JSObject* Create(k3dIPolyPointServer* PolyPointServer, JSContext* Context, JSObject* ParentObject);
	static k3dIPolyPointServer* PolyPointServer(JSContext* Context, JSObject* Object);

protected:
	// Standard JS object data ...
	typedef enum
	{
//		PROPERTY_MESH,
//		PROPERTY_POINTS,
	} PropertyID;

	static JSClass Class;
	static JSPropertySpec Properties[];
	static JSFunctionSpec Methods[];

	// K-3D PolyPointServer properties ...
//	static JSBool GetMesh(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
//	static JSBool GetPoints(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	

	// K-3D PolyPointServer methods ...
	static JSBool AddPoint(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool RemovePoint(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptPolygon

class CJavaScriptPolygon
{
public:
	static JSObject* Create(k3dIPolygon* Polygon, JSContext* Context, JSObject* MeshObject);
	static k3dIPolygon* Polygon(JSContext* Context, JSObject* Object);

protected:
	// Standard JS object data ...
	typedef enum
	{
		PROPERTY_MESH,
		PROPERTY_POINTS,
	} PropertyID;

	static JSClass Class;
	static JSPropertySpec Properties[];
	static JSFunctionSpec Methods[];

	// K-3D Polygon properties ...
	static JSBool GetMesh(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetPoints(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	

	// K-3D Polygon methods ...
//	static JSBool CreatePoint(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptMesh

class CJavaScriptMesh
{
public:
	static JSObject* Create(k3dIMesh* Mesh, JSContext* Context, JSObject* ObjectObject);
	static k3dIMesh* Mesh(JSContext* Context, JSObject* Object);

protected:
	// Standard JS object data ...
	typedef enum
	{
/*
		PROPERTY_DOCUMENT,
		PROPERTY_NAME,
		PROPERTY_POSITION,
		PROPERTY_SCALE,
		PROPERTY_ROTATION,
*/
	} PropertyID;

	static JSClass Class;
	static JSPropertySpec Properties[];
	static JSFunctionSpec Methods[];

	// K-3D Mesh properties ...
/*
	static JSBool GetDocument(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetName(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetName(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetPosition(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetPosition(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetScale(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetScale(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetRotation(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetRotation(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
*/	
	// K-3D Mesh methods ...
	static JSBool CreatePoint(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool CreatePolygon(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptObject

class CJavaScriptObject
{
public:
	static JSObject* Create(k3dIObject* Object, JSContext* Context, JSObject* DocumentObject);
	static k3dIObject* Object(JSContext* Context, JSObject* Object);

protected:
	// Standard JS object data ...
	typedef enum
	{
		PROPERTY_DOCUMENT,
		PROPERTY_NAME,
		PROPERTY_POSITION,
		PROPERTY_ORIENTATION,
		PROPERTY_SCALE,
		PROPERTY_MESH,
	} PropertyID;

	static JSClass Class;
	static JSPropertySpec Properties[];
	static JSFunctionSpec Methods[];

	// K-3D Object properties ...
	static JSBool GetDocument(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetName(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetName(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetPosition(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetPosition(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetOrientation(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetOrientation(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetScale(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetScale(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetMesh(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	
	// K-3D Object methods ...
	static JSBool EditObject(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptCamera

class CJavaScriptCamera
{
public:
	static JSObject* Create(k3dICamera* Camera, JSContext* Context, JSObject* DocumentObject);
	static k3dICamera* Camera(JSContext* Context, JSObject* Object);

protected:
	// Standard JS object data ...
	typedef enum
	{
		PROPERTY_DOCUMENT,
	} PropertyID;

	static JSClass Class;
	static JSPropertySpec Properties[];
	static JSFunctionSpec Methods[];

	// K-3D View object properties ...
	static JSBool GetDocument(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);

	// K-3D View object methods ...
	static JSBool Redraw(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool StartJob(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool QueueFrame(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool FinishJob(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptDocument

class CJavaScriptDocument
{
public:
	static JSObject* Create(k3dIDocument* Document, JSContext* Context, JSObject* ApplicationObject);
	static k3dIDocument* Document(JSContext* Context, JSObject* Object);

protected:
	// Standard JS object data ...
	typedef enum
	{
		PROPERTY_APPLICATION,
		PROPERTY_ACTIVECAMERA,
		PROPERTY_EDITMODE,
	} PropertyID;

	static JSClass Class;
	static JSPropertySpec Properties[];
	static JSFunctionSpec Methods[];

	// K-3D Document object properties ...
	static JSBool GetApplication(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetActiveCamera(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetEditMode(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool SetEditMode(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);

	// K-3D Document object methods ...
	static JSBool Save(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool SaveAs(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool CreateObject(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool Object(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool DeleteObject(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool RedrawAll(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptRenderFrame

class CJavaScriptRenderFrame
{
public:
	static JSObject* Create(k3dIRenderFrame* Job, JSContext* Context, JSObject* JobObject);
	static k3dIRenderFrame* Frame(JSContext* Context, JSObject* Object);

protected:
	// Standard JS object data ...
	typedef enum
	{
		PROPERTY_JOB,
		PROPERTY_NAME,
		PROPERTY_PATH,
		PROPERTY_PRIORITY,
		PROPERTY_VIEWDURINGRENDER,
		PROPERTY_VIEWAFTERRENDER,
		PROPERTY_ENGINE,
	} PropertyID;

	static JSClass Class;
	static JSPropertySpec Properties[];
	static JSFunctionSpec Methods[];

	// K-3D Render Frame object properties ...
	static JSBool GetJob(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetName(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetPath(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetPriority(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool SetPriority(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetViewDuringRender(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool SetViewDuringRender(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetViewAfterRender(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool SetViewAfterRender(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetEngine(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool SetEngine(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);

	// K-3D Render Frame object methods ...
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptRenderJob

class CJavaScriptRenderJob
{
public:
	static JSObject* Create(k3dIRenderJob* Job, JSContext* Context, JSObject* ApplicationObject);
	static k3dIRenderJob* Job(JSContext* Context, JSObject* Object);

protected:
	// Standard JS object data ...
	typedef enum
	{
		PROPERTY_APPLICATION,
		PROPERTY_NAME,
		PROPERTY_PATH,
	} PropertyID;

	static JSClass Class;
	static JSPropertySpec Properties[];
	static JSFunctionSpec Methods[];

	// K-3D Render Job object properties ...
	static JSBool GetApplication(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetName(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetPath(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);

	// K-3D Render Job object methods ...
	static JSBool CreateFrame(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

//////////////////////////////////////////////////////
// CJavaScriptApplication

class CJavaScriptApplication
{
public:
	static JSObject* Create(k3dIApplication* Application, JSContext* Context, JSObject* GlobalObject);
	static k3dIApplication* Application(JSContext* Context, JSObject* Object);
	
protected:
	// Standard JS object data ...
	typedef enum
	{
		PROPERTY_ACTIVEDOCUMENT,
		PROPERTY_APPLICATIONPATH,
		PROPERTY_TEMPPATH,
	} PropertyID;

	static JSClass Class;
	static JSPropertySpec Properties[];
	static JSFunctionSpec Methods[]; 
			
	// K-3D Application object properties ...
	static JSBool GetActiveDocument(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetBasePath(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);

	// K-3D Application object methods ...
	static JSBool Close(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval* Result);

	static JSBool OpenDocument(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval* Result);
	static JSBool CreateDocument(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval* Result);
	static JSBool CloseDocument(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval* Result);

	static JSBool CreateJob(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval* Result);
	static JSBool StartJob(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval* Result);
			
	static JSBool BrowserNavigate(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval *Result);
	static JSBool Message(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval *Result);
	static JSBool Query(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval *Result);
	static JSBool Error(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval *Result);
};

/////////////////////////////////////////////////////
// CJavaScriptEngine

class CJavaScriptEngine : public k3dIScriptEngine
{
public:
	CJavaScriptEngine();
	virtual ~CJavaScriptEngine();

	// sdpIUnknown overrides ...
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIScriptEngine overrides ...
	SDPCSTRING Language();
	sdpResult SetOwner(k3dIApplication* Application);
	sdpResult Execute(SDPCSTRING Script);
	
protected:
	k3dIApplication* m_Application;
	gulong m_ReferenceCount;
	
	JSRuntime* m_Runtime;
	JSContext* m_Context;
	JSObject* m_GlobalObject;
	
	static JSBool RawBranchCallback(JSContext* Context, JSScript* Script);
	JSBool BranchCallback(JSContext* Context, JSScript* Script);
};


#endif // JAVASCRIPTENGINE_H
