#ifndef ANIMATIONPARAMETERS_H
#define ANIMATIONPARAMETERS_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CAnimationParametersDialog, for altering a document's animation parameters
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpgtkobjectcontainer.h>

// Forward class references ...
class CDocument;

///////////////////////////////////////////////////////////////////
// CAnimationParametersDialog

/// Provides a user interface for altering a document's start/end times, framerate, etc.
class CAnimationParametersDialog : public sdpGtkObjectContainer
{
public:
	CAnimationParametersDialog();

	/// Called to create the UI
	bool Create(CDocument* Document);

protected:
	/// Called to handle our custom events
	void OnEvent(sdpGtkEvent* Event);

	void OnDestroy();
	void OnDuration();
	void OnFrameRate();
	void OnTotalFrames();
	void OnFirstFrame();
	void OnViewFrame();
	void OnLastFrame();
	void OnIdle();

	/// Called to put formatted data in all dialog controls
	void UpdateControls();

	/// Stores the document
	CDocument* m_Document;
	
	/// Stores the last revision of the animation parameters, so we can detect when they change
	gulong m_LastTimebaseRevision;
};

#endif ANIMATIONPARAMETERS_H
