#ifndef HIERARCHYFRAME_H
#define HIERARCHYFRAME_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CHierarchyFrame class, which displays a hierarchical view of an open K-3D document, using CHierarchyControl
		\author Tim Shead (tshead@k-3d.com)
*/

#include "hierarchycontrol.h"

// Forward class references
class CDocument;
class k3dIDependencies;
class sdpXmlIElement;

///////////////////////////////////////////////////////////
// CHierarchyFrame

class CHierarchyFrame : public sdpGtkObjectContainer
{
public:
	CHierarchyFrame();
	virtual ~CHierarchyFrame();

	// Initialization ...
	bool Create(CDocument* Document);
	bool Load(CDocument* Document, sdpXmlIElement* Element);

	// Persistence ...
	bool Save(sdpXmlIElement* Element, k3dIDependencies* Dependencies);

	/// Returns the owning document for this frame
	CDocument* Document() { return m_Document; }

	// Update titlebar ...
	sdpString Title();
	void UpdateTitlebar();

protected:
	// Event handlers ...
	void OnEvent(sdpGtkEvent* Event);
	void OnDelete(sdpGtkEvent* Event);
	void OnDestroy();
	void OnClose();
	bool OnSetFocus();

	/// Stores a reference to the owning document
	CDocument* m_Document;

	/// The hierarchy control itself
	CHierarchyControl m_Control;	
};

#endif HIERARCHYFRAME_H
