#ifndef LAYER_H
#define LAYER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CLayer class, which encapsulates a layer within a CDocument
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dilayer.h>
#include <k3dactionclient.h>

// Forward class declarations
class CDocument;
class CObjectInfo;
class CObjectContext;

///////////////////////////////////////////////////////////////////
// CLayer

class CLayer : public k3dILayer,
							public k3dActionClient
{
public:
	CLayer();
	virtual ~CLayer();

	// Creation ...
	bool Create(CDocument* Document);

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dILayer overrides
	k3dIDocument* Document();

	// k3dIActionClient overrides ...
	sdpResult HandleStep(gulong ValueCode, gulong Value, gulong ValueSize);

	// Data access ...
	SDPCSTRING Name() { return m_Name; }

	// Object management ...
	sdpResult AttachObjectContext(CObjectInfo* Info, k3dIObject* Object, bool UserVisible, bool Saveable);
	sdpResult AttachObjectContext(CObjectInfo* Info, k3dIObject* Object, sdpXmlIElement* Element);
	
	void DeleteObjectContext(k3dIObjectContext* Context);

	// Owning document ...
	CDocument* m_Document;

	// Owned objects ...
	typedef std::vector<CObjectContext*> ObjectContextList;
	typedef ObjectContextList::iterator ObjectContextIterator;
	ObjectContextList m_Objects;
	ObjectContextList m_DeletedObjects;

protected:
	typedef enum
	{
		STEP_UNCREATE_OBJECT,
		STEP_CREATE_OBJECT,
		STEP_UNDELETE_OBJECT,
		STEP_DELETE_OBJECT,
	} StepType;

	static gulong m_LayerNumber;
	sdpString m_Name;
};

#endif LAYER_H
