#ifndef OBJECTCONTEXT_H
#define OBJECTCONTEXT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CObjectContext class, which encapsulates data common to all K-3D objects
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3diobjectcontext.h>
#include <k3dipersist.h>
#include <k3dstate.h>

// Forward class references
class CLayer;
class CObjectInfo;

class k3dIUserEventClient;
class k3dITimeServer;
class k3dIAnimate;
class k3dITransform;
class k3dIMesh;
class k3dILocationServer;

///////////////////////////////////////////////////////////
// CObjectContext

class CObjectContext : public k3dIObjectContext,
										public k3dIPersist,
										public k3dState
{
friend class CDocumentFrame;
public:
	CObjectContext();
	CObjectContext(CLayer* Layer, CObjectInfo* ObjectInfo, k3dIObject* Object);
	virtual ~CObjectContext();

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIObjectContext overrides
	SDPCSTRING Name();
	k3dIObject* Object();
	const k3dID ObjectID();
	k3dIObjectContext* Parent();
	k3dIObjectInfo* ObjectInfo();
	sdpResult SetName(SDPCSTRING NewName);
	sdpResult SetActiveObject(bool State);
	k3dIState* State();
	k3dILayer* Layer();
	sdpResult QueryAncestor(k3dIObjectContext* ObjectContext);
	sdpResult QueryParent(k3dIObjectContext* ObjectContext);
	sdpResult QueryChild(k3dIObjectContext* ObjectContext);
	sdpResult QueryDescendant(k3dIObjectContext* ObjectContext);
	sdpResult SetParent(k3dIObjectContext* ObjectContext);
	sdpResult SetSibling(k3dIObjectContext* ObjectContext);

	// k3dIPersist overrides ...
	sdpResult InitNew();
	sdpResult Load(sdpXmlIElement* Element);
	sdpResult LoadComplete();
	sdpResult Save(sdpXmlIElement* Element, k3dIDependencies* Dependencies);
	sdpResult SaveComplete();

	// k3dIState overrides ...
	sdpResult SetState(gulong Mask, bool State);

	// Owners ...
	CLayer* m_Layer;

	// Cached interfaces for speed ...
	k3dIObject* m_Object;
	k3dIUserEventClient* m_UserEventClient;
	k3dITimeServer* m_TimeServer;
	k3dIAnimate* m_Animate;
	k3dITransform* m_Transform;
	k3dIMesh* m_Mesh;
	k3dILocationServer* m_LocationServer;
	k3dIPersist* m_Persist;

protected:
	// Reference count for sdpIUnknown ...
	gulong m_ReferenceCount;

	// Object info ...
	CObjectInfo* m_ObjectInfo;

	// Hierarchy ...
	CObjectContext* m_Parent;

	// Unique Object ID ...
	k3dID m_ObjectID;

	// Object Name ...
	sdpString m_Name;

	// Object hierarchy ...
	HierarchyTreeIterator GetPushRecord();
	HierarchyTreeIterator GetNodeRecord();
	HierarchyTreeIterator GetPopRecord();
};


#endif OBJECTCONTEXT_H
