#ifndef RENDERFRAME_H
#define RENDERFRAME_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CRenderFrame class, which encapsulates a frame to be rendered as part of a job
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3direnderframe.h>
#include <sdpstring.h>
#include <list>

//////////////////////////////////////////////////////////////////////////////////
// CRenderFrame

class CRenderFrame : public k3dIRenderFrame
{
public:
	CRenderFrame(k3dIRenderJob* Job, SDPCSTRING FrameName);
	virtual ~CRenderFrame();
	
	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface) { return SDP_E_NO_INTERFACE; }
	gulong AddRef() { return 0; }
	gulong Release() { return 0; }

	// k3dIRenderFrame overrides ...
	k3dIRenderJob* Job();
	SDPCSTRING Name();
	SDPCSTRING Path();
	sdpResult GetUniqueFile(SDPCSTRING File, SDPSTRING Buffer, gulong* BufferLength);
	sdpResult SetPriority(guchar Priority);
	guchar GetPriority();
	sdpResult SetViewDuringRender(bool View);
	bool GetViewDuringRender();
	sdpResult SetViewAfterRender(bool View);
	bool GetViewAfterRender();
	sdpResult SetEngineType(SDPCSTRING EngineType);
	SDPCSTRING GetEngineType();
	sdpResult SetEngine(SDPCSTRING Engine);
	SDPCSTRING GetEngine();

	/// Called to write the control file for this frame (an XML document which describes how the frame should be rendered)
	bool WriteControlFile();
	/// Called to write the ready status file for this frame (an empty file that indicates that the frame is ready for rendering)
	bool MarkReady();

protected:
	k3dIRenderJob* m_Job;
	sdpString m_Name;
	sdpString m_Path;
	guchar m_Priority;
	bool m_ViewDuringRender;
	bool m_ViewAfterRender;
	sdpString m_EngineType;
	sdpString m_Engine;
};

//////////////////////////////////////////////////////////////////////////////////
// FrameArray

typedef std::list<CRenderFrame*> RenderFrameList;
typedef RenderFrameList::iterator RenderFrameIterator;

#endif // RENDERFRAME_H
