// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the CStep and CAction classes for storing undo/redo data within a document
		\author Timothy M. Shead (tshead@k-3d.com)
*/
#include "action.h"
#include <k3diactionclient.h>

////////////////////////////////////////////////////////////////////////////////////////
// CStep

CStep::CStep()
{
	m_Client = 0;
	m_OldValueCode = 0;
	m_OldValue = 0;
	m_OldValueSize = 0;
	m_NewValueCode = 0;
	m_NewValue = 0;
	m_NewValueSize = 0;
}

CStep::CStep(k3dIActionClient* Client, gulong OldValueCode, gulong OldValue, gulong OldValueSize, gulong NewValueCode, gulong NewValue, gulong NewValueSize)
{
	// Sanity checks ...
	g_assert(Client);

	// Store the data ...
	m_Client = Client;
	m_OldValueCode = OldValueCode;
	m_OldValue = OldValue;
	m_OldValueSize = OldValueSize;
	m_NewValueCode = NewValueCode;
	m_NewValue = NewValue;
	m_NewValueSize = NewValueSize;
}

void CStep::Erase()
{
	if(m_OldValue && m_OldValueSize)
		free(reinterpret_cast<void*>(m_OldValue));
		
	if(m_NewValue && m_NewValueSize)
		free(reinterpret_cast<void*>(m_NewValue));
}

void CStep::Undo()
{
	// Tell the client to go back to the old data in this step ...
	m_Client->HandleStep(m_OldValueCode, m_OldValue, m_OldValueSize);
}

void CStep::Redo()
{
	// Tell the client to go forward to the new data in this step ...
	m_Client->HandleStep(m_NewValueCode, m_NewValue, m_NewValueSize);
}

////////////////////////////////////////////////////////////////////////////////////////
// CAction

CAction::CAction()
{
}

CAction::CAction(SDPCSTRING Name)
{
	// Sanity checks ...
	g_assert_string(Name);

	// Store the data ...
	m_Name = Name;
}



