// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the CAnimationParametersDialog, for altering a document's animation parameters
		\author Tim Shead (tshead@k-3d.com)
*/

#include "animationparameters.h"
#include "application.h"
#include "document.h"

#include <sdpgtkevent.h>
#include <sdpfile.h>

///////////////////////////////////////////////////////////////////////
// CAnimationParametersDialog

CAnimationParametersDialog::CAnimationParametersDialog()
{
	m_Document = 0;
	m_LastTimebaseRevision = 0;
}

bool CAnimationParametersDialog::Create(CDocument* Document)
{
	// Sanity checks ...
	g_assert(Document);
	m_Document = Document;

	// Load the dialog template ...
	g_return_val_if_fail(LoadFile(sdpAppendPath(Application.DialogPath(), "animationparameters.gtkml")), false);

	// Update controls ...
	UpdateControls();

	// Make it all visible ...
	RootWidget().Show();

	return true;
}

void CAnimationParametersDialog::OnEvent(sdpGtkEvent* Event)
{
	// Sanity checks ...
	g_assert(Event);

	if(Event->Name() == "ondestroy")
		OnDestroy();
	else if(Event->Name() == "onduration")
		OnDuration();
	else if(Event->Name() == "onframerate")
		OnFrameRate();
	else if(Event->Name() == "ontotalframes")
		OnTotalFrames();
	else if(Event->Name() == "onfirstframe")
		OnFirstFrame();
	else if(Event->Name() == "onviewframe")
		OnViewFrame();
	else if(Event->Name() == "onlastframe")
		OnLastFrame();
	else if(Event->Name() == "onidle")
		OnIdle();
	else
		sdpGtkObjectContainer::OnEvent(Event);
}

void CAnimationParametersDialog::OnDestroy()
{
	// No more events from this point forward ...
	DisconnectAllEvents();
	
	// Zap leftover widget pointers ...
	Clear();

}

void CAnimationParametersDialog::OnDuration()
{
	m_Document->SetDuration(atof(Entry("duration").GetText()));
}

void CAnimationParametersDialog::OnFrameRate()
{
	m_Document->SetFrameRate(atof(Entry("framerate").GetText()));
}

void CAnimationParametersDialog::OnTotalFrames()
{
	m_Document->SetTotalFrames(atol(Entry("totalframes").GetText()));
}

void CAnimationParametersDialog::OnFirstFrame()
{
	m_Document->SetStartFrame(atol(Entry("firstframe").GetText()));
}

void CAnimationParametersDialog::OnViewFrame()
{
	m_Document->SetViewFrame(atol(Entry("viewframe").GetText()));
}

void CAnimationParametersDialog::OnLastFrame()
{
	m_Document->SetEndFrame(atol(Entry("lastframe").GetText()));
}

void CAnimationParametersDialog::OnIdle()
{
	// If nothing's changed, we're done ...
	if(m_Document->m_TimebaseRevision == m_LastTimebaseRevision)
		return;
		
	m_LastTimebaseRevision = m_Document->m_TimebaseRevision;
	
	UpdateControls();
}

void CAnimationParametersDialog::UpdateControls()
{
	// Sanity checks ...
	g_assert(m_Document);

	Entry("duration").SetText(sdpFormatString("%g", m_Document->GetDuration()));
	Entry("framerate").SetText(sdpFormatString("%g", m_Document->GetFrameRate()));
	Entry("totalframes").SetText(sdpFormatString("%u", m_Document->GetTotalFrames()));
	Entry("firstframe").SetText(sdpFormatString("%u", m_Document->GetStartFrame()));
	Entry("viewframe").SetText(sdpFormatString("%u", m_Document->GetViewFrame()));
	Entry("lastframe").SetText(sdpFormatString("%u", m_Document->GetEndFrame()));
//	Entry("firstframe").SetText(sdpFormatString("%d", gulong(floor(m_StartTime * m_Duration * m_FrameRate + 1.5))));
//	Entry("lastframe").SetText(sdpFormatString("%d", gulong(floor(m_EndTime * m_Duration * m_FrameRate + 1.5))));
}

