// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CMessageBox, CErrorBox, and CQueryBox informational dialogs
		\author Tim Shead (tshead@k-3d.com)
*/

#include "dialogs.h"
#include "application.h"

#include <sdpgtkevent.h>
#include <sdpfile.h>

//////////////////////////////////////////////////////////////////////
// CMessageBox

void CMessageBox::DoModal(SDPCSTRING Caption, SDPCSTRING Message)
{
	g_return_if_fail(Create(Caption, Message));
	sdpGtkObjectContainer::DoModal();
}

bool CMessageBox::Create(SDPCSTRING Caption, SDPCSTRING Message)
{
	// Load the dialog template ...
	g_return_val_if_fail(LoadFile(sdpAppendPath(Application.DialogPath(), "message.gtkml")), false);

	// Set our window caption and title
	RootWindow().SetTitle(Caption);
	Label("message").SetText(Message);
	
	// Center it ...
	gtk_window_set_position(RootWindow(), GTK_WIN_POS_CENTER);

	// Make it all visible ...
	RootWidget().Show();

	return true;
}

void CMessageBox::OnEvent(sdpGtkEvent* Event)
{
	// Sanity checks ...
	g_assert(Event);

	if(Event->Name() == "onok")
		RootWidget().Destroy();
	else if(Event->Name() == "ondestroy")
		CancelModal();
	else		
		sdpGtkObjectContainer::OnEvent(Event);
}

//////////////////////////////////////////////////////////////////////
// CErrorBox

void CErrorBox::DoModal(SDPCSTRING Caption, SDPCSTRING Message)
{
	g_return_if_fail(Create(Caption, Message));
	sdpGtkObjectContainer::DoModal();
}

bool CErrorBox::Create(SDPCSTRING Caption, SDPCSTRING Message)
{
	// Load the dialog template ...
	g_return_val_if_fail(LoadFile(sdpAppendPath(Application.DialogPath(), "error.gtkml")), false);

	// Set our window caption and title
	RootWindow().SetTitle(Caption);
	Label("message").SetText(Message);
	
	// Center it ...
	gtk_window_set_position(RootWindow(), GTK_WIN_POS_CENTER);

	// Make it all visible ...
	RootWidget().Show();

	return true;
}

void CErrorBox::OnEvent(sdpGtkEvent* Event)
{
	// Sanity checks ...
	g_assert(Event);

	if(Event->Name() == "onok")
		RootWidget().Destroy();
	else if(Event->Name() == "ondestroy")
		CancelModal();
	else		
		sdpGtkObjectContainer::OnEvent(Event);
}

//////////////////////////////////////////////////////////////////////
// CQueryBox

CQueryBox::CQueryBox()
{
	m_Result = -1;
}

gint CQueryBox::DoModal(SDPCSTRING Caption, SDPCSTRING Message, SDPCSTRING Label1, SDPCSTRING Label2)
{
	g_return_val_if_fail(Create(Caption, Message, Label1, Label2), -1);
	sdpGtkObjectContainer::DoModal();
	
	return m_Result;
}

bool CQueryBox::Create(SDPCSTRING Caption, SDPCSTRING Message, SDPCSTRING Label1, SDPCSTRING Label2)
{
	// Load the dialog template ...
	g_return_val_if_fail(LoadFile(sdpAppendPath(Application.DialogPath(), "query.gtkml")), false);

	// Set our window caption and title ...
	RootWindow().SetTitle(Caption);
	Label("message").SetText(Message);
	
	// Set our button labels ...
	Label("label1").SetText(Label1);
	Label("label2").SetText(Label2);
	
	// Center it ...
	gtk_window_set_position(RootWindow(), GTK_WIN_POS_CENTER);

	// Make it all visible ...
	RootWidget().Show();

	return true;
}

void CQueryBox::OnEvent(sdpGtkEvent* Event)
{
	// Sanity checks ...
	g_assert(Event);

	if(Event->Name() == "onbutton1")
		{
			m_Result = 0;
			RootWidget().Destroy();
		}
	else if(Event->Name() == "onbutton2")
		{
			m_Result = 1;
			RootWidget().Destroy();
		}
	else if(Event->Name() == "ondestroy")
		CancelModal();
	else		
		sdpGtkObjectContainer::OnEvent(Event);
}



