#ifndef K3DIAPPLICATION_H
#define K3DIAPPLICATION_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIApplication interface, which encapsulates public functionality of the K-3D program
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3diobjectinfo.h>
#include <k3dishaderinfo.h>

// Forward class references
class k3dIDocument;
class sdpXmlIDocument;
class k3dIScriptEngine;

class k3dIApplication : public virtual sdpIUnknown
{
public:
	static const sdpIID iid;

	/// Closes K-3D
	virtual sdpResult Close() = 0;

	/// Opens a K-3D document
	virtual k3dIDocument* OpenDocument(SDPCSTRING FilePath) = 0;
	/// Creates a new K-3D document
	virtual k3dIDocument* CreateDocument() = 0;
	/// Returns the currently active K-3D document
	virtual k3dIDocument* GetActiveDocument() = 0;
	/// Closes an open K-3D document
	virtual sdpResult CloseDocument(k3dIDocument* Document) = 0;

	/// Returns a handle to the first object info instance
	virtual K3DOBJECTINFOHANDLE GetFirstObjectInfo() = 0;
	/// Returns a handle to the next object info instance
	virtual K3DOBJECTINFOHANDLE GetNextObjectInfo(K3DOBJECTINFOHANDLE Handle) = 0;
	/// Converts a handle into an object info instance
	virtual k3dIObjectInfo* GetObjectInfo(K3DOBJECTINFOHANDLE Handle) = 0;

	/// Returns a handle to the first shader info instance
	virtual K3DSHADERINFOHANDLE GetFirstShaderInfo() = 0;
	/// Returns a handle to the next shader info instance
	virtual K3DSHADERINFOHANDLE GetNextShaderInfo(K3DSHADERINFOHANDLE Handle) = 0;
	/// Converts a handle into a shader info instance
	virtual k3dIShaderInfo* GetShaderInfo(K3DSHADERINFOHANDLE Handle) = 0;

	/// Returns the absolute path to the K-3D base directory (contains plugins, dialog templates, help files, etc.)
	virtual SDPCSTRING BasePath() = 0;
	/// Returns the absolute path to the user's K-3D options directory
	virtual SDPCSTRING OptionsPath() = 0;
	/// Returns the absolute path to the K-3D shaders directory(s)
	virtual SDPCSTRING ShadersPath() = 0;
	/// Displays a URL in the user's preferred web browser
	virtual sdpResult BrowserNavigate(SDPCSTRING URL) = 0;
	/// Displays an informational message in a modal dialog box
	virtual sdpResult Message(SDPCSTRING Message, SDPCSTRING Title) = 0;
	/// Prompts the user with a modal OK/Cancel dialog box
	virtual sdpResult Query(SDPCSTRING Message, SDPCSTRING Title) = 0;
	/// Displays an error message in a modal dialog box
	virtual sdpResult Error(SDPCSTRING Message, SDPCSTRING Title) = 0;
	
	/// Returns the XML document that contains user options
	virtual sdpXmlIDocument* Options() = 0;

	/// Returns a scripting engine object that supports the requested language
	virtual k3dIScriptEngine* ScriptEngine(sdpCLSID Language) = 0;
};

#endif // K3DIAPPLICATION_H
