#ifndef K3DICAMERA_H
#define K3DICAMERA_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dICamera interface, which encapsulates the functionality of a display into the 3D workspace
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpcom.h>

// Forward class references
class k3dIRenderJob;
class k3dIRenderFrame;

class k3dICamera : public virtual sdpIUnknown
{
public:
	static const sdpIID iid;

	/// Makes the camera viewport visible
	virtual sdpResult ShowViewport() = 0;
	/// Closes the camera viewport
	virtual sdpResult HideViewport() = 0;
	/// Returns the width of the camera viewport (could be zero)
	virtual gulong Width() = 0;
	/// Returns the height of the camera viewport (could be zero)
	virtual gulong Height() = 0;
	/// Requests a redraw of the camera viewport
	virtual sdpResult Redraw(bool Synchronous) = 0;
	virtual sdpResult StartJob(k3dIRenderJob* Job) = 0;
	virtual sdpResult QueueFrame(k3dIRenderFrame* Frame) = 0;
	virtual sdpResult FinishJob(k3dIRenderJob* Job) = 0;
	virtual sdpResult SnapshotCameraCoordinates() = 0;
	virtual sdpResult SnapshotScreenCoordinates() = 0;
	virtual sdpResult GetWorldPosition(gdouble Coords[3]) = 0;
	virtual sdpResult GetWorldOrientation(gdouble Angles[3]) = 0;
	virtual sdpResult GetViewingFrustum(gdouble* Left, gdouble* Right, gdouble* Top, gdouble* Bottom, gdouble* Near, gdouble* Far) = 0;
	virtual sdpResult GetWorldToCameraMatrix(gdouble Matrix[16]) = 0;
};

#endif // K3DICAMERA_H
