#ifndef K3DICHANNEL_H
#define K3DICHANNEL_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIChannel interface encapsulating data that can change over time
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpcom.h>

/// Encapsulates data that can change over time
class k3dIChannel : public virtual sdpIUnknown
{
public:
	/// Stores the unique sdpCom Interface ID
	static const sdpIID iid;

	typedef enum
	{
		DATA_SCALAR = 100,
		DATA_TIME = 101,
		DATA_ANGLE = 102,
		DATA_DISTANCE = 103,
		DATA_AREA = 104,
		DATA_VOLUME = 105,
		DATA_MASS = 106,
		DATA_FORCE = 107,
		DATA_PRESSURE = 108,
		DATA_COLOR = 109,
		DATA_ROTATIONXYZ = 110,
		DATA_SCALEXYZ = 111,
	} DataType;

	/// Sets the type of data stored by this channel
	virtual sdpResult SetDatatype(const DataType Data) = 0;
	/// Returns the type of data stored by this channel
	virtual DataType GetDatatype() = 0;
	/// Returns the range of curve values stored by this channel
	virtual sdpResult GetCurveRange(gdouble* Min, gdouble* Max) = 0;
	/// Returns the one-dimensional value of the channel at the given time
	virtual gdouble GetValue1(const gdouble Time, const gdouble MaxError) = 0;
	/// Returns the two-dimensional value of the channel at the given time
	virtual sdpResult GetValue2(const gdouble Time, const gdouble MaxError, gdouble Value[2]) = 0;
	/// Returns the three-dimensional value of the channel at the given time
	virtual sdpResult GetValue3(const gdouble Time, const gdouble MaxError, gdouble Value[3]) = 0;
};

#endif // K3DICHANNEL_H
