#ifndef K3DIOBJECTCONTEXT_H
#define K3DIOBJECTCONTEXT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIObjectContext interface that encapsulates functionality common to all K-3D objects (and maintained by the server)
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dcom.h>

// Forward class declarations
class k3dIObject;
class k3dIObjectInfo;
class k3dIState;
class k3dILayer;

SDP_DECLARE_HANDLE(K3DOBJECTCONTEXTHANDLE);

/// Defines the functionality of an Object Context
class k3dIObjectContext : public virtual sdpIUnknown
{
public:
	/// Stores the interface ID
	static const sdpIID iid;

	/// Returns the owned-object's name
	virtual SDPCSTRING Name() = 0;
	/// Returns a k3dIObject interface to the owned-object
	virtual k3dIObject* Object() = 0;
	/// Returns the owned-object's ID
	virtual const k3dID ObjectID() = 0;
	/// Returns a k3dIObjectContext interface to the owned-object's parent object
	virtual k3dIObjectContext* Parent() = 0;
	/// Returns a k3dIObjectInfo interface describing the owned-object's sdpCom class
	virtual k3dIObjectInfo* ObjectInfo() = 0;
	/// Sets the owned-object's name
	virtual sdpResult SetName(SDPCSTRING NewName) = 0;
	/// Makes this object the active object
	virtual sdpResult SetActiveObject(bool State) = 0;
	/// Returns a k3dIState interface describing the current state of the owned-object
	virtual k3dIState* State() = 0;
	/// Returns a k3dILayer interface to the layer that owns the object
	virtual k3dILayer* Layer() = 0;
	/// Returns SDP_S_OK if the given object is an ancestor of this one
	virtual sdpResult QueryAncestor(k3dIObjectContext* ObjectContext) = 0;
	/// Returns SDP_S_OK if the given object is this one's direct parent
	virtual sdpResult QueryParent(k3dIObjectContext* ObjectContext) = 0;
	/// Returns SDP_S_OK if the given object is a direct child of this one
	virtual sdpResult QueryChild(k3dIObjectContext* ObjectContext) = 0;
	/// Returns SDP_S_OK if the given object is a descendant of this one
	virtual sdpResult QueryDescendant(k3dIObjectContext* ObjectContext) = 0;
	/// Sets the owned-object's parent object
	virtual sdpResult SetParent(k3dIObjectContext* ObjectContext) = 0;
	/// Sets the owned-object's sibling object
	virtual sdpResult SetSibling(k3dIObjectContext* SiblingContext) = 0;
};

#endif // K3DIOBJECTCONTEXT_H
