#ifndef K3DIOBJECTINFO_H
#define K3DIOBJECTINFO_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIObjectInfo interface, which encapsulates information about an entire class of K-3D objects
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpcom.h>
#include <vector>
#include <list>
#include <set>

// Forward class declarations ...
class k3dIObjectLibrary;

SDP_DECLARE_HANDLE(K3DOBJECTINFOHANDLE);
SDP_DECLARE_HANDLE(K3DBOOKMARKHANDLE);

class k3dIObjectInfo : public virtual sdpIUnknown
{
public:
	static const sdpIID iid;

	virtual k3dIObjectLibrary* Library() = 0;
	virtual const sdpCLSID& ClassID() = 0;
	virtual SDPCSTRING Name() = 0;
	virtual sdpResult QueryObjectInterface(const sdpIID& Interface) = 0;
	virtual K3DBOOKMARKHANDLE GetFirstBookmarkHandle() = 0;
	virtual K3DBOOKMARKHANDLE GetNextBookmarkHandle(K3DBOOKMARKHANDLE Handle) = 0;
	virtual SDPCSTRING GetBookmark(K3DBOOKMARKHANDLE Handle) = 0;
};

/// Contains an array of object infos
typedef std::vector<k3dIObjectInfo*> k3dObjectInfoArray;
/// Contains a linked-list of object infos
typedef std::list<k3dIObjectInfo*> k3dObjectInfoList;
/// Contains a set of object infos
typedef std::set<k3dIObjectInfo*> k3dObjectInfoSet;

#endif // K3DIOBJECTINFO_H
