#ifndef K3DIPOINT_H
#define K3DIPOINT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIPoint interface, which encapsulates a geometric point
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dcom.h>

#include <vector>
#include <list>
#include <set>

// Forward class declarations
class k3dIState;
class k3dILocation;
class k3dIMesh;
class k3dIPoint;

SDP_DECLARE_HANDLE(K3DPOINTHANDLE);

class k3dIPoint : public virtual sdpIUnknown
{
public:
	static const sdpIID iid;

	virtual k3dIState* State() = 0;
	virtual k3dILocation* Location() = 0;
	virtual k3dIMesh* Mesh() = 0;
	virtual const k3dID ID() = 0;
};

/// Contains an array of points
typedef std::vector<k3dIPoint*> k3dPointArray;
/// Contains a linked-list of points
typedef std::list<k3dIPoint*> k3dPointList;
/// Contains a set of points
typedef std::set<k3dIPoint*> k3dPointSet;

#endif // K3DIPOINT_H
