#ifndef K3DIPOLYPOINT_H
#define K3DIPOLYPOINT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIPolyPoint interface, which encapsulates the many-to-many relationship between k3dIPoint and k3dIPolygon / k3dINURBSCurve
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dcom.h>

#include <vector>
#include <list>
#include <set>

// Forward class declarations
class k3dIState;
class k3dILocation;
class k3dIMesh;
class k3dIPolygon;
class k3dIPoint;

SDP_DECLARE_HANDLE(K3DPOLYPOINTHANDLE);

class k3dIPolyPoint : public virtual sdpIUnknown
{
public:
	static const sdpIID iid;

	virtual k3dIPoint* Point() = 0;
	virtual k3dIState* State() = 0;
	virtual k3dILocation* Location() = 0;
	virtual const k3dID ID() = 0;
};

/// Contains an array of polypoints
typedef std::vector<k3dIPolyPoint*> k3dPolyPointArray;
/// Contains a linked-list of polypoints
typedef std::list<k3dIPolyPoint*> k3dPolyPointList;
/// Contains a set of polypoints
typedef std::set<k3dIPolyPoint*> k3dPolyPointSet;

#endif // K3DIPOLYPOINT_H
