#ifndef K3DIRIENGINE_H
#define K3DIRIENGINE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/////////////////////////////////////////////////////////////////////////////
//
// kiriengine.h - an sdpCom language binding to the RenderMan (R) interface
// 
// The RenderMan (R) Interface Procedures and RIB Protocol are:
// Copyright 1988, 1989, Pixar.  All rights reserved.
// RenderMan (R) is a registered trademark of Pixar.
//
// Note: this interface conforms to the RenderMan (R) RIB Protocol, with
// the exception of varargs functions, which are specifically prohibited
// by sdpCom.  Use the Ri...V() versions of the functions, instead.
//
/////////////////////////////////////////////////////////////////////////////

/** \file
		\brief Declares the k3dIRiEngine interface, which encapsulates an object capable of rendering 3D graphics as a RIB file for final rendering by a Pixar Renderman Interface compatible engine
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpcom.h>

// Forward class declarations ...
class k3dICamera;

/// Provides an sdpCom language binding to the RenderMan (R) Interface
class k3dIRiEngine : public virtual sdpIUnknown
{
public:
	static const sdpIID iid;

	virtual k3dICamera* Camera() = 0;
	virtual gulong RiAreaLightSourceV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiAtmosphereV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiAttributeBegin() = 0;
	virtual sdpResult RiAttributeEnd() = 0;
	virtual sdpResult RiAttributeV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiBasis(gdouble UBasis[16], gint UStep, gdouble VBasis[16], gint VStep) = 0;
	virtual sdpResult RiBound(gdouble Bound[6]) = 0;
	virtual sdpResult RiClipping(gdouble Hither, gdouble Yon) = 0;
	virtual sdpResult RiColor(gdouble Color[3]) = 0;
	virtual sdpResult RiColorSamples(gulong ParameterCount, gdouble nRGB[], gdouble RGBn[]) = 0;
	virtual sdpResult RiComment(SDPCSTRING Comment) = 0;
	virtual sdpResult RiConcatTransform(gdouble Transform[16]) = 0;
	virtual sdpResult RiConeV(gdouble Height, gdouble Radius, gdouble ThetaMax, 	gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiCoordSysTransform(SDPCSTRING Space) = 0;
	virtual sdpResult RiCoordinateSystem(SDPCSTRING Space) = 0;
	virtual sdpResult RiCropWindow(gdouble XMin, gdouble XMax, gdouble YMin, gdouble YMax) = 0;
	virtual sdpResult RiCurvesV(SDPCSTRING Degree, gulong ParameterCountcurves, gulong ParameterCountverts[], SDPCSTRING Wrap, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiCylinderV(gdouble Radius, gdouble ZMin, gdouble ZMax, gdouble ThetaMax,	gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiDeclare(SDPCSTRING Name, SDPCSTRING Type) = 0;
	virtual sdpResult RiDeformationV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiDepthOfField(gdouble FStop, gdouble FocalLength, gdouble FocalDistance) = 0;
	virtual sdpResult RiDetail(gdouble Bound[6]) = 0;
	virtual sdpResult RiDetailRange(gdouble MinVis, gdouble LowTran, gdouble UpTran, gdouble MaxVis) = 0;
	virtual sdpResult RiDiskV(gdouble Height, gdouble Radius, gdouble ThetaMax, 	gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiDisplacementV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiDisplayV(SDPCSTRING Name, SDPCSTRING Type, SDPCSTRING Mode, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiErrorAbort(gint Code, gint Severity, SDPCSTRING Message) = 0;
	virtual sdpResult RiErrorIgnore(gint Code, gint Severity, SDPCSTRING Message) = 0;
	virtual sdpResult RiErrorPrint(gint Code, gint Severity, SDPCSTRING Message) = 0;
	virtual sdpResult RiExposure(gdouble Gain, gdouble Gamma) = 0;
	virtual sdpResult RiExteriorV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiFormat(gint XResolution, gint YResolution, gdouble AspectRatio) = 0;
	virtual sdpResult RiFrameAspectRatio(gdouble AspectRatio) = 0;
	virtual sdpResult RiFrameBegin(gint FrameNumber) = 0;
	virtual sdpResult RiFrameEnd() = 0;
	virtual sdpResult RiGeneralPolygonV(gulong LoopCount, gulong VertexCounts[], gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiGeometricApproximation(SDPCSTRING Type, gdouble Value) = 0;
	virtual sdpResult RiGeometricRepresentation(SDPCSTRING Type) = 0;
	virtual sdpResult RiGeometryV(SDPCSTRING Type, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiHiderV(SDPCSTRING Type, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiHyperboloidV(gdouble Point1[3], gdouble Point2[3], gdouble ThetaMax, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiIdentity() = 0;
	virtual sdpResult RiIlluminate(gulong LightHandle, bool OnOff) = 0;
	virtual sdpResult RiImagerV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiInteriorV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual gulong RiLightSourceV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiMatte(bool OnOff) = 0;
	virtual sdpResult RiMotionBeginV(gulong ParameterCount, gdouble Times[]) = 0;
	virtual sdpResult RiMotionEnd() = 0;
	virtual sdpResult RiNewline() = 0;
	virtual sdpResult RiNuPatchV(gulong UCount, gint UOrder, gdouble UKnot[], gdouble UMin, gdouble UMax, gulong VCount, gint VOrder, gdouble VKnot[], gdouble VMin, gdouble VMax, 	gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual gulong RiObjectBegin() = 0;
	virtual sdpResult RiObjectEnd() = 0;
	virtual sdpResult RiObjectInstance(gulong Object) = 0;
	virtual sdpResult RiOpacity(gdouble Color[3]) = 0;
	virtual sdpResult RiOptionV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiOrientation(SDPCSTRING Orientation) = 0;
	virtual sdpResult RiParaboloidV(gdouble RMax, gdouble ZMin, gdouble ZMax, gdouble ThetaMax, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiPatchMeshV(SDPCSTRING Type, gulong UCount, SDPCSTRING UWrap, gulong VCount, SDPCSTRING VWrap, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiPatchV(SDPCSTRING Type, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiPerspective(gdouble FieldOfView) = 0;
	virtual sdpResult RiPixelFilter(SDPCSTRING FilterName, gdouble XWidth, gdouble YWidth) = 0;
	virtual sdpResult RiPixelSamples(gdouble XSamples, gdouble YSamples) = 0;
	virtual sdpResult RiPixelVariance(gdouble Variation) = 0;
	virtual sdpResult RiPointsGeneralPolygonsV(gulong PolygonCount, gulong LoopCounts[], gulong VertexCounts[], gulong Vertices[], gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiPointsPolygonsV(gulong PolygonCount, gulong VertexCounts[], gulong Vertices[], gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiPolygonV(gulong VertexCount, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiProjectionV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiQuantize(SDPCSTRING Type, gint One, gint QMin, gint QMax, gdouble Amplitude) = 0;
	virtual sdpResult RiReadArchive(SDPCSTRING Archive) = 0;
	virtual sdpResult RiRelativeDetail(gdouble RelativeDetail) = 0;
	virtual sdpResult RiReverseOrientation() = 0;
	virtual sdpResult RiRotate(gdouble angle, gdouble DX, gdouble DY, gdouble DZ) = 0;
	virtual sdpResult RiScale(gdouble DX, gdouble DY, gdouble DZ) = 0;
	virtual sdpResult RiScreenWindow(gdouble Left, gdouble Right, gdouble Bottom, gdouble Top) = 0;
	virtual sdpResult RiShadingInterpolation(SDPCSTRING Type) = 0;
	virtual sdpResult RiShadingRate(gdouble Size) = 0;
	virtual sdpResult RiShutter(gdouble SMin, gdouble SMax) = 0;
	virtual sdpResult RiSides(gint Sides) = 0;
	virtual sdpResult RiSkew(gdouble Angle, gdouble DX1, gdouble DY1, gdouble DZ1, gdouble DX2, gdouble DY2, gdouble DZ2) = 0;
	virtual sdpResult RiSolidBegin(SDPCSTRING Type) = 0;
	virtual sdpResult RiSolidEnd() = 0;
	virtual sdpResult RiSphereV(gdouble Radius, gdouble ZMin, gdouble ZMax, gdouble ThetaMax, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiStructure(SDPCSTRING Structure) = 0;
	virtual sdpResult RiSurfaceV(SDPCSTRING Name, gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiTextureCoordinates(gdouble S1, gdouble T1, gdouble S2, gdouble T2, gdouble S3, gdouble T3, gdouble S4, gdouble T4) = 0;
	virtual sdpResult RiTorusV(gdouble MajorRadius, gdouble MinorRadius, gdouble PhiMin,	gdouble PhiMax, gdouble ThetaMax,	gulong ParameterCount, SDPCSTRING Tokens[], gpointer Parameters[]) = 0;
	virtual sdpResult RiTransform(gdouble Transform[16]) = 0;
	virtual sdpResult RiTransformBegin() = 0;
	virtual sdpResult RiTransformEnd() = 0;
	virtual sdpResult RiTranslate(gdouble DX, gdouble DY, gdouble DZ) = 0;
	virtual sdpResult RiTrimCurve(gulong LoopCount, gulong CuveCounts[], gulong Order[], gdouble Knot[], gdouble AMin[], gdouble AMax[],	gulong ParameterCount, gdouble U[], gdouble V[], gdouble W[]) = 0;
	virtual sdpResult RiWorldBegin() = 0;
	virtual sdpResult RiWorldEnd() = 0;
	
	static SDPCSTRING const RI_A;
	static SDPCSTRING const RI_ABORT; 
	static SDPCSTRING const RI_AMBIENTLIGHT;
	static SDPCSTRING const RI_AMPLITUDE;
	static SDPCSTRING const RI_AZ;
	static SDPCSTRING const RI_BACKGROUND;
	static SDPCSTRING const RI_BEAMDISTRIBUTION;
	static SDPCSTRING const RI_BICUBIC;
	static SDPCSTRING const RI_BILINEAR; 
	static SDPCSTRING const RI_BLACK;
	static SDPCSTRING const RI_BOX;
	static SDPCSTRING const RI_BUMPY;
	static SDPCSTRING const RI_CAMERA;
	static SDPCSTRING const RI_CATMULL_ROM; 
	static SDPCSTRING const RI_CLAMP; 
	static SDPCSTRING const RI_COMMENT;
	static SDPCSTRING const RI_CONEANGLE;
	static SDPCSTRING const RI_CONEDELTAANGLE;
	static SDPCSTRING const RI_CONSTANT;
	static SDPCSTRING const RI_CONSTANTWIDTH;	
	static SDPCSTRING const RI_CS;
	static SDPCSTRING const RI_CUBIC; 
	static SDPCSTRING const RI_DEPTHCUE;
	static SDPCSTRING const RI_DIFFERENCE;
	static SDPCSTRING const RI_DISTANCE; 
	static SDPCSTRING const RI_DISTANTLIGHT;
	static SDPCSTRING const RI_FILE;
	static SDPCSTRING const RI_FLATNESS; 
	static SDPCSTRING const RI_FOG;
	static SDPCSTRING const RI_FOV;
	static SDPCSTRING const RI_FRAMEBUFFER;
	static SDPCSTRING const RI_FROM;
	static SDPCSTRING const RI_GAUSSIAN;
	static SDPCSTRING const RI_HANDLER;
	static SDPCSTRING const RI_HIDDEN;
	static SDPCSTRING const RI_IDENTIFIER;
	static SDPCSTRING const RI_IGNORE; 
	static SDPCSTRING const RI_INSIDE;
	static SDPCSTRING const RI_INTENSITY;
	static SDPCSTRING const RI_INTERSECTION;
	static SDPCSTRING const RI_KA; 
	static SDPCSTRING const RI_KD; 
	static SDPCSTRING const RI_KR;
	static SDPCSTRING const RI_KS; 
	static SDPCSTRING const RI_LH;
	static SDPCSTRING const RI_LIGHTCOLOR;
	static SDPCSTRING const RI_LINEAR;
	static SDPCSTRING const RI_MATTE;
	static SDPCSTRING const RI_MAXDISTANCE;
	static SDPCSTRING const RI_METAL; 
	static SDPCSTRING const RI_MINDISTANCE; 
	static SDPCSTRING const RI_N; 
	static SDPCSTRING const RI_NAME;
	static SDPCSTRING const RI_NONPERIODIC; 
	static SDPCSTRING const RI_NP; 
	static SDPCSTRING const RI_OBJECT;
	static SDPCSTRING const RI_ORIGIN;
	static SDPCSTRING const RI_ORTHOGRAPHIC;
	static SDPCSTRING const RI_OS;
	static SDPCSTRING const RI_OUTSIDE;
	static SDPCSTRING const RI_P;
	static SDPCSTRING const RI_PAINT;
	static SDPCSTRING const RI_PAINTEDPLASTIC;
	static SDPCSTRING const RI_PERIODIC; 
	static SDPCSTRING const RI_PERSPECTIVE;
	static SDPCSTRING const RI_PLASTIC;
	static SDPCSTRING const RI_POINTLIGHT; 
	static SDPCSTRING const RI_PRIMITIVE; 
	static SDPCSTRING const RI_PRINT; 
	static SDPCSTRING const RI_PW;
	static SDPCSTRING const RI_PZ; 
	static SDPCSTRING const RI_RASTER;
	static SDPCSTRING const RI_RGB;
	static SDPCSTRING const RI_RGBA;
	static SDPCSTRING const RI_RGBAZ;
	static SDPCSTRING const RI_RGBZ;
	static SDPCSTRING const RI_RH;
	static SDPCSTRING const RI_ROUGHNESS; 
	static SDPCSTRING const RI_S; 
	static SDPCSTRING const RI_SCREEN; 
	static SDPCSTRING const RI_SHINYMETAL;
	static SDPCSTRING const RI_SINC;
	static SDPCSTRING const RI_SMOOTH;
	static SDPCSTRING const RI_SPECULARCOLOR;
	static SDPCSTRING const RI_SPOTLIGHT;
	static SDPCSTRING const RI_ST;
	static SDPCSTRING const RI_STRUCTURE;
	static SDPCSTRING const RI_T; 
	static SDPCSTRING const RI_TEXTURENAME;
	static SDPCSTRING const RI_TRIANGLE;
	static SDPCSTRING const RI_TO;
	static SDPCSTRING const RI_UNION;
	static SDPCSTRING const RI_WIDTH; 
	static SDPCSTRING const RI_WORLD;
	static SDPCSTRING const RI_Z;
};

#endif // K3DIRIENGINE_H
