#ifndef K3DISPLINECHANNEL_H
#define K3DISPLINECHANNEL_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dISplineChannel interface, encapsulating a two-dimensional control spline
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpcom.h>
#include <k3displinechannelsegment.h>
#include <k3displinechannelnode.h>

/// Encapsulates a two-dimensional control spline
class k3dISplineChannel : public virtual sdpIUnknown
{
public:
	/// Stores the unique sdpCom Interface ID
	static const sdpIID iid;

	/// Sets the range of allowable values for one-dimensional data
	virtual sdpResult SetRange1(const gdouble Min, const gdouble Max) = 0;
	/// Sets the range of allowable values for two-dimensional data
	virtual sdpResult SetRange2(const gdouble Min[2], const gdouble Max[2]) = 0;
	/// Sets the range of allowable values for three-dimensional data
	virtual sdpResult SetRange3(const gdouble Min[3], const gdouble Max[3]) = 0;
	/// Returns the range of allowable values for one-dimensional data
	virtual sdpResult GetRange1(gdouble* Min, gdouble* Max) = 0;
	/// Returns the range of allowable values for two-dimensional data
	virtual sdpResult GetRange2(gdouble Min[2], gdouble Max[2]) = 0;
	/// Returns the range of allowable values for three-dimensional data
	virtual sdpResult GetRange3(gdouble Min[3], gdouble Max[3]) = 0;
	
	/// Returns the total number of segments in the channel
	virtual gulong GetSegmentCount() = 0;
	/// Returns an opaque handle to the first segment (if any)
	virtual K3DSPLINECHANNELSEGMENTHANDLE GetFirstSegment() = 0;
	/// Returns an opaque handle to the next segment (if any)
	virtual K3DSPLINECHANNELSEGMENTHANDLE GetNextSegment(const K3DSPLINECHANNELSEGMENTHANDLE Handle) = 0;
	/// Converts an opaque segment handle into a segment interface
	virtual k3dISplineChannelSegment* GetSegment(const K3DSPLINECHANNELSEGMENTHANDLE Handle) = 0;
	/// Creates a new segment and appends it to the end of the control spline
	virtual k3dISplineChannelSegment* CreateSegment() = 0;
	/// Destroys a segment
	virtual sdpResult DestroySegment(const k3dISplineChannelSegment* Segment) = 0;

	/// Returns the total number of nodes in the channel
	virtual gulong GetNodeCount() = 0;
	/// Returns an opaque handle to the first node (if any)
	virtual K3DSPLINECHANNELNODEHANDLE GetFirstNode() = 0;
	/// Returns an opaque handle to the next node (if any)
	virtual K3DSPLINECHANNELNODEHANDLE GetNextNode(const K3DSPLINECHANNELNODEHANDLE Handle) = 0;
	/// Converts an opaque node handle into a node interface
	virtual k3dISplineChannelNode* GetNode(const K3DSPLINECHANNELNODEHANDLE Handle) = 0;
};

#endif // K3DISPLINECHANNEL_H
