#ifndef K3DISPLINECHANNELNODE_H
#define K3DISPLINECHANNELNODE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIChannelNode interface, which encapsulates a single spline channel node
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpcom.h>

SDP_DECLARE_HANDLE(K3DSPLINECHANNELNODEHANDLE);

/// Encapsulates a single channel node
class k3dISplineChannelNode : public virtual sdpIUnknown
{
public:
	/// Stores the unique sdpCom Interface ID
	static const sdpIID iid;

	/// Returns whether this node is a "value node"
	virtual sdpResult ValueNode() = 0;
	/// Sets the position of this node
	virtual sdpResult SetPosition(gdouble Position[2]) = 0;
	/// Sets the value of one-dimensional data
	virtual sdpResult SetValue1(gdouble Value) = 0;
	/// Sets the value of two-dimensional data
	virtual sdpResult SetValue2(gdouble Value[2]) = 0;
	/// Sets the value of three-dimensional data
	virtual sdpResult SetValue3(gdouble Value[3]) = 0;
	/// Returns the position of this node
	virtual sdpResult GetPosition(gdouble Position[2]) = 0;
	/// Returns the one-dimensional node value
	virtual sdpResult GetValue1(gdouble* Value) = 0;
	/// Returns the two-dimensional node value
	virtual sdpResult GetValue2(gdouble Value[2]) = 0;
	/// Returns the three-dimensional node value
	virtual sdpResult GetValue3(gdouble Value[3]) = 0;
};

#endif // K3DISPLINECHANNELNODE_H
