#ifndef K3DITIMEBASE_H
#define K3DITIMEBASE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dITimeBase interface for managing a document's time-related information
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpcom.h>

/// Manages a document's time-related information
class k3dITimeBase : public virtual sdpIUnknown
{
public:
	static const sdpIID iid;

	/// Returns the animation start time as a percentage
	virtual gdouble GetStartTime() = 0;
	/// Returns the animation start time as a one-based integer frame number
	virtual gulong GetStartFrame() = 0;
	/// Returns the current view time as a percentage
	virtual gdouble GetViewTime() = 0;
	/// Returns the current view time as a one-based integer frame number
	virtual gulong GetViewFrame() = 0;
	/// Returns the animation end time as a percentage
	virtual gdouble GetEndTime() = 0;
	/// Returns the animation end time as a one-based integer frame number
	virtual gulong GetEndFrame() = 0;
	/// Returns the length of the animation in seconds
	virtual gdouble GetDuration() = 0;
	/// Returns the length of the animation in frames
	virtual gulong GetTotalFrames() = 0;
	/// Returns the animation frame rate in frames-per-second
	virtual gdouble GetFrameRate() = 0;
	/// Returns the length of an animation frame as a percentage
	virtual gdouble GetFrameDelta() = 0;
	/// Returns the current render time as a percentage
	virtual gdouble GetRenderTime() = 0;
	/// Returns the length of the current render as a percentage
	virtual gdouble GetRenderDeltaTime() = 0;

	/// Sets the animation start time as a percentage
	virtual sdpResult SetStartTime(gdouble Time) = 0;
	/// Sets the animation start time as a one-based integer frame number
	virtual sdpResult SetStartFrame(gulong Frame) = 0;
	/// Sets the current view time as a percentage
	virtual sdpResult SetViewTime(gdouble Time) = 0;
	/// Sets the current view time as a one-based integer frame number
	virtual sdpResult SetViewFrame(gulong Frame) = 0;
	/// Sets the animation end time as a percentage
	virtual sdpResult SetEndTime(gdouble Time) = 0;
	/// Sets the animation end time as a one-based integer frame number
	virtual sdpResult SetEndFrame(gulong Frame) = 0;
	/// Sets the length of the animation in seconds
	virtual sdpResult SetDuration(gdouble Time) = 0;
	/// Sets the lneght of the animation in frames
	virtual sdpResult SetTotalFrames(gulong Frames) = 0;
	/// Sets the animation frame rate in frames-per-second
	virtual sdpResult SetFrameRate(gdouble Time) = 0;
	/// Sets the current render time as a percentage
	virtual sdpResult SetRenderTime(gdouble Time) = 0;
	/// Sets the length of the current render as a percentage
	virtual sdpResult SetRenderDeltaTime(gdouble Delta) = 0;
};

#endif // K3DITIMEBASE_H
