#ifndef K3DITRANSFORM_H
#define K3DITRANSFORM_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dITransform interface, which defines operations for objects that maintain a homogeneous transformation frame
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <sdpcom.h>

/// Defines operations for objects that contain an internal coordinate reference frame
class k3dITransform : public virtual sdpIUnknown
{
public:
	/// Stores the sdpCom interface ID
	static const sdpIID iid;

	/// Enumerates supported coordinate references frames
	typedef enum
	{
		OLD_FRAME = 0,
		CURRENT_FRAME = 1,
		ANIMATED_FRAME = 2,
		OLD_CHILD_FRAME = 3,
		CURRENT_CHILD_FRAME = 4,
		ANIMATED_CHILD_FRAME = 5
	} ReferenceFrame;

	virtual sdpResult GetPosition(const ReferenceFrame Frame, gdouble Offsets[3]) = 0;
	virtual sdpResult GetOrientation(const ReferenceFrame Frame, gdouble AngleAxis[4]) = 0;
	virtual sdpResult GetScale(const ReferenceFrame Frame, gdouble Scale[3]) = 0;

	virtual sdpResult SetPosition(const ReferenceFrame Frame, gdouble Offsets[3]) = 0;
	virtual sdpResult SetOrientation(const ReferenceFrame Frame, gdouble AngleAxis[4]) = 0;
	virtual sdpResult SetScale(const ReferenceFrame Frame, gdouble Scale[3]) = 0;
};

#endif // K3DITRANSFORM_H
