#ifndef K3DFRAMERANGE_H
#define K3DFRAMERANGE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dFrameRange class for working with ranges of numbered files
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <sdpstring.h>

///////////////////////////////////////////////////////////////////////////////////////
// k3dFrameRange

/// A utility class for working with ranges of numbered files
class k3dFrameRange
{
public:
	k3dFrameRange();

	/// Initializes a range of frames based on a single filesystem sample
	bool Initialize(SDPCSTRING BaseFileName);
	/// Initializes a range of frames from a user-specified basefilename
	bool Initialize(SDPCSTRING BaseFileName, gulong FirstFrame, gulong LastFrame, gulong FrameDigits);
	/// Initialize a range of frames from scratch
	bool Initialize(SDPCSTRING PreDigits, SDPCSTRING PostDigits, SDPCSTRING Extension, gulong FirstFrame, gulong LastFrame, gulong FrameDigits);

	/// Generates a filename for the given frame number
	sdpString Frame(gulong FrameNumber);

	/// Returns the largest possible frame number, based on the number of digits
	gulong MaxFrame();

	/// Returns "true" if this instance has been properly initialized
	bool IsInitialized();

	/// Returns the text that precedes frame numbers
	SDPCSTRING PreDigits() { return m_PreDigits; }
	/// Returns the text that follows frame numbers
	SDPCSTRING PostDigits() { return m_PostDigits; }
	/// Returns the file extension that is added to the end of the filename
	SDPCSTRING Extension() { return m_Extension; }
	/// Returns the first frame number
	gulong FirstFrame() { return m_FirstFrame; }
	/// Returns the last frame number
	gulong LastFrame() { return m_LastFrame; }
	/// Returns the number of digits in the frame number
	gulong FrameDigits() { return m_FrameDigits; }

protected:
	/// Stores the text that precedes frame numbers
	sdpString m_PreDigits;
	/// Stores the text that follows frame numbers
	sdpString m_PostDigits;
	/// Stores the file extension that is appended to the end of the filename
	sdpString m_Extension;
	/// Stores the first frame number
	gulong m_FirstFrame;
	/// Stores the last frame number
	gulong m_LastFrame;
	/// Stores the number of digits in the frame number
	gulong m_FrameDigits;
};

#endif // K3DFRAMERANGE_H
