#ifndef K3DOBJECT_H
#define K3DOBJECT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dObject, a default implementation of the k3dIObject interface, for use as a base class for K-3D objects
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3diobject.h>
#include <k3diobjectcontext.h>
#include <k3dilayer.h>
#include <k3didocument.h>
#include <k3dactionclient.h>

#include <sdpstring.h>
#include <sdpmath.h>

////////////////////////////////////////////////////////////////////////////////////////////
// k3dObjectClient

/// Encapsulates a registered client's request to be notified of an event
class k3dObjectClient
{
public:
	k3dObjectClient() {};
	k3dObjectClient(k3dIEventClient* Client, gulong ClientData, SDPCSTRING Event)
	{
		m_Client = Client;
		m_ClientData = ClientData;
		m_Event = Event;
	}

	k3dIEventClient* m_Client;
	gulong m_ClientData;
	sdpString m_Event;
};

/// A default implementation of the k3dIObject interface, for use as a base class for K-3D objects
class k3dObject : public k3dIObject,
								public k3dActionClient
{
public:
	k3dObject();
	virtual ~k3dObject();

	/// Called when the object is initially created
	virtual bool Create();

	/// Returns the object's owning application
	k3dIApplication* Application() { return m_ObjectContext->Layer()->Document()->Application(); }
	/// Returns the object's owning document
	k3dIDocument* Document() { return m_ObjectContext->Layer()->Document(); }

	// k3dIObject overrides ...
	sdpResult SetObjectContext(k3dIObjectContext* Context);

	k3dIObjectContext* ObjectContext();
	sdpResult EditObject();

	sdpResult AddClient(k3dIEventClient* Client, gulong UserData, SDPCSTRING Event);
	sdpResult RemoveClient(k3dIEventClient* Client, gulong UserData, SDPCSTRING Event);
	sdpResult Notify(SDPCSTRING Event);
	sdpResult StartQueueEvents();
	sdpResult EndQueueEvents();

	sdpResult QuerySelectObject();
	sdpResult ObjectSelected(bool State);
	sdpResult QueryRenameObject();
	sdpResult ObjectRenamed(SDPCSTRING NewName);
	sdpResult QueryDeleteObject();
	sdpResult ObjectDeleted();

protected:
	/// Internal notification method that can't be overridden by the queueing mechanism
	void RawNotify(SDPCSTRING Event);

	/// Stores the sdpCom reference count for this object
	gulong m_ReferenceCount;

	/// Stores the Object Context for this object
	k3dIObjectContext* m_ObjectContext;

	typedef std::list<k3dObjectClient> Clients;
	typedef Clients::iterator ClientIterator;
	/// Stores clients requesting event notification
	Clients m_Clients;

	typedef std::list<sdpString> EventQueue;
	typedef EventQueue::iterator EventIterator;
	/// Stores queued events
	EventQueue m_Events;
	
	/// Keeps track of whether we're queueing events or not
	bool m_QueueEvents;
};

#endif // K3DOBJECT_H
