#ifndef K3DOBJECTSELECTOR_H
#define K3DOBJECTSELECTOR_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dObjectSelector class, which provides a user interface for either creating new, or selecting existing, K-3D objects
		\author Tim Shead (tshead@k-3d.com)
*/

// Forward class references ...
class k3dObjectVariableBase;

#include <sdpgtkobjectcontainer.h>
#include <k3didocument.h>
#include <k3diobject.h>

///////////////////////////////////////////////////////////////////////////
// k3dObjectSelector

/// Provides a user interface for selecting new & existing objects
class k3dObjectSelector : public sdpGtkObjectContainer
{
public:
	k3dObjectSelector();
	~k3dObjectSelector();

	/// Creates the user interface
	bool Create(k3dIDocument* Document, k3dObjectVariableBase* ObjectContainer, bool AllowNew, bool AllowExisting, bool AllowNone, sdpCLSID SelectionClass, sdpIID SelectionInterface);
	/// Creates the user interface and attaches it to a parent widget
	bool Create(k3dIDocument* Document, k3dObjectVariableBase* ObjectContainer, bool AllowNew, bool AllowExisting, bool AllowNone, sdpCLSID SelectionClass, sdpIID SelectionInterface, GtkContainer* Parent);
	/// Deletes the button and resets internal state for reuse
	void Release();

protected:
	/// Custom event-handling ...
	void OnEvent(sdpGtkEvent* Event);

	/// Called when the UI is about to go away
	void OnDestroy();
	/// Called when the user wants to create a new object
	void OnNew(sdpString EventName);
	/// Called when the user decides to select no object
	void OnNone();
	/// Called when the user wants to edit an existing object
	void OnEdit();
	/// Called when the user selects an existing object
	void OnSelect(sdpString EventName);

	void SetObject(k3dIObjectContext* Context);
			
	/// Stores the owning document
	k3dIDocument* m_Document;
	/// Stores the underlying object container
	k3dObjectVariableBase* m_ObjectContainer;
	/// Stores whether the user is allowed to create a new object
	bool m_AllowNew;
	/// Stores whether the user is allowed to select an existing object
	bool m_AllowExisting;
	/// Stores whether the user is allowed to choose no object
	bool m_AllowNone;
	/// Stores the class of objects the user can create/select
	sdpCLSID m_SelectionClass;
	/// Stores the interface types of objects the user can create/select
	sdpIID m_SelectionInterface;
};

#endif // K3DOBJECTSELECTOR_H
