#ifndef K3DPOINT_H
#define K3DPOINT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dPoint class, which implements a geometric point
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dipoint.h>
#include <k3dilocation.h>
#include <k3dstate.h>
#include <k3dactionclient.h>

#include <sdpmath.h>

// Forward class declarations
class k3dMesh;

class k3dPoint : public k3dIPoint,
							public k3dILocation,
							public k3dState,
							public k3dActionClient
{
public:
	k3dPoint(k3dMesh* Mesh, k3dID ID);
	virtual ~k3dPoint();

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface) { return SDP_E_NO_INTERFACE; }
	gulong AddRef() { return 0; }
	gulong Release() { return 0; }

	// k3dIPoint overrides
	k3dIState* State();
	k3dILocation* Location();
	k3dIMesh* Mesh();
	const k3dID ID();

	// k3dILocation overrides
//	k3dIState* State();	Already declared above !
	sdpResult GetOldXYZ(double Coords[3]);
	sdpResult GetLocalXYZ(double Coords[3]);
	sdpResult GetAnimXYZ(double Coords[3]);
	sdpResult GetWorldXYZ(double Coords[3]);
	sdpResult GetCameraXYZ(double Coords[3]);
	sdpResult GetScreenXYZ(double Coords[3]);
	sdpResult GetNormalXYZ(double Coords[3]);
	sdpResult GetTextureXYZ(double Coords[3]);
	sdpResult GetImplicitTextureXYZ(double Coords[3]);
	sdpResult SetOldXYZ(double Coords[3]);
	sdpResult SetLocalXYZ(double Coords[3]);
	sdpResult SetAnimXYZ(double Coords[3]);
	sdpResult SetWorldXYZ(double Coords[3]);
	sdpResult SetCameraXYZ(double Coords[3]);
	sdpResult SetScreenXYZ(double Coords[3]);
	sdpResult SetNormalXYZ(double Coords[3]);
	sdpResult SetTextureXYZ(double Coords[3]);
	sdpResult SetImplicitTextureXYZ(double Coords[3]);

	// k3dIState overrides
	sdpResult SetState(gulong Mask, bool State);

	// k3dIActionClient overrides ...
	sdpResult HandleStep(gulong ValueCode, gulong Value, gulong ValueSize);

	// Helper functions
	sdpResult Save(sdpXmlIElement* Element, k3dIDependencies* Dependencies);
	sdpResult Load(sdpXmlIElement* Element);

	/// Stores the parent mesh
	k3dMesh* m_Mesh;
	/// Stores a unique (within the parent mesh) ID
	k3dID m_ID;

	/// Stores old local coordinates for undos / interactive tools
	sdpVector3 m_OldXYZ;
	/// Stores current local coordinates
	sdpVector3 m_LocalXYZ;
	/// Stores animated local coordinates
	sdpVector3 m_AnimXYZ;
	/// Stores current world coordinates (calculated only on-demand)
	sdpVector3 m_WorldXYZ;
	/// Stores current camera coordinates (calculated only on-demand)
	sdpVector3 m_CameraXYZ;
	/// Stores current screen coordinates (calculated only on-demand)
	sdpVector3 m_ScreenXYZ;
	
	/// Enumerates undo-able steps
	typedef enum
	{
		STEP_STATE,
		STEP_POSITION,
	} StepType;
};

#endif K3DPOINT_H
