#ifndef K3DPRIMTOOL_H
#define K3DPRIMTOOL_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dPrimTool class, which encapsulates a K-3D tool capable of generating new geometry
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dtool.h>
#include <k3dipoint.h>
#include <k3dipolygon.h>
#include <k3dipolypoint.h>
#include <k3dimesh.h>

#include <sdpgtkentry.h>

class k3dPrimTool : public k3dTool
{
public:
	k3dPrimTool();
	virtual ~k3dPrimTool();

	// Helper functions ...
	virtual void DeleteAll();			// Deletes all new geometry
	virtual void CreateAll();			// Makes new geometry permanent

	// Statistics ...
	gulong PointCount();
	gulong PolyCount();
	gulong MeshCount();

	// User Interface ...
	void DisplayStatistics(sdpGtkEntry Entry);

//	void GetNewGeometryDimensions(sdpVector3& Min, sdpVector3& Max);
//	void CenterNewGeometry();

	// Points ...
	void ReservePoints(gulong Columns);
	void ReservePoints(gulong Rows, gulong Columns);

	k3dIPoint* CreatePoint(k3dIMesh* Mesh);
	k3dIPoint* CreatePoint(k3dIMesh* Mesh, gulong Column);
	k3dIPoint* CreatePoint(k3dIMesh* Mesh, gulong Row, gulong Column);

	// Polygons ...
	void ReservePolygons(gulong Columns);
	void ReservePolygons(gulong Rows, gulong Columns);

	k3dIPolygon* CreatePolygon(k3dIMesh* Mesh);
	k3dIPolygon* CreatePolygon(k3dIMesh* Mesh, gulong Column);
	k3dIPolygon* CreatePolygon(k3dIMesh* Mesh, gulong Rows, gulong Column);

	// Meshes ...
	k3dIMesh* CreateMesh(SDPCSTRING Name);
	k3dIMesh* CreateMesh(SDPCSTRING Name, gulong PointColumns, gulong PolygonColumns);
	k3dIMesh* CreateMesh(SDPCSTRING Name, gulong PointRows, gulong PointColumns, gulong PolygonRows, gulong PolygonColumns);
	k3dIMesh* CreateMesh(SDPCSTRING Name, gulong Rows, gulong Columns, bool StitchTop, bool StitchSide);

	gulong MeshColumn(gulong Column, gulong Columns, bool StitchSide);
	gulong MeshRow(gulong Row, gulong Rows, bool StitchTop);

	// Used to signal that geometric topology has changed
	bool m_CreateGeometry;

	// Arrays used to keep track of what we create ...
	typedef std::vector<k3dIMesh*> Meshes;
	typedef Meshes::iterator MeshIterator;
	Meshes m_Meshes;

	typedef std::vector<k3dIObject*> Objects;
	typedef Objects::iterator ObjectIterator;
	Objects m_Objects;

	typedef std::vector<k3dIPoint*> PointRow;
	typedef PointRow::iterator PointRowIterator;
	typedef std::vector<PointRow> Points;
	typedef Points::iterator PointIterator;
	Points m_Points;

	typedef std::vector<k3dIPolygon*> PolygonRow;
	typedef PolygonRow::iterator PolygonRowIterator;
	typedef std::vector<PolygonRow> Polygons;
	typedef Polygons::iterator PolygonIterator;
	Polygons m_Polygons;
	
	// Statics used to color new geometry
	static sdpVector3 m_NewColor;
	static sdpVector3 m_NormalColor;
};

#endif // K3DPRIMTOOL_H
