#ifndef K3DSHADERSELECTOR_H
#define K3DSHADERSELECTOR_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dShaderSelector class, which provides a user interface for choosing Pixar Renderman shaders
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpgtkobjectcontainer.h>
#include <k3diapplication.h>
#include <k3dshaderinstance.h>
#include <k3dshaderargumentspage.h>

///////////////////////////////////////////////////////////////////////////
// k3dShaderSelector

/// Provides a user interface for selecting new & existing objects
class k3dShaderSelector : public sdpGtkObjectContainer
{
public:
	k3dShaderSelector();
	~k3dShaderSelector();

	/// Creates the button
	bool Create(k3dIApplication* Application, k3dShaderInstance* ShaderInstance, k3dShaderArgumentsPage* ShaderPage, bool AllowNone, gulong ShaderTypes);
	/// Creates the button and attaches it to a parent widget
	bool Create(k3dIApplication* Application, k3dShaderInstance* ShaderInstance, k3dShaderArgumentsPage* ShaderPage, bool AllowNone, gulong ShaderTypes, GtkContainer* Parent);
	/// Deletes the button and resets internal state for reuse
	void Release();

protected:
	/// Custom event-handling ...
	void OnEvent(sdpGtkEvent* Event);

	/// Called when the user decides to select no shader
	void OnNone();
	/// Called when the user selects a shader
	void OnSelect(sdpString EventName);
	/// Called to do the actual work of selecting a shader
	void SetShader(const K3DSHADERINFOHANDLE ShaderHandle);
	/// Called when the GTK+ widgets are about to go away
	void OnDestroy();
		
	/// Stores the owning application
	k3dIApplication* m_Application;
	/// Stores the shader instance that will be storing the shader we choose
	k3dShaderInstance* m_ShaderInstance;
	/// Stores an (optional) shader page that should be notified if the shader changes
	k3dShaderArgumentsPage* m_ShaderArgumentsPage;
	/// Stores whether the user is allowed to choose no shader
	bool m_AllowNone;
	/// Stores the types of shaders the user is allowed to select
	gulong m_ShaderTypes;
};

#endif // K3DSHADERSELECTOR_H
