#ifndef K3DSHADINGNODE_H
#define K3DSHADINGNODE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
///
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dShadingNode class, which caches information used to simulate smooth-shading of adjacent polygons
		\author Tim Shead (tshead@k-3d.com)
*/

#include <list>
#include <sdptypes.h>

// Forward class references ...
class k3dMesh;
class k3dPolyPoint;
class k3dIDependencies;
class sdpXmlIElement;

/////////////////////////////////////////////////////////////////////
// k3dShadingNode

class k3dShadingNode
{
public:
	k3dShadingNode();
	~k3dShadingNode();

	gulong PolyPointCount() { return m_PolyPoints.size(); }

	void RemovePolygons(gulong State);

	sdpResult Save(sdpXmlIElement* Element, k3dIDependencies* Dependencies);
	sdpResult Load(sdpXmlIElement* Element, k3dMesh* Mesh);

	typedef std::list<k3dPolyPoint*> PolyPoints;
	typedef PolyPoints::iterator PolyPointIterator;
	/// Stores the polypoints that should be smooth-shaded
	PolyPoints m_PolyPoints;
};

#endif K3DSHADINGNODE_H
