
// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dDocument class, which encapsulates a K3DML (XML) document
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3ddocument.h>
#include <sdpfile.h>
#include <fstream>

bool k3dDocument::LoadOptions(SDPCSTRING Override)
{
	// Load the document ...
	return LoadFile(OptionsFilePath(Override));
}

bool k3dDocument::LoadFile(SDPCSTRING FilePath)
{
	// Sanity checks ...
	g_return_val_if_string_fail(FilePath, false);
	
	// Load the document ...
	if(!sdpXmlDocument::LoadFile(FilePath))
		return false;
	
	if(!IsK3DMLDocument())
		return false;

	return true;
}

bool k3dDocument::Load(std::istream& Stream)
{
	// Load the document ...
	if(!sdpXmlDocument::Load(Stream, ""))
		return false;
		
	if(!IsK3DMLDocument())
		return false;

	return true;
}

bool k3dDocument::SaveOptions(SDPCSTRING Override)
{
	// Make sure the options directory exists ...
	if(!sdpFileExists(OptionsPath(Override)))
		g_return_val_if_fail(sdpCreateDirectory(OptionsPath(Override)), false);

	// Save the document ...
	return SaveFile(OptionsFilePath(Override));
}

bool k3dDocument::IsK3DMLDocument()
{
	// Make sure it isn't empty ...
	sdpXmlIElement* root = GetRootElement();
	if(!root)
		return false;

	// And make sure it's really a K-3D Markup Language document ...
	if(root->Name() != sdpString("k3dml"))
		return false;
	
	return true;
}

sdpString k3dDocument::OptionsPath(SDPCSTRING Override)
{
	// If an override is provided, we're done ...
	if(Override)
		if(strlen(Override))
			return sdpString(Override);

	// Otherwise, check for an environment variable ...
	sdpString result = sdpGetEnv("K3D_OPTIONS_PATH");
	if(result.size())
		return result;

	// Otherwise, go with ~/.k3d ...
	result = sdpGetHomeDirectory();
	return sdpAppendPath(result, ".k3d");
}

sdpString k3dDocument::OptionsFilePath(SDPCSTRING Override)
{
	sdpString result = OptionsPath(Override);
	return sdpAppendPath(result, "options.k3d");
}

