// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dTransform class, which provides a pluggable implementation of k3dITransform
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dtransform.h>
#include <sdpxml.h>
#include <sdpgl.h>

#include <stack>


k3dTransform::k3dTransform()
{
	for(gulong i = 0; i < sizeof(m_Scale) / sizeof(sdpVector3); i++)
		m_Scale[i] = 1.0;
}

////////////////////////////////////////////////////////////////
// k3dITransform overrides

sdpResult k3dTransform::GetPosition(const ReferenceFrame Frame, gdouble Offsets[3])
{
	m_Position[Frame].CopyArray(Offsets);
	return SDP_S_OK;
}

sdpResult k3dTransform::GetOrientation(const ReferenceFrame Frame, gdouble AngleAxis[4])
{
	m_Orientation[Frame].CopyArray(AngleAxis);
	return SDP_S_OK;
}

sdpResult k3dTransform::GetScale(const ReferenceFrame Frame, gdouble Scale[3])
{
	m_Scale[Frame].CopyArray(Scale);
	return SDP_S_OK;
}

sdpResult k3dTransform::SetPosition(const ReferenceFrame Frame, gdouble Offsets[3])
{
	m_Position[Frame] = Offsets;
	return SDP_S_OK;
}

sdpResult k3dTransform::SetOrientation(const ReferenceFrame Frame, gdouble AngleAxis[4])
{
	m_Orientation[Frame] = AngleAxis;
	return SDP_S_OK;
}

sdpResult k3dTransform::SetScale(const ReferenceFrame Frame, gdouble Scale[3])
{
	m_Scale[Frame] = Scale;
	return SDP_S_OK;
}

////////////////////////////////////////////////////////////////////////////
// Helper functions ...

sdpResult k3dTransform::Save(k3dObject* Object, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(Object);
	g_assert(Element);

	sdpXmlCreateElement(Element, "transformation", 0, "frame", "self", "position", m_Position[k3dITransform::CURRENT_FRAME], "orientation", m_Orientation[k3dITransform::CURRENT_FRAME], "scale", m_Scale[k3dITransform::CURRENT_FRAME]);
	sdpXmlCreateElement(Element, "transformation", 0, "frame", "child", "position", m_Position[k3dITransform::CURRENT_CHILD_FRAME], "orientation", m_Orientation[k3dITransform::CURRENT_CHILD_FRAME], "scale", m_Scale[k3dITransform::CURRENT_CHILD_FRAME]);

	return SDP_S_OK;
}

sdpResult k3dTransform::Load(k3dObject* Object, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(Object);
	g_assert(Element);

	sdpXmlIElement* selftransform = sdpXmlFindElement(Element, "transformation", "frame", "self");
	if(selftransform)
		{
			sdpXmlGetAttribute(selftransform, "position", m_Position[k3dITransform::CURRENT_FRAME]);
			sdpXmlGetAttribute(selftransform, "orientation", m_Orientation[k3dITransform::CURRENT_FRAME]);
			sdpXmlGetAttribute(selftransform, "scale", m_Scale[k3dITransform::CURRENT_FRAME]);
		}

	sdpXmlIElement* childtransform = sdpXmlFindElement(Element, "transformation", "frame", "self");
	if(childtransform)
		{
			sdpXmlGetAttribute(childtransform, "position", m_Position[k3dITransform::CURRENT_CHILD_FRAME]);
			sdpXmlGetAttribute(childtransform, "orientation", m_Orientation[k3dITransform::CURRENT_CHILD_FRAME]);
			sdpXmlGetAttribute(childtransform, "scale", m_Scale[k3dITransform::CURRENT_CHILD_FRAME]);
		}

	return SDP_S_OK;
}

