// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dUserObject class, which serves as a base class for K-3D objects that are intended to respond to user events
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3duserobject.h>
#include <k3dmisc.h>
#include <k3dvariable.h>

#include <sdpfile.h>

#include <algorithm>

#ifdef	SDPWIN32
#include <iostream.h>
#endif

////////////////////////////////////////////////////////////////////////////
// class k3dUserObject

k3dUserObject::k3dUserObject()
{
}

k3dUserObject::~k3dUserObject()
{
}

// Initialization that could fail
bool k3dUserObject::Create()
{
	g_return_val_if_fail(k3dObject::Create(), false);

	return true;
}

///////////////////////////////////////////////////////////////////////////
// k3dIObject overrides

/////////////////////////////////////////////////////////////////////////////
// Simplified channel / selector creation

sdpResult k3dUserObject::DetachObjects()
{
g_assert_not_reached();
return SDP_E_FAIL;
/*
	for(gulong i = 0; i < m_Objects.size(); i++)
		{
			if(!m_Objects[i]->Interface()) continue;

			if(m_Objects[i]->InterfaceID() == IID_k3dIChannel) ((k3dIChannel*)m_Objects[i]->Interface())->Detach();
			else if(m_Objects[i]->InterfaceID() == IID_k3dIObjectSelector) ((k3dIObjectSelector*)m_Objects[i]->Interface())->Detach();
			else if(m_Objects[i]->InterfaceID() == IID_k3dIShaderSelector) ((k3dIShaderSelector*)m_Objects[i]->Interface())->Detach();
		}

	return SDP_S_OK;
*/
}

class k3dUserObjectAssertUniqueName
{
public:
	k3dUserObjectAssertUniqueName(SDPCSTRING Name)
	{
		m_Name = Name;
	}
	
	void operator()(k3dVariableBase* Variable)
	{
		if(m_Name == Variable->Name())
			#ifdef	SDPWIN32
			cerr << "Duplicate name \"" << m_Name << "\" in object!!!" << endl;
			#else
			std::cerr << "Duplicate name \"" << m_Name << "\" in object!!!" << std::endl;
			#endif
	}
	
protected:
	sdpString m_Name;
};

// Add a variable to the list of variables ...
void k3dUserObject::AddVariable(k3dVariableBase* Variable)
{
	// Sanity checks ...
	g_assert(Variable);

	// Make sure the variable name is unique ...
	std::for_each(m_Variables.begin(), m_Variables.end(), k3dUserObjectAssertUniqueName(Variable->Name()));

	// Add this variable to our list ...
	m_Variables.push_back(Variable);
}

// Add an object to the object list ...
void k3dUserObject::AddObject(k3dObjectVariableBase* Object)
{
	// Sanity checks ...
	g_assert(Object);

	// Make sure the variable name is unique ...
	std::for_each(m_Objects.begin(), m_Objects.end(), k3dUserObjectAssertUniqueName(Object->Name()));

	// Add this variable to our list ...
	m_Objects.push_back(Object);
}

void k3dUserObject::OnSet()
{
	OnModify(true);
}

void k3dUserObject::OnModify(bool SaveAction)
{
	k3dRedrawAll(Document(), false);
}


