#ifndef BOOLEANOPERATOR_H
#define BOOLEANOPERATOR_H


// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CBooleanOperator K-3D object, which allows geometry to be rendered with boolean effects
		\author Tim Shead (tshead@k-3d.com)
*/

// Forward class declarations ...
class CBooleanOperator;

#include <k3ddialog.h>
#include <k3dpersistobject.h>
#include <k3drirender.h>
#include <k3dvariable.h>

/////////////////////////////////////////////////////////////////////////////
// CBooleanOperatorDialog

class CBooleanOperatorDialog : public k3dDialog
{
public:
	CBooleanOperatorDialog();

	bool Create(CBooleanOperator* Object);

protected:
	CBooleanOperator* m_Object;

	// Event handling ...
	void OnEvent(sdpGtkEvent* Event);

	void OnIntersection();
	void OnUnion();
	void OnDifference();

	// Update controls ...
	void UpdateControls();
};

/////////////////////////////////////////////////////
// CBooleanOperator

class CBooleanOperator :	public k3dPersistObject,
											public k3dRiRender
{
friend class CBooleanOperatorDialog;
public:
	CBooleanOperator();
	~CBooleanOperator();

	CBooleanOperatorDialog m_UserInterface;

	typedef enum
	{
		BOOLEAN_INTERSECTION,
		BOOLEAN_UNION,
		BOOLEAN_DIFFERENCE
	} BooleanType;

	class VariableBooleanType : public k3dCustomVariable<BooleanType>
	{
	protected:
		sdpString ConvertToString() { return sdpFormatString("%d", m_Value); }
		bool ConvertFromString(SDPCSTRING String) { m_Value = BooleanType(atoi(String)); return true; }
	};

	VariableBooleanType m_Type;

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIObject overrides
	sdpResult EditObject();
	sdpResult ObjectRenamed(SDPCSTRING NewName);
	sdpResult ObjectDeleted();

	// k3dIRiRender overrides
	sdpResult RiPushRender(k3dIRenderFrame* Frame, k3dIRiEngine* Engine);
	sdpResult RiPopRender(k3dIRenderFrame* Frame, k3dIRiEngine* Engine);
};

#endif // BOOLEANOPERATOR_H
