// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the CParticleBurst K-3D object, which creates a parametric particle system
		\author Tim Shead (tshead@k-3d.com)
*/

#include "particleburst.h"

#include <k3didocument.h>
#include <k3dichannel.h>
#include <k3ditexture.h>
#include <k3dmisc.h>
#include <sdpfile.h>

#include <sdpgl.h>

////////////////////////////////////////////////////////////////////////////
// CParticle

CParticle::CParticle()
{
}

k3dIState* CParticle::State()
{
	return this;
}

sdpResult CParticle::GetOldXYZ(double Coords[3])
{
	return SDP_S_FALSE;
}

sdpResult CParticle::GetLocalXYZ(double Coords[3])
{
	return SDP_S_FALSE;
}

sdpResult CParticle::GetAnimXYZ(double Coords[3])
{
	m_AnimXYZ.CopyArray(Coords);
	return SDP_S_OK;
}

sdpResult CParticle::GetWorldXYZ(double Coords[3])
{
	m_WorldXYZ.CopyArray(Coords);
	return SDP_S_OK;
}

sdpResult CParticle::GetCameraXYZ(double Coords[3])
{
	m_CameraXYZ.CopyArray(Coords);
	return SDP_S_OK;
}

sdpResult CParticle::GetScreenXYZ(double Coords[3])
{
	m_ScreenXYZ.CopyArray(Coords);
	return SDP_S_OK;
}

sdpResult CParticle::GetNormalXYZ(double Coords[3])
{
	return SDP_S_FALSE;
}

sdpResult CParticle::GetTextureXYZ(double Coords[3])
{
	return SDP_S_FALSE;
}

sdpResult CParticle::GetImplicitTextureXYZ(double Coords[3])
{
	return SDP_S_FALSE;
}

sdpResult CParticle::SetOldXYZ(double Coords[3])
{
	return SDP_S_FALSE;
}

sdpResult CParticle::SetLocalXYZ(double Coords[3])
{
	return SDP_S_FALSE;
}

sdpResult CParticle::SetAnimXYZ(double Coords[3])
{
	m_AnimXYZ = Coords;
	return SDP_S_OK;
}

sdpResult CParticle::SetWorldXYZ(double Coords[3])
{
	m_WorldXYZ = Coords;
	return SDP_S_OK;
}

sdpResult CParticle::SetCameraXYZ(double Coords[3])
{
	m_CameraXYZ = Coords;
	return SDP_S_OK;
}

sdpResult CParticle::SetScreenXYZ(double Coords[3])
{
	m_ScreenXYZ = Coords;
	return SDP_S_OK;
}

sdpResult CParticle::SetNormalXYZ(double Coords[3])
{
	return SDP_S_FALSE;
}

sdpResult CParticle::SetTextureXYZ(double Coords[3])
{
	return SDP_S_FALSE;
}

sdpResult CParticle::SetImplicitTextureXYZ(double Coords[3])
{
	return SDP_S_FALSE;
}

////////////////////////////////////////////////////////////////////////////
// CParticleBurst

CParticleBurst::CParticleBurst() : k3dTransformObject()
{
	AddVariable(&m_ParticleType("particletype", POINTS));
	AddVariable(&m_ColorType("colortype", LIFETIME));
	AddVariable(&m_EmitterPositionSamples("emitterpositionsamples", 1.0));
	AddVariable(&m_BirthRateSamples("birthratesamples", 1.0));
	AddVariable(&m_RandomSeed("randomseed", 12345));
	AddVariable(&m_DirectionRandom("directionrandom", true));
	AddVariable(&m_DirectionEmitter("directionemitter", true));
	AddVariable(&m_TwistRandom("twistrandom", true));
	AddVariable(&m_TwistRelative("twistrelative", false));
	AddVariable(&m_TwistLifetime("twistlifetime", true));
	AddVariable(&m_WidthRandom("widthrandom", true));
	AddVariable(&m_WidthRelative("widthrelative", false));
	AddVariable(&m_WidthLifetime("widthlifetime", true));
	AddVariable(&m_HeightRandom("heightrandom", true));
	AddVariable(&m_HeightRelative("heightrelative", false));
	AddVariable(&m_HeightLifetime("heightlifetime", true));
	AddVariable(&m_AngleAlign("anglealign", false));
	AddVariable(&m_AngleRandom("anglerandom", true));
	AddVariable(&m_AngleRelative("anglerelative", false));
	AddVariable(&m_AngleLifetime("anglelifetime", true));
	AddVariable(&m_DistanceRandom("distancerandom", true));
	AddVariable(&m_DistanceRelative("distancerelative", false));
	AddVariable(&m_DistanceLifetime("distancelifetime", true));
	AddVariable(&m_DriftRandom("driftrandom", true));
	AddVariable(&m_DriftRelative("driftrelative", false));
	AddVariable(&m_DriftLifetime("driftlifetime", true));
	AddVariable(&m_StrengthRandom("strengthrandom", true));
	AddVariable(&m_StrengthRelative("strengthrelative", false));
	AddVariable(&m_StrengthLifetime("strengthlifetime", true));
	AddVariable(&m_SizeRandom("sizerandom", true));
	AddVariable(&m_SizeRelative("sizerelative", false));
	AddVariable(&m_SizeLifetime("sizelifetime", true));

	AddObject(&m_Globaltime("globaltime"));
	AddObject(&m_Birthrate("birthrate"));
	AddObject(&m_Minlife("minlife"));
	AddObject(&m_Maxlife("maxlife"));
	AddObject(&m_Globalstrength("globalstrength"));
	AddObject(&m_Minstrength("minstrength"));
	AddObject(&m_Minstrengthend("minstrengthend"));
	AddObject(&m_Maxstrength("maxstrength"));
	AddObject(&m_Maxstrengthend("maxstrengthend"));
	AddObject(&m_Strengthlife("strengthlife"));
	AddObject(&m_Minsize("minsize"));
	AddObject(&m_Minsizeend("minsizeend"));
	AddObject(&m_Maxsize("maxsize"));
	AddObject(&m_Maxsizeend("maxsizeend"));
	AddObject(&m_Sizelife("sizelife"));
	AddObject(&m_Mindistance("mindistance"));
	AddObject(&m_Mindistanceend("mindistanceend"));
	AddObject(&m_Maxdistance("maxdistance"));
	AddObject(&m_Maxdistanceend("maxdistanceend"));
	AddObject(&m_Distancelife("distancelife"));
	AddObject(&m_Mindriftx("mindriftx"));
	AddObject(&m_Mindriftxend("mindriftxend"));
	AddObject(&m_Maxdriftx("maxdriftx"));
	AddObject(&m_Maxdriftxend("maxdriftxend"));
	AddObject(&m_Driftxlife("driftxlife"));
	AddObject(&m_Mindrifty("mindrifty"));
	AddObject(&m_Mindriftyend("mindriftyend"));
	AddObject(&m_Maxdrifty("maxdrifty"));
	AddObject(&m_Maxdriftyend("maxdriftyend"));
	AddObject(&m_Driftylife("driftylife"));
	AddObject(&m_Mindriftz("mindriftz"));
	AddObject(&m_Mindriftzend("mindriftzend"));
	AddObject(&m_Maxdriftz("maxdriftz"));
	AddObject(&m_Maxdriftzend("maxdriftzend"));
	AddObject(&m_Driftzlife("driftzlife"));
	AddObject(&m_Mindirectionx("mindirectionx"));
	AddObject(&m_Maxdirectionx("maxdirectionx"));
	AddObject(&m_Mindirectiony("mindirectiony"));
	AddObject(&m_Maxdirectiony("maxdirectiony"));
	AddObject(&m_Mindirectionz("mindirectionz"));
	AddObject(&m_Maxdirectionz("maxdirectionz"));
	AddObject(&m_Mintwistx("mintwistx"));
	AddObject(&m_Mintwistxend("mintwistxend"));
	AddObject(&m_Maxtwistx("maxtwistx"));
	AddObject(&m_Maxtwistxend("maxtwistxend"));
	AddObject(&m_Twistxlife("twistxlife"));
	AddObject(&m_Mintwisty("mintwisty"));
	AddObject(&m_Mintwistyend("mintwistyend"));
	AddObject(&m_Maxtwisty("maxtwisty"));
	AddObject(&m_Maxtwistyend("maxtwistyend"));
	AddObject(&m_Twistylife("twistylife"));
	AddObject(&m_Mintwistz("mintwistz"));
	AddObject(&m_Mintwistzend("mintwistzend"));
	AddObject(&m_Maxtwistz("maxtwistz"));
	AddObject(&m_Maxtwistzend("maxtwistzend"));
	AddObject(&m_Twistzlife("twistzlife"));
	AddObject(&m_Minwidth("minwidth"));
	AddObject(&m_Minwidthend("minwidthend"));
	AddObject(&m_Maxwidth("maxwidth"));
	AddObject(&m_Maxwidthend("maxwidthend"));
	AddObject(&m_Widthlife("widthlife"));
	AddObject(&m_Minheight("minheight"));
	AddObject(&m_Minheightend("minheightend"));
	AddObject(&m_Maxheight("maxheight"));
	AddObject(&m_Maxheightend("maxheightend"));
	AddObject(&m_Heightlife("heightlife"));
	AddObject(&m_Minangle("minangle"));
	AddObject(&m_Minangleend("minangleend"));
	AddObject(&m_Maxangle("maxangle"));
	AddObject(&m_Maxangleend("maxangleend"));
	AddObject(&m_Anglelife("anglelife"));
	AddObject(&m_Color("color"));
//	AddObject(&m_Texture("texture"));
//	AddObject(&m_Emitter("emitter"));

	m_TimeBase = 0;

	m_Animate = false;
}

CParticleBurst::~CParticleBurst()
{
	if(m_TimeBase)
		m_TimeBase->Release();
}

bool CParticleBurst::Create()
{
	g_return_val_if_fail(k3dTransformObject::Create(), false);

	m_TimeBase = sdpQueryInterface<k3dITimeBase>(Document());
	g_return_val_if_fail(m_TimeBase, false);

	return true;
}

///////////////////////////////////////////////////////////////
// k3dIUnknown overrides

sdpResult CParticleBurst::QueryInterface(const sdpIID& InterfaceID, void** Interface)
{
	if(sdpSupportInterface<k3dIAnimate>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dIEditorRender>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dIEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dILocationServer>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dIObject>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dIPersist>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dIRiRender>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dITarget>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dITimeServer>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dITransform>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dIUserEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<sdpIUnknown>(static_cast<k3dIObject*>(this), InterfaceID, Interface)) return SDP_S_OK;										

	return SDP_E_NO_INTERFACE;
}

gulong CParticleBurst::AddRef()
{
	return sdpComAddRef(m_ReferenceCount);
}

gulong CParticleBurst::Release()
{
	return sdpComRelease(this, m_ReferenceCount);
}

//////////////////////////////////////////////////////
// Random number generator

#define RANDMODULUS			2147483647
#define RANDMULTIPLIER	16807
#define RANDINCREMENT		0

double CParticleBurst::DRand(gulong& Seed)
{
	Seed = ((RANDMULTIPLIER * Seed) + RANDINCREMENT) % RANDMODULUS;
	return((double)Seed / (double)RANDMODULUS);
}

///////////////////////////////////////////////////////////////
// k3dIObject overrides

sdpResult CParticleBurst::SetObjectContext(k3dIObjectContext* Context)
{
	g_return_val_if_fail(sdpSucceeded(k3dTransformObject::SetObjectContext(Context)), SDP_E_FAIL);
	
	// Initialize our shaders ...
//	g_return_val_if_fail(m_Shader.Create(Document()), SDP_E_FAIL);
	
	return SDP_S_OK;
}

sdpResult CParticleBurst::EditObject()
{
	g_return_val_if_fail(m_UserInterface.Create(this), SDP_E_FAIL);
	return SDP_S_OK;
}

sdpResult CParticleBurst::ObjectRenamed(SDPCSTRING NewName)
{
	m_UserInterface.ObjectRenamed(NewName);
	return k3dTransformObject::ObjectRenamed(NewName);
}

sdpResult CParticleBurst::ObjectDeleted()
{
	m_UserInterface.ObjectDeleted();
	return k3dTransformObject::ObjectDeleted();
}

///////////////////////////////////////////////////////////////
// k3dIEventClient overrides ...

sdpResult CParticleBurst::OnEvent(k3dIObject* Object, SDPCSTRING Event, gulong ClientData)
{
/*
	if(ObjectID == m_Birthrate.ObjectID())
		UpdateCreateParticles();
	else
		UpdateCalculateParticles();
*/

	if(Event == sdpString("channel-changed"))
		{
			UpdateCalculateParticles();
			OnModify(false);
			return SDP_S_OK;
		}

	return k3dTransformObject::OnEvent(Object, Event, ClientData);
}

/*
sdpResult CParticleBurst::OnPosition(const k3dID ObjectID)
{

	k3dTransformObject::OnPosition(ObjectID);
	return SDP_S_OK;
}

sdpResult CParticleBurst::OnTrack(const k3dID ObjectID)
{
	return SDP_S_OK;
}

sdpResult CParticleBurst::OnSelectionChanged(const k3dID ObjectID)
{
	return SDP_S_OK;
}

sdpResult CParticleBurst::OnSelectionDeparting(const k3dID ObjectID)
{
	return SDP_S_OK;
}
*/

///////////////////////////////////////////////////////////////
// k3dIEditorRender overrides

sdpResult CParticleBurst::EditorRender(k3dIEditorEngine* Engine)
{
	if(POINTS == m_ParticleType() || FLARES == m_ParticleType())
		{
			double globaltime = m_Globaltime->GetValue1(m_CurrentTime, m_DeltaTime);

			glDisable(GL_LIGHTING);
			glShadeModel(GL_FLAT);
			glDisable(GL_TEXTURE_1D);
			glDisable(GL_TEXTURE_2D);

			glColor3d(0.0, 0.0, 0.0);
			glPointSize(2.0f);
			glDisable(GL_POINT_SMOOTH);

			// Draw particles
			glBegin(GL_POINTS);
			ParticleIterator end = m_Particles.end();
			for(ParticleIterator particle = m_Particles.begin(); particle != end; particle++)
				{
					// Ignore dead and unborn particles
					if(particle->m_Time[0] > globaltime || particle->m_Time[1] < globaltime)
						continue;

					glVertex3dv(particle->m_AnimXYZ);
				}
			glEnd();
		}

	DrawTransformationHandles(this, 1.0);

	return SDP_S_OK;
}

///////////////////////////////////////////////////////////////
// k3dIRiRender overrides
sdpResult CParticleBurst::RiRender(k3dIRenderFrame* Frame, k3dIRiEngine* Engine)
{
	double strength, particlestrength;
	double globaltime, particletime, particlesystemtime, particlesize;
	double strengthtime, sizetime;

	sdpVector3 pcolor;
	sdpVector3 opacity;


	globaltime = m_Globaltime->GetValue1(m_CurrentTime, m_DeltaTime);
	strength = m_Globalstrength->GetValue1(globaltime, m_DeltaTime);

	if(POINTS == m_ParticleType())
		{
			if(GLOBAL == m_ColorType())
				{
					m_Color->GetValue3(globaltime, m_DeltaTime, pcolor);
				}

			ParticleIterator end = m_Particles.end();
			for(ParticleIterator particle = m_Particles.begin(); particle != end; particle++)
				{
					// Ignore dead or unborn particles
					if(particle->m_Time[0] > globaltime || particle->m_Time[1] < globaltime)
						continue;

					particlesystemtime = globaltime - particle->m_Time[0];
					particletime = particlesystemtime / (particle->m_Time[1] - particle->m_Time[0]);

					strengthtime = m_StrengthLifetime() ? particletime : particlesystemtime;
					sizetime = m_SizeLifetime() ? particletime : particlesystemtime;


					if(m_StrengthRelative())
						particlestrength = m_Strengthlife->GetValue1(strengthtime, m_DeltaTime) * particle->m_Strength[1] + particle->m_Strength[0];
					else
						particlestrength = m_Strengthlife->GetValue1(strengthtime, m_DeltaTime) * (particle->m_Strength[1] - particle->m_Strength[0]) + particle->m_Strength[0];

					if(LIFETIME == m_ColorType())
						m_Color->GetValue3(particletime, m_DeltaTime, pcolor);
					else if(RANDOM == m_ColorType())
						pcolor = particle->m_Color;

					if(m_SizeRelative())
						particlesize = m_Sizelife->GetValue1(sizetime, m_DeltaTime) * particle->m_Size[1] + particle->m_Size[0];
					else
						particlesize = m_Sizelife->GetValue1(sizetime, m_DeltaTime) * (particle->m_Size[1] - particle->m_Size[0]) + particle->m_Size[0];

					Engine->RiTransformBegin();
					Engine->RiIdentity();

					Engine->RiTranslate((float)particle->m_AnimXYZ[0], (float)particle->m_AnimXYZ[1], (float)particle->m_AnimXYZ[2]);
					Engine->RiScale((float)particlesize, (float)particlesize, (float)particlesize);

					Engine->RiColor(pcolor);

					opacity = particlestrength * strength;
					Engine->RiOpacity(opacity);

					Engine->RiSurfaceV("particleburstglow", 0, 0, 0);
					Engine->RiSphereV(0.5, -0.5, 0.5, 360.0, 0, 0, 0);

					Engine->RiTransformEnd();
				}
		}

	return SDP_S_OK;
}

///////////////////////////////////////////////////////////////
// k3dILocationServer overrides

gulong CParticleBurst::GetLocationCount()
{
	return m_Particles.size();
}

K3DLOCATIONHANDLE CParticleBurst::GetFirstLocation()
{
	return K3DLOCATIONHANDLE(m_Particles.size() ? 1 : 0);
}

K3DLOCATIONHANDLE CParticleBurst::GetNextLocation(K3DLOCATIONHANDLE Location)
{
	// Sanity checks ...
	g_return_val_if_fail(Location, 0);
	g_return_val_if_fail(gulong(Location) <= m_Particles.size(), 0);

	return K3DLOCATIONHANDLE(gulong(Location) < m_Particles.size() ? gulong(Location)+1 : 0);
}

k3dILocation* CParticleBurst::GetLocation(K3DLOCATIONHANDLE Location)
{
	// Sanity checks ...
	g_return_val_if_fail(Location, 0);
	g_return_val_if_fail(gulong(Location) <= m_Particles.size(), 0);

	return &m_Particles[gulong(Location)-1];
}

///////////////////////////////////////////////////////////////
// k3dIPersist overrides

sdpResult CParticleBurst::InitNew()
{
	g_return_val_if_fail(sdpSucceeded(k3dTransformObject::InitNew()), SDP_E_FAIL);

	g_return_val_if_fail(m_Globaltime.Create(Document(), "Particle Burst Global Time", 0.0, true, 0.0, 1.0, "Time", "Time", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	k3dChannelSetLinearValue(m_Globaltime.Interface(), 0.0, 1.0);

	g_return_val_if_fail(m_Birthrate.Create(Document(), "Particle Burst Birthrate", 100.0, true, 0.0, DBL_MAX, "Time", "Birthrate", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Minlife.Create(Document(), "Particle Burst Minimum Life", 1.0, true, 0.0, 1.0, "Time", "Minimum Life", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxlife.Create(Document(), "Particle Burst Maximum Life", 1.0, true, 0.0, 1.0, "Time", "Maximum Life", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Globalstrength.Create(Document(), "Particle Burst Global Strength", 1.0, true, 0.0, 1.0, "Time", "Global Strength", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Minstrength.Create(Document(), "Particle Burst Start Strength Minimum", 1.0, true, 0.0, 1.0, "Time", "Minimum Strength", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Minstrengthend.Create(Document(), "Particle Burst End Strength Minimum", 1.0, true, 0.0, 1.0, "Time", "Minimum Strength", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxstrength.Create(Document(), "Particle Burst Start Strength Maximum", 1.0, true, 0.0, 1.0, "Time", "Maximum Strength", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxstrengthend.Create(Document(), "Particle Burst End Strength Maximum", 1.0, true, 0.0, 1.0, "Time", "Maximum Strength", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);

	g_return_val_if_fail(m_Strengthlife.Create(Document(), "Particle Burst Lifetime Strength", 1.0, true, 0.0, 1.0, "Time", "Strength", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	k3dChannelSetLinearValue(m_Strengthlife.Interface(), 0.0, 1.0);

	g_return_val_if_fail(m_Minsize.Create(Document(), "Particle Burst Start Size Minimum", 0.2, true, 0.0, 1.0, "Time", "Minimum Size", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Minsizeend.Create(Document(), "Particle Burst End Size Minimum", 0.2, true, 0.0, 1.0, "Time", "Minimum Size", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxsize.Create(Document(), "Particle Burst Start Size Maximum", 0.2, true, 0.0, 1.0, "Time", "Maximum Size", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxsizeend.Create(Document(), "Particle Burst End Size Maximum", 0.2, true, 0.0, 1.0, "Time", "Maximum Size", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);

	g_return_val_if_fail(m_Sizelife.Create(Document(), "Particle Burst Lifetime Size", 1.0, true, 0.0, 1.0, "Time", "Size", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	k3dChannelSetLinearValue(m_Sizelife.Interface(), 0.0, 1.0);

	g_return_val_if_fail(m_Mindistance.Create(Document(), "Particle Burst Start Distance Minimum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Distance", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Mindistanceend.Create(Document(), "Particle Burst End Distance Minimum", 10.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Distance", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxdistance.Create(Document(), "Particle Burst Start Distance Maximum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Distance", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxdistanceend.Create(Document(), "Particle Burst End Distance Maximum", 10.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Distance", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);

	g_return_val_if_fail(m_Distancelife.Create(Document(), "Particle Burst Lifetime Distance", 1.0, true, 0.0, 1.0, "Time", "Distance", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	k3dChannelSetLinearValue(m_Distancelife.Interface(), 0.0, 1.0);

	g_return_val_if_fail(m_Mindriftx.Create(Document(), "Particle Burst Start Drift X Minimum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Drift X", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Mindriftxend.Create(Document(), "Particle Burst End Drift X Minimum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Drift X", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxdriftx.Create(Document(), "Particle Burst Start Drift X Maximum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Drift X", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxdriftxend.Create(Document(), "Particle Burst End Drift X Maximum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Drift X", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);

	g_return_val_if_fail(m_Driftxlife.Create(Document(), "Particle Burst Lifetime Drift X", 1.0, true, 0.0, 1.0, "Time", "Drift X", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	k3dChannelSetLinearValue(m_Driftxlife.Interface(), 0.0, 1.0);

	g_return_val_if_fail(m_Mindrifty.Create(Document(), "Particle Burst Start Drift Y Minimum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Drift Y", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Mindriftyend.Create(Document(), "Particle Burst End Drift Y Minimum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Drift Y", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxdrifty.Create(Document(), "Particle Burst Start Drift Y Maximum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Drift Y", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxdriftyend.Create(Document(), "Particle Burst End Drift Y Maximum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Drift Y", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);

	g_return_val_if_fail(m_Driftylife.Create(Document(), "Particle Burst Lifetime Drift Y", 1.0, true, 0.0, 1.0, "Time", "Drift Y", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	k3dChannelSetLinearValue(m_Driftylife.Interface(), 0.0, 1.0);

	g_return_val_if_fail(m_Mindriftz.Create(Document(), "Particle Burst Start Drift Z Minimum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Drift Z", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Mindriftzend.Create(Document(), "Particle Burst End Drift Z Minimum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Drift Z", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxdriftz.Create(Document(), "Particle Burst Start Drift Z Maximum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Drift Z", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxdriftzend.Create(Document(), "Particle Burst End Drift Z Maximum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Drift Z", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);

	g_return_val_if_fail(m_Driftzlife.Create(Document(), "Particle Burst Lifetime Drift Z", 1.0, true, 0.0, 1.0, "Time", "Drift Z", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	k3dChannelSetLinearValue(m_Driftzlife.Interface(), 0.0, 1.0);

	g_return_val_if_fail(m_Mindirectionx.Create(Document(), "Particle Burst Start Direction X Minimum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Direction X", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxdirectionx.Create(Document(), "Particle Burst Start Direction X Maximum", sdpPi, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Direction X", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Mindirectiony.Create(Document(), "Particle Burst Start Direction Y Minimum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Direction Y", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxdirectiony.Create(Document(), "Particle Burst Start Direction Y Maximum", sdpPiTimes2, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Direction Y", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Mindirectionz.Create(Document(), "Particle Burst Start Direction Z Minimum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Direction Z", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxdirectionz.Create(Document(), "Particle Burst Start Direction Z Maximum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Direction Z", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Mintwistx.Create(Document(), "Particle Burst Start Twist X Minimum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Twist X", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Mintwistxend.Create(Document(), "Particle Burst End Twist X Minimum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Twist X", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxtwistx.Create(Document(), "Particle Burst Start Twist X Maximum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Twist X", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxtwistxend.Create(Document(), "Particle Burst End Twist X Maximum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Twist X", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);

	g_return_val_if_fail(m_Twistxlife.Create(Document(), "Particle Burst Lifetime Twist X", 1.0, true, 0.0, 1.0, "Time", "Twist X", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	k3dChannelSetLinearValue(m_Twistxlife.Interface(), 0.0, 1.0);

	g_return_val_if_fail(m_Mintwisty.Create(Document(), "Particle Burst Start Twist Y Minimum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Twist Y", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Mintwistyend.Create(Document(), "Particle Burst End Twist Y Minimum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Twist Y", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxtwisty.Create(Document(), "Particle Burst Start Twist Y Maximum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Twist Y", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxtwistyend.Create(Document(), "Particle Burst End Twist Y Maximum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Twist Y", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);

	g_return_val_if_fail(m_Twistylife.Create(Document(), "Particle Burst Lifetime Twist Y", 1.0, true, 0.0, 1.0, "Time", "Twist Y", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	k3dChannelSetLinearValue(m_Twistylife.Interface(), 0.0, 1.0);

	g_return_val_if_fail(m_Mintwistz.Create(Document(), "Particle Burst Start Twist Z Minimum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Twist Z", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Mintwistzend.Create(Document(), "Particle Burst End Twist Z Minimum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Minimum Twist Z", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxtwistz.Create(Document(), "Particle Burst Start Twist Z Maximum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Twist Z", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxtwistzend.Create(Document(), "Particle Burst End Twist Z Maximum", 0.0, true, -1.0 * DBL_MAX, DBL_MAX, "Time", "Maximum Twist Z", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);

	g_return_val_if_fail(m_Twistzlife.Create(Document(), "Particle Burst Lifetime Twist Z", 1.0, true, 0.0, 1.0, "Time", "Twist Z", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	k3dChannelSetLinearValue(m_Twistzlife.Interface(), 0.0, 1.0);

	g_return_val_if_fail(m_Minwidth.Create(Document(), "Particle Burst Start Width Minimum", 1.0, true, 0.0, 1.0, "Time", "Minimum Width", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Minwidthend.Create(Document(), "Particle Burst End Width Minimum", 1.0, true, 0.0, 1.0, "Time", "Minimum Width", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxwidth.Create(Document(), "Particle Burst Start Width Maximum", 1.0, true, 0.0, 1.0, "Time", "Maximum Width", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxwidthend.Create(Document(), "Particle Burst End Width Maximum", 1.0, true, 0.0, 1.0, "Time", "Maximum Width", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);

	g_return_val_if_fail(m_Widthlife.Create(Document(), "Particle Burst Lifetime Width", 1.0, true, 0.0, 1.0, "Time", "Width", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	k3dChannelSetLinearValue(m_Widthlife.Interface(), 0.0, 1.0);

	g_return_val_if_fail(m_Minheight.Create(Document(), "Particle Burst Start Height Minimum", 1.0, true, 0.0, 1.0, "Time", "Minimum Height", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Minheightend.Create(Document(), "Particle Burst End Height Minimum", 1.0, true, 0.0, 1.0, "Time", "Minimum Height", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxheight.Create(Document(), "Particle Burst Start Height Maximum", 1.0, true, 0.0, 1.0, "Time", "Maximum Height", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxheightend.Create(Document(), "Particle Burst End Height Maximum", 1.0, true, 0.0, 1.0, "Time", "Maximum Height", k3dIChannel::DATA_DISTANCE, true, this), SDP_E_FAIL);

	g_return_val_if_fail(m_Heightlife.Create(Document(), "Particle Burst Lifetime Height", 1.0, true, 0.0, 1.0, "Time", "Height", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	k3dChannelSetLinearValue(m_Heightlife.Interface(), 0.0, 1.0);

	g_return_val_if_fail(m_Minangle.Create(Document(), "Particle Burst Start Angle Minimum", 0.0, true, 0.0, 1.0, "Time", "Minimum Angle", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Minangleend.Create(Document(), "Particle Burst End Angle Minimum", 0.0, true, 0.0, 1.0, "Time", "Minimum Angle", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxangle.Create(Document(), "Particle Burst Start Angle Maximum", 0.0, true, 0.0, 1.0, "Time", "Maximum Angle", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);
	g_return_val_if_fail(m_Maxangleend.Create(Document(), "Particle Burst End Angle Maximum", 0.0, true, 0.0, 1.0, "Time", "Maximum Angle", k3dIChannel::DATA_ANGLE, true, this), SDP_E_FAIL);

	g_return_val_if_fail(m_Anglelife.Create(Document(), "Particle Burst Lifetime Angle", 1.0, true, 0.0, 1.0, "Time", "Angle", k3dIChannel::DATA_SCALAR, true, this), SDP_E_FAIL);
	k3dChannelSetLinearValue(m_Anglelife.Interface(), 0.0, 1.0);

	g_return_val_if_fail(m_Color.Create(Document(), "Particle Burst Color", sdpVector3(1.0), true, "Time", "Color", true, this), SDP_E_FAIL);

//	g_return_val_if_fail(m_Texture.CreateObjectSelector(Document(), "ParticleBurst texture", IID_k3dITexture, true, true, true, true)));
//	g_return_val_if_fail(m_Emitter.Create(Document(), "ParticleBurst emitter", false, true, true, NULL, KGUID_NULL)));

	// Force calculation of the particle system
	UpdateFull();

	return SDP_S_OK;
}

sdpResult CParticleBurst::LoadComplete()
{
	// Call the base-class version ...
	g_return_val_if_fail(k3dTransformObject::LoadComplete(), SDP_E_FAIL);

	UpdateFull();
	return SDP_S_OK;
}

///////////////////////////////////////////////////////////////
// k3dIAnimate overrides

sdpResult CParticleBurst::PopAnimate()
{
	double globaltime, particlesystemtime, particletime;
	double twisttime, distancetime, drifttime;
	sdpVector3 direction, twist;

	double px, py;
	double cosx, cosy, cosz;
	double sinx, siny, sinz;

	if(m_Animate)
		{
			if(POINTS == m_ParticleType() || FLARES == m_ParticleType())
				{
					// Calculate Global time
					globaltime = m_Globaltime->GetValue1(m_CurrentTime, m_DeltaTime);

					// For each particle ...
					ParticleIterator end = m_Particles.end();
					for(ParticleIterator particle = m_Particles.begin(); particle != end; particle++)
						{
							// Ignore dead and unborn particles
							if(particle->m_Time[0] > globaltime || particle->m_Time[1] < globaltime) continue;

							// Calculate particle times
							particlesystemtime = globaltime - particle->m_Time[0];
							particletime = particlesystemtime / (particle->m_Time[1] - particle->m_Time[0]);
							twisttime = m_TwistLifetime() ? particletime : particlesystemtime;
							distancetime = m_DistanceLifetime() ? particletime : particlesystemtime;
							drifttime = m_DriftLifetime() ? particletime : particlesystemtime;

							// Calculate particle position
							particle->m_AnimXYZ = 0.0;

							if(m_DistanceRelative())
								{
									particle->m_AnimXYZ[1] = particle->m_Distance[0] + m_Distancelife->GetValue1(distancetime, m_DeltaTime) * particle->m_Distance[1];
								}
							else
								{
									particle->m_AnimXYZ[1] = particle->m_Distance[0] + m_Distancelife->GetValue1(distancetime, m_DeltaTime) * (particle->m_Distance[1] - particle->m_Distance[0]);
								}


							direction = particle->m_DirectionXYZ;

							cosx = cos(direction[0]);
							sinx = sin(direction[0]);
							cosy = cos(direction[1]);
							siny = sin(direction[1]);
							cosz = cos(direction[2]);
							sinz = sin(direction[2]);

							py = particle->m_AnimXYZ[1];
							particle->m_AnimXYZ[1] = (cosx * particle->m_AnimXYZ[1]) - (sinx * particle->m_AnimXYZ[2]);
							particle->m_AnimXYZ[2] = (sinx * py) + (cosx * particle->m_AnimXYZ[2]);

							px = particle->m_AnimXYZ[0];
							particle->m_AnimXYZ[0] = (cosy * particle->m_AnimXYZ[0]) + (siny * particle->m_AnimXYZ[2]);
							particle->m_AnimXYZ[2] = (cosy * particle->m_AnimXYZ[2]) - (siny * px);

							px = particle->m_AnimXYZ[0];
							particle->m_AnimXYZ[0] = (cosz * particle->m_AnimXYZ[0]) - (sinz * particle->m_AnimXYZ[1]);
							particle->m_AnimXYZ[1] = (sinz * px) + (cosz * particle->m_AnimXYZ[1]);


							if(m_TwistRelative())
								{
									twist[0] = particle->m_TwistXYZ[0][0] + m_Twistxlife->GetValue1(twisttime, m_DeltaTime) * particle->m_TwistXYZ[1][0];
									twist[1] = particle->m_TwistXYZ[0][1] + m_Twistylife->GetValue1(twisttime, m_DeltaTime) * particle->m_TwistXYZ[1][1];
									twist[2] = particle->m_TwistXYZ[0][2] + m_Twistzlife->GetValue1(twisttime, m_DeltaTime) * particle->m_TwistXYZ[1][2];
								}
							else
								{
									twist[0] = particle->m_TwistXYZ[0][0] + m_Twistxlife->GetValue1(twisttime, m_DeltaTime) * (particle->m_TwistXYZ[1][0] - particle->m_TwistXYZ[0][0]);
									twist[1] = particle->m_TwistXYZ[0][1] + m_Twistylife->GetValue1(twisttime, m_DeltaTime) * (particle->m_TwistXYZ[1][1] - particle->m_TwistXYZ[0][1]);
									twist[2] = particle->m_TwistXYZ[0][2] + m_Twistzlife->GetValue1(twisttime, m_DeltaTime) * (particle->m_TwistXYZ[1][2] - particle->m_TwistXYZ[0][2]);
								}

							cosx = cos(twist[0]);
							sinx = sin(twist[0]);
							cosy = cos(twist[1]);
							siny = sin(twist[1]);
							cosz = cos(twist[2]);
							sinz = sin(twist[2]);


							py = particle->m_AnimXYZ[1];
							particle->m_AnimXYZ[1] = (cosx * particle->m_AnimXYZ[1]) - (sinx * particle->m_AnimXYZ[2]);
							particle->m_AnimXYZ[2] = (sinx * py) + (cosx * particle->m_AnimXYZ[2]);

							px = particle->m_AnimXYZ[0];
							particle->m_AnimXYZ[0] = (cosy * particle->m_AnimXYZ[0]) + (siny * particle->m_AnimXYZ[2]);
							particle->m_AnimXYZ[2] = (cosy * particle->m_AnimXYZ[2]) - (siny * px);

							px = particle->m_AnimXYZ[0];
							particle->m_AnimXYZ[0] = (cosz * particle->m_AnimXYZ[0]) - (sinz * particle->m_AnimXYZ[1]);
							particle->m_AnimXYZ[1] = (sinz * px) + (cosz * particle->m_AnimXYZ[1]);


							if(m_DriftRelative())
								{
									particle->m_AnimXYZ[0] += particle->m_DriftXYZ[0][0] + m_Driftxlife->GetValue1(drifttime, m_DeltaTime) * particle->m_DriftXYZ[1][0];
									particle->m_AnimXYZ[1] += particle->m_DriftXYZ[0][1] + m_Driftylife->GetValue1(drifttime, m_DeltaTime) * particle->m_DriftXYZ[1][1];
									particle->m_AnimXYZ[2] += particle->m_DriftXYZ[0][2] + m_Driftzlife->GetValue1(drifttime, m_DeltaTime) * particle->m_DriftXYZ[1][2];
								}
							else
								{
									particle->m_AnimXYZ[0] += particle->m_DriftXYZ[0][0] + m_Driftxlife->GetValue1(drifttime, m_DeltaTime) * (particle->m_DriftXYZ[1][0] - particle->m_DriftXYZ[0][0]);
									particle->m_AnimXYZ[1] += particle->m_DriftXYZ[0][1] + m_Driftylife->GetValue1(drifttime, m_DeltaTime) * (particle->m_DriftXYZ[1][1] - particle->m_DriftXYZ[0][1]);
									particle->m_AnimXYZ[2] += particle->m_DriftXYZ[0][2] + m_Driftzlife->GetValue1(drifttime, m_DeltaTime) * (particle->m_DriftXYZ[1][2] - particle->m_DriftXYZ[0][2]);
								}

							particle->m_AnimXYZ += particle->m_BirthXYZ;
						}
				}

/*
			else if(POLYGONS == m_ParticleType())
				{
					double minlife, maxlife, dlife;
					double minsize, minsizeend, maxsize, maxsizeend, dsize, dsizeend;
					double mindistance, mindistanceend, maxdistance, maxdistanceend, ddistance, ddistanceend;
					double mindriftx, mindriftxend, maxdriftx, maxdriftxend, ddriftx, ddriftxend;
					double mindrifty, mindriftyend, maxdrifty, maxdriftyend, ddrifty, ddriftyend;
					double mindriftz, mindriftzend, maxdriftz, maxdriftzend, ddriftz, ddriftzend;
					double mindirectionx, maxdirectionx, ddirectionx;
					double mindirectiony, maxdirectiony, ddirectiony;
					double mindirectionz, maxdirectionz, ddirectionz;
					double mintwistx, mintwistxend, maxtwistx, maxtwistxend, dtwistx, dtwistxend;
					double mintwisty, mintwistyend, maxtwisty, maxtwistyend, dtwisty, dtwistyend;
					double mintwistz, mintwistzend, maxtwistz, maxtwistzend, dtwistz, dtwistzend;



					// Calculate Global time
					globaltime = m_Globaltime->GetValue1(m_CurrentTime, m_DeltaTime);

					// Find min / max values at current time
					minlife = m_Minlife->GetValue1(globaltime, m_DeltaTime);
					maxlife = m_Maxlife->GetValue1(globaltime, m_DeltaTime);
					dlife = maxlife - minlife;
					minsize = m_Minsize->GetValue1(globaltime, m_DeltaTime);
					maxsize = m_Maxsize->GetValue1(globaltime, m_DeltaTime);
					dsize = maxsize - minsize;
					minsizeend = m_Minsizeend->GetValue1(globaltime, m_DeltaTime);
					maxsizeend = m_Maxsizeend->GetValue1(globaltime, m_DeltaTime);
					dsizeend = maxsizeend - minsizeend;
					mindistance = m_Mindistance->GetValue1(globaltime, m_DeltaTime);
					maxdistance = m_Maxdistance->GetValue1(globaltime, m_DeltaTime);
					ddistance = maxdistance - mindistance;
					mindistanceend = m_Mindistanceend->GetValue1(globaltime, m_DeltaTime);
					maxdistanceend = m_Maxdistanceend->GetValue1(globaltime, m_DeltaTime);
					ddistanceend = maxdistanceend - mindistanceend;
					mindriftx = m_Mindriftx->GetValue1(globaltime, m_DeltaTime);
					maxdriftx = m_Maxdriftx->GetValue1(globaltime, m_DeltaTime);
					ddriftx = maxdriftx - mindriftx;
					mindriftxend = m_Mindriftxend->GetValue1(globaltime, m_DeltaTime);
					maxdriftxend = m_Maxdriftxend->GetValue1(globaltime, m_DeltaTime);
					ddriftxend = maxdriftxend - mindriftxend;
					mindrifty = m_Mindrifty->GetValue1(globaltime, m_DeltaTime);
					maxdrifty = m_Maxdrifty->GetValue1(globaltime, m_DeltaTime);
					ddrifty = maxdrifty - mindrifty;
					mindriftyend = m_Mindriftyend->GetValue1(globaltime, m_DeltaTime);
					maxdriftyend = m_Maxdriftyend->GetValue1(globaltime, m_DeltaTime);
					ddriftyend = maxdriftyend - mindriftyend;
					mindriftz = m_Mindriftz->GetValue1(globaltime, m_DeltaTime);
					maxdriftz = m_Maxdriftz->GetValue1(globaltime, m_DeltaTime);
					ddriftz = maxdriftz - mindriftz;
					mindriftzend = m_Mindriftzend->GetValue1(globaltime, m_DeltaTime);
					maxdriftzend = m_Maxdriftzend->GetValue1(globaltime, m_DeltaTime);
					ddriftzend = maxdriftzend - mindriftzend;
					mindirectionx = m_Mindirectionx->GetValue1(globaltime, m_DeltaTime);
					maxdirectionx = m_Maxdirectionx->GetValue1(globaltime, m_DeltaTime);
					ddirectionx = maxdirectionx - mindirectionx;
					mindirectiony = m_Mindirectiony->GetValue1(globaltime, m_DeltaTime);
					maxdirectiony = m_Maxdirectiony->GetValue1(globaltime, m_DeltaTime);
					ddirectiony = maxdirectiony - mindirectiony;
					mindirectionz = m_Mindirectionz->GetValue1(globaltime, m_DeltaTime);
					maxdirectionz = m_Maxdirectionz->GetValue1(globaltime, m_DeltaTime);
					ddirectionz = maxdirectionz - mindirectionz;
					mintwistx = m_Mintwistx->GetValue1(globaltime, m_DeltaTime);
					maxtwistx = m_Maxtwistx->GetValue1(globaltime, m_DeltaTime);
					dtwistx = maxtwistx - mintwistx;
					mintwistxend = m_Mintwistxend->GetValue1(globaltime, m_DeltaTime);
					maxtwistxend = m_Maxtwistxend->GetValue1(globaltime, m_DeltaTime);
					dtwistxend = maxtwistxend - mintwistxend;
					mintwisty = m_Mintwisty->GetValue1(globaltime, m_DeltaTime);
					maxtwisty = m_Maxtwisty->GetValue1(globaltime, m_DeltaTime);
					dtwisty = maxtwisty - mintwisty;
					mintwistyend = m_Mintwistyend->GetValue1(globaltime, m_DeltaTime);
					maxtwistyend = m_Maxtwistyend->GetValue1(globaltime, m_DeltaTime);
					dtwistyend = maxtwistyend - mintwistyend;
					mintwistz = m_Mintwistz->GetValue1(globaltime, m_DeltaTime);
					maxtwistz = m_Maxtwistz->GetValue1(globaltime, m_DeltaTime);
					dtwistz = maxtwistz - mintwistz;
					mintwistzend = m_Mintwistzend->GetValue1(globaltime, m_DeltaTime);
					maxtwistzend = m_Maxtwistzend->GetValue1(globaltime, m_DeltaTime);
					dtwistzend = maxtwistzend - mintwistzend;


					// Reset particle system to recalculate
					seed = (gulong)m_RandomSeed();

					// Get Emitter coordinates
					m_Emitter->SetFirstLocation();
					m_Emitter->GetAnimXYZ(emittercoords);

					// Enumerate child objects and alter
					for(k3dIObjectContext* context = Document()->GetFirstObjectContext(); context; context = Document()->GetNextObjectContext(context))
						{
							// Is the object a descendant of this path?
							if(SDP_S_OK ! m_ObjectContext->QueryAncestor(m_ThisHandle)) continue;

							object = m_ObjectContext->Object();
							if(object)
								{
									if(POLYGONS == m_ParticleType())
										{
											// Does this object support k3dIPolygonServer?
											hr = object->QueryInterface(IID_k3dIPolygonServer, (void**)&polygonclient);
											if(sdpSucceeded(hr))
												{
													for(hr = polygonclient->SetFirstPolygon(); SDP_S_OK == hr; hr = polygonclient->SetNextPolygon())
														{
															// Calculate particle position
															seed = Rand(seed);
															particle->m_Time[0] = 0.0;
															seed = Rand(seed);
															particle->m_Time[1] = dlife * (DRand(seed)) + minlife;

															if(m_DistanceRandom())
																{
																	seed = Rand(seed);
																	particle->m_Distance[0] = ddistance * (DRand(seed)) + mindistance;
																	seed = Rand(seed);
																	particle->m_Distance[1] = ddistanceend * (DRand(seed)) + mindistanceend;
																}
															else
																{
																	particle->m_Distance[0] = ddistance * fraction + mindistance;
																	particle->m_Distance[1] = ddistanceend * fraction + mindistanceend;
																}

															if(m_DriftRandom())
																{
																	seed = Rand(seed);
																	particle->m_DriftXYZ[0] = ddriftx * (DRand(seed)) + mindriftx;
																	seed = Rand(seed);
																	particle->m_DriftXYZ[1] = ddriftxend * (DRand(seed)) + mindriftxend;
																	seed = Rand(seed);
																	particle->m_DriftXYZ[0] = ddrifty * (DRand(seed)) + mindrifty;
																	seed = Rand(seed);
																	particle->m_DriftXYZ[1] = ddriftyend * (DRand(seed)) + mindriftyend;
																	seed = Rand(seed);
																	particle->m_DriftXYZ[0] = ddriftz * (DRand(seed)) + mindriftz;
																	seed = Rand(seed);
																	particle->m_DriftXYZ[1] = ddriftzend * (DRand(seed)) + mindriftzend;
																}
															else
																{
																	particle->m_DriftXYZ[0] = ddriftx * fraction + mindriftx;
																	particle->m_DriftXYZ[1] = ddriftxend * fraction + mindriftxend;
																	particle->m_DriftXYZ[0] = ddrifty * fraction + mindrifty;
																	particle->m_DriftXYZ[1] = ddriftyend * fraction + mindriftyend;
																	particle->m_DriftXYZ[0] = ddriftz * fraction + mindriftz;
																	particle->m_DriftXYZ[1] = ddriftzend * fraction + mindriftzend;
																}

															if(m_DirectionRandom())
																{
																	seed = Rand(seed);
																	particle->m_DirectionXYZ[0] = ddirectionx * (DRand(seed)) + mindirectionx;
																	seed = Rand(seed);
																	particle->m_DirectionXYZ[1] = ddirectiony * (DRand(seed)) + mindirectiony;
																	seed = Rand(seed);
																	particle->m_DirectionXYZ[2] = ddirectionz * (DRand(seed)) + mindirectionz;
																}
															else
																{
																	particle->m_DirectionXYZ[0] = ddirectionx * fraction + mindirectionx;
																	particle->m_DirectionXYZ[1] = ddirectiony * fraction + mindirectiony;
																	particle->m_DirectionXYZ[2] = ddirectionz * fraction + mindirectionz;
																}

															if(m_TwistRandom())
																{
																	seed = Rand(seed);
																	particle->m_TwistXYZ[0] = dtwistx * (DRand(seed)) + mintwistx;
																	seed = Rand(seed);
																	particle->m_TwistXYZ[1] = dtwistxend * (DRand(seed)) + mintwistxend;
																	seed = Rand(seed);
																	particle->m_TwistXYZ[0] = dtwisty * (DRand(seed)) + mintwisty;
																	seed = Rand(seed);
																	particle->m_TwistXYZ[1] = dtwistyend * (DRand(seed)) + mintwistyend;
																	seed = Rand(seed);
																	particle->m_TwistXYZ[0] = dtwistz * (DRand(seed)) + mintwistz;
																	seed = Rand(seed);
																	particle->m_TwistXYZ[1] = dtwistzend * (DRand(seed)) + mintwistzend;
																}
															else
																{
																	particle->m_TwistXYZ[0] = dtwistx * fraction + mintwistx;
																	particle->m_TwistXYZ[1] = dtwistxend * fraction + mintwistxend;
																	particle->m_TwistXYZ[0] = dtwisty * fraction + mintwisty;
																	particle->m_TwistXYZ[1] = dtwistyend * fraction + mintwistyend;
																	particle->m_TwistXYZ[0] = dtwistz * fraction + mintwistz;
																	particle->m_TwistXYZ[1] = dtwistzend * fraction + mintwistzend;
																}

															if(m_SizeRandom())
																{
																	seed = Rand(seed);
																	particle->m_Size[0] = dsize * (DRand(seed)) + minsize;
																	seed = Rand(seed);
																	particle->m_Size[1] = dsizeend * (DRand(seed)) + minsizeend;
																}
															else
																{
																	particle->m_Size[0] = dsize * fraction + minsize;
																	particle->m_Size[1] = dsizeend * fraction + minsizeend;
																}


															// Calculate particle time
															if(particle->m_Time[0] > globaltime)
																{
																	continue;
																}
															else
																{
																	particlesystemtime = globaltime - particle->m_Time[0];
																	particletime = particlesystemtime / (particle->m_Time[1] - particle->m_Time[0]);
																}

															particlesystemtime = min(1.0, particlesystemtime);
															particletime = min(1.0, particletime);

															twisttime = m_TwistLifetime() ? particletime : particlesystemtime;
															distancetime = m_DistanceLifetime() ? particletime : particlesystemtime;
															drifttime = m_DriftLifetime() ? particletime : particlesystemtime;
															sizetime = m_SizeLifetime() ? particletime : particlesystemtime;

															if(m_DirectionEmitter())
																{
																	pointcount = 0;
																	centercoords[0] = 0.0;
																	centercoords[1] = 0.0;
																	centercoords[2] = 0.0;


																	m_PolygonList->SetHandle(polygonclient->GetPolygonHandle());
																	for(hr = m_Polygon->SetFirstPoint(); SDP_S_OK == hr; hr = m_Polygon->SetNextPoint())
																		{
																			m_PolygonLocation->GetAnimXYZ(coords);

																			centercoords[0] += coords[0];
																			centercoords[1] += coords[1];
																			centercoords[2] += coords[2];

																			pointcount++;

																			m_PolygonLocation->SetAnimXYZ(coords);
																		}

																	if(pointcount)
																		{
																			centercoords[0] /= pointcount;
																			centercoords[1] /= pointcount;
																			centercoords[2] /= pointcount;
																		}
																	dx = centercoords[0] - emittercoords[0];
																	dy = centercoords[1] - emittercoords[1];
																	dz = centercoords[2] - emittercoords[2];

																	dist = sqrt(dx * dx + dy * dy + dz * dz);

																	if(0.0 != dist)
																		{
																			dx /= dist;
																			dy /= dist;
																			dz /= dist;
																		}

																	if(m_DistanceRelative())
																		{
																			particle->m_LocalXYZ[0] = dx * (particle->m_Distance[0] + m_Distancelife->GetValue1(distancetime, m_DeltaTime) * particle->m_Distance[1]);
																			particle->m_LocalXYZ[1] = dy * (particle->m_Distance[0] + m_Distancelife->GetValue1(distancetime, m_DeltaTime) * particle->m_Distance[1]);
																			particle->m_LocalXYZ[2] = dz * (particle->m_Distance[0] + m_Distancelife->GetValue1(distancetime, m_DeltaTime) * particle->m_Distance[1]);
																		}
																	else
																		{
																			particle->m_LocalXYZ[0] = dx * (m_Distancelife->GetValue1(distancetime, m_DeltaTime) * (particle->m_Distance[1] - particle->m_Distance[0]) + particle->m_Distance[0]);
																			particle->m_LocalXYZ[1] = dy * (m_Distancelife->GetValue1(distancetime, m_DeltaTime) * (particle->m_Distance[1] - particle->m_Distance[0]) + particle->m_Distance[0]);
																			particle->m_LocalXYZ[2] = dz * (m_Distancelife->GetValue1(distancetime, m_DeltaTime) * (particle->m_Distance[1] - particle->m_Distance[0]) + particle->m_Distance[0]);
																		}
																}
															else
																{
																	particle->m_LocalXYZ[0] = 0.0;
																	particle->m_LocalXYZ[2] = 0.0;

																	if(m_DistanceRelative())
																		{
																			particle->m_LocalXYZ[1] += particle->m_Distance[0] + m_Distancelife->GetValue1(distancetime, m_DeltaTime) * particle->m_Distance[1];
																		}
																	else
																		{
																			particle->m_LocalXYZ[1] += m_Distancelife->GetValue1(distancetime, m_DeltaTime) * (particle->m_Distance[1] - particle->m_Distance[0]) + particle->m_Distance[0];
																		}


																	direction[0] = particle->m_DirectionXYZ[0];
																	direction[1] = particle->m_DirectionXYZ[1];
																	direction[2] = particle->m_DirectionXYZ[2];

																	cosx = cos(direction[0]);
																	sinx = sin(direction[0]);
																	cosy = cos(direction[1]);
																	siny = sin(direction[1]);
																	cosz = cos(direction[2]);
																	sinz = sin(direction[2]);

																	py = particle->m_LocalXYZ[1];
																	particle->m_LocalXYZ[1] = (cosx * particle->m_LocalXYZ[1]) - (sinx * particle->m_LocalXYZ[2]);
																	particle->m_LocalXYZ[2] = (sinx * py) + (cosx * particle->m_LocalXYZ[2]);

																	px = particle->m_LocalXYZ[0];
																	particle->m_LocalXYZ[0] = (cosy * particle->m_LocalXYZ[0]) + (siny * particle->m_LocalXYZ[2]);
																	particle->m_LocalXYZ[2] = (cosy * particle->m_LocalXYZ[2]) - (siny * px);

																	px = particle->m_LocalXYZ[0];
																	particle->m_LocalXYZ[0] = (cosz * particle->m_LocalXYZ[0]) - (sinz * particle->m_LocalXYZ[1]);
																	particle->m_LocalXYZ[1] = (sinz * px) + (cosz * particle->m_LocalXYZ[1]);
																}

															particle->m_LocalXYZ[0] -= emittercoords[0];
															particle->m_LocalXYZ[1] -= emittercoords[1];
															particle->m_LocalXYZ[2] -= emittercoords[2];


															if(m_TwistRelative())
																{
																	twist[0] = particle->m_TwistXYZ[0][0] + m_Twistxlife->GetValue1(twisttime, m_DeltaTime) * particle->m_TwistXYZ[1][0];
																	twist[1] = particle->m_TwistXYZ[0][1] + m_Twistylife->GetValue1(twisttime, m_DeltaTime) * particle->m_TwistXYZ[1][1];
																	twist[2] = particle->m_TwistXYZ[0][2] + m_Twistzlife->GetValue1(twisttime, m_DeltaTime) * particle->m_TwistXYZ[1][2];
																}
															else
																{
																	twist[0] = m_Twistxlife->GetValue1(twisttime, m_DeltaTime) * (particle->m_TwistXYZ[1][0] - particle->m_TwistXYZ[0][0]) + particle->m_TwistXYZ[0][0];
																	twist[1] = m_Twistylife->GetValue1(twisttime, m_DeltaTime) * (particle->m_TwistXYZ[1][1] - particle->m_TwistXYZ[0][1]) + particle->m_TwistXYZ[0][1];
																	twist[2] = m_Twistzlife->GetValue1(twisttime, m_DeltaTime) * (particle->m_TwistXYZ[1][2] - particle->m_TwistXYZ[0][2]) + particle->m_TwistXYZ[0][2];
																}

															cosx = cos(twist[0]);
															sinx = sin(twist[0]);
															cosy = cos(twist[1]);
															siny = sin(twist[1]);
															cosz = cos(twist[2]);
															sinz = sin(twist[2]);



															if(m_SizeRelative())
																{
																	particlesize = m_Sizelife->GetValue1(sizetime, m_DeltaTime) * particle->m_Size[1] + particle->m_Size[0];
																}
															else
																{
																	particlesize = m_Sizelife->GetValue1(sizetime, m_DeltaTime) * (particle->m_Size[1] - particle->m_Size[0]) + particle->m_Size[0];
																}



															if(m_DriftRelative())
																{
																	drift[0] = particle->m_DriftXYZ[0][0] + m_Driftxlife->GetValue1(drifttime, m_DeltaTime) * particle->m_DriftXYZ[1][0] + emittercoords[0];
																	drift[1] = particle->m_DriftXYZ[0][1] + m_Driftylife->GetValue1(drifttime, m_DeltaTime) * particle->m_DriftXYZ[1][1] + emittercoords[1];
																	drift[2] = particle->m_DriftXYZ[0][2] + m_Driftzlife->GetValue1(drifttime, m_DeltaTime) * particle->m_DriftXYZ[1][2] + emittercoords[2];
																}
															else
																{
																	drift[0] = m_Driftxlife->GetValue1(drifttime, m_DeltaTime) * (particle->m_DriftXYZ[1][0] - particle->m_DriftXYZ[0][0]) + particle->m_DriftXYZ[0][0] + emittercoords[0];
																	drift[1] = m_Driftylife->GetValue1(drifttime, m_DeltaTime) * (particle->m_DriftXYZ[1][1] - particle->m_DriftXYZ[0][1]) + particle->m_DriftXYZ[0][1] + emittercoords[1];
																	drift[2] = m_Driftzlife->GetValue1(drifttime, m_DeltaTime) * (particle->m_DriftXYZ[1][2] - particle->m_DriftXYZ[0][2]) + particle->m_DriftXYZ[0][2] + emittercoords[2];
																}

															m_PolygonList->SetHandle(polygonclient->GetPolygonHandle());
															for(hr = m_Polygon->SetFirstPoint(); SDP_S_OK == hr; hr = m_Polygon->SetNextPoint())
																{
																	m_PolygonLocation->GetAnimXYZ(coords);

																	coords[0] = ((coords[0] - centercoords[0]) * particlesize) + centercoords[0];
																	coords[1] = ((coords[1] - centercoords[1]) * particlesize) + centercoords[1];
																	coords[2] = ((coords[2] - centercoords[2]) * particlesize) + centercoords[2];

																	coords[0] += particle->m_LocalXYZ[0];
																	coords[1] += particle->m_LocalXYZ[1];
																	coords[2] += particle->m_LocalXYZ[2];

																	py = coords[1];
																	coords[1] = (cosx * coords[1]) - (sinx * coords[2]);
																	coords[2] = (sinx * py) + (cosx * coords[2]);

																	px = coords[0];
																	coords[0] = (cosy * coords[0]) + (siny * coords[2]);
																	coords[2] = (cosy * coords[2]) - (siny * px);

																	px = coords[0];
																	coords[0] = (cosz * coords[0]) - (sinz * coords[1]);
																	coords[1] = (sinz * px) + (cosz * coords[1]);

																	coords[0] += drift[0];
																	coords[1] += drift[1];
																	coords[2] += drift[2];

																	m_PolygonLocation->SetAnimXYZ(coords);
																}
														}
													polygonclient->Release();
												}
										}
								}
						}
				}
*/
		}

	return SDP_S_OK;
}

void CParticleBurst::CreateParticles()
{
	double totalframes;
	double particlequantity, dsample, pcount;
	double i, j;

	// Start over from scratch
	m_Particles.clear();

	totalframes = m_TimeBase->GetDuration() * m_TimeBase->GetFrameRate();
	dsample = (1.0 / m_BirthRateSamples()) / (double)totalframes;

	particlequantity = 0.0;

	for(j = dsample; j < 1.0; j += dsample)
		{
			pcount = m_Birthrate->GetValue1(j, dsample * 0.1) * dsample;
			particlequantity += pcount;

			i = 0.0;
			for(; particlequantity >= 1.0; particlequantity -= 1.0)
				{
					CParticle particle;
					if(pcount)
						{
							particle.m_Time[0] = j + (i++) * (dsample / pcount);
						}
					else
						{
							particle.m_Time[0] = j;
						}
					m_Particles.push_back(particle);
				}
		}
}

void CParticleBurst::RecalculateParticles()
{
	double minlife, maxlife, dlife;
	double minstrength, minstrengthend, maxstrength, maxstrengthend, dstrength, dstrengthend;
	double minsize, minsizeend, maxsize, maxsizeend, dsize, dsizeend;
	double mindistance, mindistanceend, maxdistance, maxdistanceend, ddistance, ddistanceend;
	double mindriftx, mindriftxend, maxdriftx, maxdriftxend, ddriftx, ddriftxend;
	double mindrifty, mindriftyend, maxdrifty, maxdriftyend, ddrifty, ddriftyend;
	double mindriftz, mindriftzend, maxdriftz, maxdriftzend, ddriftz, ddriftzend;
	double mindirectionx, maxdirectionx, ddirectionx;
	double mindirectiony, maxdirectiony, ddirectiony;
	double mindirectionz, maxdirectionz, ddirectionz;
	double mintwistx, mintwistxend, maxtwistx, maxtwistxend, dtwistx, dtwistxend;
	double mintwisty, mintwistyend, maxtwisty, maxtwistyend, dtwisty, dtwistyend;
	double mintwistz, mintwistzend, maxtwistz, maxtwistzend, dtwistz, dtwistzend;
	double minwidth, minwidthend, maxwidth, maxwidthend, dwidth, dwidthend;
	double minheight, minheightend, maxheight, maxheightend, dheight, dheightend;
	double minangle, minangleend, maxangle, maxangleend, dangle, dangleend;

	double i, quantity, fraction;

	i = 0.0;
	quantity = m_Particles.size();

	// Reset the seed
	gulong seed = (gulong)m_RandomSeed();

	// For each particle ...
	ParticleIterator end = m_Particles.end();
	for(ParticleIterator particle = m_Particles.begin(); particle != end; particle++)
		{
			fraction = i / quantity;

			// Find min / max values
			minlife = m_Minlife->GetValue1(fraction, m_DeltaTime);
			maxlife = m_Maxlife->GetValue1(fraction, m_DeltaTime);
			dlife = maxlife - minlife;
			minstrength = m_Minstrength->GetValue1(fraction, m_DeltaTime);
			maxstrength = m_Maxstrength->GetValue1(fraction, m_DeltaTime);
			dstrength = maxstrength - minstrength;
			minstrengthend = m_Minstrengthend->GetValue1(fraction, m_DeltaTime);
			maxstrengthend = m_Maxstrengthend->GetValue1(fraction, m_DeltaTime);
			dstrengthend = maxstrengthend - minstrengthend;
			minsize = m_Minsize->GetValue1(fraction, m_DeltaTime);
			maxsize = m_Maxsize->GetValue1(fraction, m_DeltaTime);
			dsize = maxsize - minsize;
			minsizeend = m_Minsizeend->GetValue1(fraction, m_DeltaTime);
			maxsizeend = m_Maxsizeend->GetValue1(fraction, m_DeltaTime);
			dsizeend = maxsizeend - minsizeend;
			mindistance = m_Mindistance->GetValue1(fraction, m_DeltaTime);
			maxdistance = m_Maxdistance->GetValue1(fraction, m_DeltaTime);
			ddistance = maxdistance - mindistance;
			mindistanceend = m_Mindistanceend->GetValue1(fraction, m_DeltaTime);
			maxdistanceend = m_Maxdistanceend->GetValue1(fraction, m_DeltaTime);
			ddistanceend = maxdistanceend - mindistanceend;
			mindriftx = m_Mindriftx->GetValue1(fraction, m_DeltaTime);
			maxdriftx = m_Maxdriftx->GetValue1(fraction, m_DeltaTime);
			ddriftx = maxdriftx - mindriftx;
			mindriftxend = m_Mindriftxend->GetValue1(fraction, m_DeltaTime);
			maxdriftxend = m_Maxdriftxend->GetValue1(fraction, m_DeltaTime);
			ddriftxend = maxdriftxend - mindriftxend;
			mindrifty = m_Mindrifty->GetValue1(fraction, m_DeltaTime);
			maxdrifty = m_Maxdrifty->GetValue1(fraction, m_DeltaTime);
			ddrifty = maxdrifty - mindrifty;
			mindriftyend = m_Mindriftyend->GetValue1(fraction, m_DeltaTime);
			maxdriftyend = m_Maxdriftyend->GetValue1(fraction, m_DeltaTime);
			ddriftyend = maxdriftyend - mindriftyend;
			mindriftz = m_Mindriftz->GetValue1(fraction, m_DeltaTime);
			maxdriftz = m_Maxdriftz->GetValue1(fraction, m_DeltaTime);
			ddriftz = maxdriftz - mindriftz;
			mindriftzend = m_Mindriftzend->GetValue1(fraction, m_DeltaTime);
			maxdriftzend = m_Maxdriftzend->GetValue1(fraction, m_DeltaTime);
			ddriftzend = maxdriftzend - mindriftzend;
			mindirectionx = m_Mindirectionx->GetValue1(fraction, m_DeltaTime);
			maxdirectionx = m_Maxdirectionx->GetValue1(fraction, m_DeltaTime);
			ddirectionx = maxdirectionx - mindirectionx;
			mindirectiony = m_Mindirectiony->GetValue1(fraction, m_DeltaTime);
			maxdirectiony = m_Maxdirectiony->GetValue1(fraction, m_DeltaTime);
			ddirectiony = maxdirectiony - mindirectiony;
			mindirectionz = m_Mindirectionz->GetValue1(fraction, m_DeltaTime);
			maxdirectionz = m_Maxdirectionz->GetValue1(fraction, m_DeltaTime);
			ddirectionz = maxdirectionz - mindirectionz;
			mintwistx = m_Mintwistx->GetValue1(fraction, m_DeltaTime);
			maxtwistx = m_Maxtwistx->GetValue1(fraction, m_DeltaTime);
			dtwistx = maxtwistx - mintwistx;
			mintwistxend = m_Mintwistxend->GetValue1(fraction, m_DeltaTime);
			maxtwistxend = m_Maxtwistxend->GetValue1(fraction, m_DeltaTime);
			dtwistxend = maxtwistxend - mintwistxend;
			mintwisty = m_Mintwisty->GetValue1(fraction, m_DeltaTime);
			maxtwisty = m_Maxtwisty->GetValue1(fraction, m_DeltaTime);
			dtwisty = maxtwisty - mintwisty;
			mintwistyend = m_Mintwistyend->GetValue1(fraction, m_DeltaTime);
			maxtwistyend = m_Maxtwistyend->GetValue1(fraction, m_DeltaTime);
			dtwistyend = maxtwistyend - mintwistyend;
			mintwistz = m_Mintwistz->GetValue1(fraction, m_DeltaTime);
			maxtwistz = m_Maxtwistz->GetValue1(fraction, m_DeltaTime);
			dtwistz = maxtwistz - mintwistz;
			mintwistzend = m_Mintwistzend->GetValue1(fraction, m_DeltaTime);
			maxtwistzend = m_Maxtwistzend->GetValue1(fraction, m_DeltaTime);
			dtwistzend = maxtwistzend - mintwistzend;
			minwidth = m_Minwidth->GetValue1(fraction, m_DeltaTime);
			maxwidth = m_Maxwidth->GetValue1(fraction, m_DeltaTime);
			dwidth = maxwidth - minwidth;
			minwidthend = m_Minwidthend->GetValue1(fraction, m_DeltaTime);
			maxwidthend = m_Maxwidthend->GetValue1(fraction, m_DeltaTime);
			dwidthend = maxwidthend - minwidthend;
			minheight = m_Minheight->GetValue1(fraction, m_DeltaTime);
			maxheight = m_Maxheight->GetValue1(fraction, m_DeltaTime);
			dheight = maxheight - minheight;
			minheightend = m_Minheightend->GetValue1(fraction, m_DeltaTime);
			maxheightend = m_Maxheightend->GetValue1(fraction, m_DeltaTime);
			dheightend = maxheightend - minheightend;
			minangle = m_Minangle->GetValue1(fraction, m_DeltaTime);
			maxangle = m_Maxangle->GetValue1(fraction, m_DeltaTime);
			dangle = maxangle - minangle;
			minangleend = m_Minangleend->GetValue1(fraction, m_DeltaTime);
			maxangleend = m_Maxangleend->GetValue1(fraction, m_DeltaTime);
			dangleend = maxangleend - minangleend;

			// Calculate particle values
			particle->m_Time[1] = particle->m_Time[0] + dlife * (DRand(seed)) + minlife;

			if(m_DistanceRandom())
				{
					particle->m_Distance[0] = ddistance * (DRand(seed)) + mindistance;
					particle->m_Distance[1] = ddistanceend * (DRand(seed)) + mindistanceend;
				}
			else
				{
					particle->m_Distance[0] = ddistance * fraction + mindistance;
					particle->m_Distance[1] = ddistanceend * fraction + mindistanceend;
				}

			if(m_DriftRandom())
				{
					particle->m_DriftXYZ[0][0] = ddriftx * (DRand(seed)) + mindriftx;
					particle->m_DriftXYZ[1][0] = ddriftxend * (DRand(seed)) + mindriftxend;
					particle->m_DriftXYZ[0][1] = ddrifty * (DRand(seed)) + mindrifty;
					particle->m_DriftXYZ[1][1] = ddriftyend * (DRand(seed)) + mindriftyend;
					particle->m_DriftXYZ[0][2] = ddriftz * (DRand(seed)) + mindriftz;
					particle->m_DriftXYZ[1][2] = ddriftzend * (DRand(seed)) + mindriftzend;
				}
			else
				{
					particle->m_DriftXYZ[0][0] = ddriftx * fraction + mindriftx;
					particle->m_DriftXYZ[1][0] = ddriftxend * fraction + mindriftxend;
					particle->m_DriftXYZ[0][1] = ddrifty * fraction + mindrifty;
					particle->m_DriftXYZ[1][1] = ddriftyend * fraction + mindriftyend;
					particle->m_DriftXYZ[0][2] = ddriftz * fraction + mindriftz;
					particle->m_DriftXYZ[1][2] = ddriftzend * fraction + mindriftzend;
				}

			if(m_DirectionRandom())
				{
					particle->m_DirectionXYZ[0] = ddirectionx * (DRand(seed)) + mindirectionx;
					particle->m_DirectionXYZ[1] = ddirectiony * (DRand(seed)) + mindirectiony;
					particle->m_DirectionXYZ[2] = ddirectionz * (DRand(seed)) + mindirectionz;
				}
			else
				{
					particle->m_DirectionXYZ[0] = ddirectionx * fraction + mindirectionx;
					particle->m_DirectionXYZ[1] = ddirectiony * fraction + mindirectiony;
					particle->m_DirectionXYZ[2] = ddirectionz * fraction + mindirectionz;
				}

			if(m_TwistRandom())
				{
					particle->m_TwistXYZ[0][0] = dtwistx * (DRand(seed)) + mintwistx;
					particle->m_TwistXYZ[1][0] = dtwistxend * (DRand(seed)) + mintwistxend;
					particle->m_TwistXYZ[0][1] = dtwisty * (DRand(seed)) + mintwisty;
					particle->m_TwistXYZ[1][1] = dtwistyend * (DRand(seed)) + mintwistyend;
					particle->m_TwistXYZ[0][2] = dtwistz * (DRand(seed)) + mintwistz;
					particle->m_TwistXYZ[1][2] = dtwistzend * (DRand(seed)) + mintwistzend;
				}
			else
				{
					particle->m_TwistXYZ[0][0] = dtwistx * fraction + mintwistx;
					particle->m_TwistXYZ[1][0] = dtwistxend * fraction + mintwistxend;
					particle->m_TwistXYZ[0][1] = dtwisty * fraction + mintwisty;
					particle->m_TwistXYZ[1][1] = dtwistyend * fraction + mintwistyend;
					particle->m_TwistXYZ[0][2] = dtwistz * fraction + mintwistz;
					particle->m_TwistXYZ[1][2] = dtwistzend * fraction + mintwistzend;
				}

			if(m_StrengthRandom())
				{
					particle->m_Strength[0] = dstrength * (DRand(seed)) + minstrength;
					particle->m_Strength[1] = dstrengthend * (DRand(seed)) + minstrengthend;
				}
			else
				{
					particle->m_Strength[0] = dstrength * fraction + minstrength;
					particle->m_Strength[1] = dstrengthend * fraction + minstrengthend;
				}

			if(m_SizeRandom())
				{
					particle->m_Size[0] = dsize * (DRand(seed)) + minsize;
					particle->m_Size[1] = dsizeend * (DRand(seed)) + minsizeend;
				}
			else
				{
					particle->m_Size[0] = dsize * fraction + minsize;
					particle->m_Size[1] = dsizeend * fraction + minsizeend;
				}

			if(m_WidthRandom())
				{
					particle->m_Width[0] = dwidth * (DRand(seed)) + minwidth;
					particle->m_Width[1] = dwidthend * (DRand(seed)) + minwidthend;
				}
			else
				{
					particle->m_Width[0] = dwidth * fraction + minwidth;
					particle->m_Width[1] = dwidthend * fraction + minwidthend;
				}

			if(m_HeightRandom())
				{
					particle->m_Height[0] = dheight * (DRand(seed)) + minheight;
					particle->m_Height[1] = dheightend * (DRand(seed)) + minheightend;
				}
			else
				{
					particle->m_Height[0] = dheight * fraction + minheight;
					particle->m_Height[1] = dheightend * fraction + minheightend;
				}

			if(m_AngleRandom())
				{
					particle->m_Angle[0] = dangle * (DRand(seed)) + minangle;
					particle->m_Angle[1] = dangleend * (DRand(seed)) + minangleend;
				}
			else
				{
					particle->m_Angle[0] = dangle * fraction + minangle;
					particle->m_Angle[1] = dangleend * fraction + minangleend;
				}

			m_Color->GetValue3(DRand(seed), m_DeltaTime, particle->m_Color);

			i++;
		}
}

void CParticleBurst::RecalculateEmitterPosition()
{
	// Disable k3dIAnimate calculations
	m_Animate = FALSE;

	double totalframes, dsample, j;

	sdpVector3 emittercoords;

	if(0.0 == m_EmitterPositionSamples()) // Calculate particle positions exactly at birth for each particle
		{
			ParticleIterator end = m_Particles.end();
			for(ParticleIterator particle = m_Particles.begin(); particle != end; particle++)
				{
					m_TimeBase->SetRenderTime(particle->m_Time[0]);
					Document()->AnimateDocument();

//					m_Emitter->GetTargetCoordinates(emittercoords);

					particle->m_BirthXYZ = emittercoords;
				}
		}
	else // Calculate particle positions at some multiple of frame rate
		{
			totalframes = m_TimeBase->GetDuration() * m_TimeBase->GetFrameRate();
			dsample = (1.0 / m_EmitterPositionSamples()) / (double)totalframes;

			ParticleIterator particle = m_Particles.begin();
			for(j = dsample; j < 1.0; j += dsample)
				{
					m_TimeBase->SetRenderTime(j - dsample);
					Document()->AnimateDocument();

//					m_Emitter->GetTargetCoordinates(emittercoords);

					while(particle != m_Particles.end())
						{
							if(particle->m_Time[0] < j)
								{
									particle->m_BirthXYZ = emittercoords;
								}
							else
								{
									break;
								}

							particle++;
						}

					if(!particle) break;
				}
		}

	// Enable k3dIAnimate calculations
	m_Animate = TRUE;
}

void CParticleBurst::UpdateFull()
{
	if(POINTS == m_ParticleType() || FLARES == m_ParticleType())
		{
//			CParticleBurstUpdateDlg dlg;
//			dlg.Create(CParticleBurstUpdateDlg::IDD);
			CreateParticles();
			RecalculateParticles();
			RecalculateEmitterPosition();
		}
	else // Polygons or meshes type
		{
			m_Particles.clear();
		}
}

void CParticleBurst::UpdateCalculateParticles()
{
	if(POINTS == m_ParticleType() || FLARES == m_ParticleType())
		{
//			CParticleBurstUpdateDlg dlg;
//			dlg.Create(CParticleBurstUpdateDlg::IDD);

			RecalculateParticles();
		}
	else // Polygons or meshes type
		{
			m_Particles.clear();
		}
}

void CParticleBurst::UpdateCreateParticles()
{
	if(POINTS == m_ParticleType() || FLARES == m_ParticleType())
		{
//			CParticleBurstUpdateDlg dlg;
//			dlg.Create(CParticleBurstUpdateDlg::IDD);

			CreateParticles();
			RecalculateParticles();
		}
	else // Polygons or meshes type
		{
			m_Particles.clear();
		}
}

/////////////////////////////////////////////////////////////////////////////
// CParticleBurstDialog

CParticleBurstDialog::CParticleBurstDialog()
{
	m_Object = 0;
}

bool CParticleBurstDialog::Create(CParticleBurst* Object)
{
	// Store our parent object ...
	m_Object = Object;

	// Load our template ...
	g_return_val_if_fail(k3dDialog::Create(Object, false, true, "particleburst.gtkml", "particleburst.html"), false);

	return true;
}

bool CParticleBurstDialog::InitializeControls()
{
	// Strength buttons
	ConnectChannelButton(m_Object->m_Minstrength.Interface(), "minstrength");
	ConnectChannelButton(m_Object->m_Minstrengthend.Interface(), "minstrengthend");
	ConnectChannelButton(m_Object->m_Maxstrength.Interface(), "maxstrength");
	ConnectChannelButton(m_Object->m_Maxstrengthend.Interface(), "maxstrengthend");
	ConnectChannelButton(m_Object->m_Strengthlife.Interface(), "strengthlife");
	ConnectChannelButton(m_Object->m_Color.Interface(), "color");

	// Size buttons
	ConnectChannelButton(m_Object->m_Minsize.Interface(), "minsize");
	ConnectChannelButton(m_Object->m_Minsizeend.Interface(), "minsizeend");
	ConnectChannelButton(m_Object->m_Maxsize.Interface(), "maxsize");
	ConnectChannelButton(m_Object->m_Maxsizeend.Interface(), "maxsizeend");
	ConnectChannelButton(m_Object->m_Sizelife.Interface(), "sizelife");

	// Direction buttons
	ConnectChannelButton(m_Object->m_Mindirectionx.Interface(), "mindirectionx");
	ConnectChannelButton(m_Object->m_Maxdirectionx.Interface(), "maxdirectionx");
	ConnectChannelButton(m_Object->m_Mindirectiony.Interface(), "mindirectiony");
	ConnectChannelButton(m_Object->m_Maxdirectiony.Interface(), "maxdirectiony");
	ConnectChannelButton(m_Object->m_Mindirectionz.Interface(), "mindirectionz");
	ConnectChannelButton(m_Object->m_Maxdirectionz.Interface(), "maxdirectionz");

	// Distance buttons
	ConnectChannelButton(m_Object->m_Mindistance.Interface(), "mindistance");
	ConnectChannelButton(m_Object->m_Mindistanceend.Interface(), "mindistanceend");
	ConnectChannelButton(m_Object->m_Maxdistance.Interface(), "maxdistance");
	ConnectChannelButton(m_Object->m_Maxdistanceend.Interface(), "maxdistanceend");
	ConnectChannelButton(m_Object->m_Distancelife.Interface(), "distancelife");

	// Setup drift buttons ...
	ConnectChannelButton(m_Object->m_Mindriftx.Interface(), "mindriftx");
	ConnectChannelButton(m_Object->m_Mindriftxend.Interface(), "mindriftxend");
	ConnectChannelButton(m_Object->m_Maxdriftx.Interface(), "maxdriftx");
	ConnectChannelButton(m_Object->m_Maxdriftxend.Interface(), "maxdriftxend");
	ConnectChannelButton(m_Object->m_Mindrifty.Interface(), "mindrifty");
	ConnectChannelButton(m_Object->m_Mindriftyend.Interface(), "mindriftyend");
	ConnectChannelButton(m_Object->m_Maxdrifty.Interface(), "maxdrifty");
	ConnectChannelButton(m_Object->m_Maxdriftyend.Interface(), "maxdriftyend");
	ConnectChannelButton(m_Object->m_Mindriftz.Interface(), "mindriftz");
	ConnectChannelButton(m_Object->m_Mindriftzend.Interface(), "mindriftzend");
	ConnectChannelButton(m_Object->m_Maxdriftz.Interface(), "maxdriftz");
	ConnectChannelButton(m_Object->m_Maxdriftzend.Interface(), "maxdriftzend");

	ConnectChannelButton(m_Object->m_Driftxlife.Interface(), "driftxlife");
	ConnectChannelButton(m_Object->m_Driftylife.Interface(), "driftylife");
	ConnectChannelButton(m_Object->m_Driftzlife.Interface(), "driftzlife");

	// Setup twist buttons ...
	ConnectChannelButton(m_Object->m_Mintwistx.Interface(), "mintwistx");
	ConnectChannelButton(m_Object->m_Mintwistxend.Interface(), "mintwistxend");
	ConnectChannelButton(m_Object->m_Maxtwistx.Interface(), "maxtwistx");
	ConnectChannelButton(m_Object->m_Maxtwistxend.Interface(), "maxtwistxend");
	ConnectChannelButton(m_Object->m_Mintwisty.Interface(), "mintwisty");
	ConnectChannelButton(m_Object->m_Mintwistyend.Interface(), "mintwistyend");
	ConnectChannelButton(m_Object->m_Maxtwisty.Interface(), "maxtwisty");
	ConnectChannelButton(m_Object->m_Maxtwistyend.Interface(), "maxtwistyend");
	ConnectChannelButton(m_Object->m_Mintwistz.Interface(), "mintwistz");
	ConnectChannelButton(m_Object->m_Mintwistzend.Interface(), "mintwistzend");
	ConnectChannelButton(m_Object->m_Maxtwistz.Interface(), "maxtwistz");
	ConnectChannelButton(m_Object->m_Maxtwistzend.Interface(), "maxtwistzend");

	ConnectChannelButton(m_Object->m_Twistxlife.Interface(), "twistxlife");
	ConnectChannelButton(m_Object->m_Twistylife.Interface(), "twistylife");
	ConnectChannelButton(m_Object->m_Twistzlife.Interface(), "twistzlife");

	// Setup flare buttons ...
	ConnectChannelButton(m_Object->m_Minwidth.Interface(), "minwidth");
	ConnectChannelButton(m_Object->m_Minwidthend.Interface(), "minwidthend");
	ConnectChannelButton(m_Object->m_Maxwidth.Interface(), "maxwidth");
	ConnectChannelButton(m_Object->m_Maxwidthend.Interface(), "maxwidthend");
	ConnectChannelButton(m_Object->m_Minheight.Interface(), "minheight");
	ConnectChannelButton(m_Object->m_Minheightend.Interface(), "minheightend");
	ConnectChannelButton(m_Object->m_Maxheight.Interface(), "maxheight");
	ConnectChannelButton(m_Object->m_Maxheightend.Interface(), "maxheightend");
	ConnectChannelButton(m_Object->m_Minangle.Interface(), "minangle");
	ConnectChannelButton(m_Object->m_Minangleend.Interface(), "minangleend");
	ConnectChannelButton(m_Object->m_Maxangle.Interface(), "maxangle");
	ConnectChannelButton(m_Object->m_Maxangleend.Interface(), "maxangleend");

	ConnectChannelButton(m_Object->m_Widthlife.Interface(), "widthlife");
	ConnectChannelButton(m_Object->m_Heightlife.Interface(), "heightlife");
	ConnectChannelButton(m_Object->m_Anglelife.Interface(), "anglelife");

	return true;
}

void CParticleBurstDialog::UpdateControls()
{
	// Setup editor options ...
//	ToggleButton("castshadows").SetState(m_Object->m_CastShadows());
}

void CParticleBurstDialog::OnEvent(sdpGtkEvent* Event)
{
	// Sanity checks ...
	g_assert(Event);

/*
	if(Event->Name() == "oncastshadows")
		OnCastShadows();
	else if(Event->Name() == "onshader")
		OnShader();
	else
*/	
		k3dDialog::OnEvent(Event);
}

/*
void CParticleBurstDialog::OnCastShadows()
{
	m_Object->m_CastShadows() = ToggleButton("castshadows").GetState();
	m_Object->OnModify(false);
}

void CParticleBurstDialog::OnShader()
{
	// Prompt the user for a new shader ...
	k3dShaderSelector selector(m_Object->m_Application, false, k3dIShaderInfo::LIGHT);
	selector.Create();
	selector.DoModal();

	// If the user didn't cancel ...
	if(selector.OK())
		{
			// If the user chose the same shader, do nothing ...
			if(m_Object->m_Shader.ShaderHandle() == selector.ShaderHandle())
				return;

			// Set the new shader ...
			m_Object->m_Shader.SetShaderHandle(selector.ShaderHandle());

			// Update our button label ...
			Label("shader").SetText(m_Object->m_Shader.ShaderName());

			// Update our arguments ...
			m_ShaderPage.Create(m_Object->m_Shader, VBox("shadercontainer"));
		}
}
*/

