// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the CParticleEmitter class, which emits CParticleBurst particles
		\author Tim Shead (tshead@k-3d.com)
*/

#include "particleemitter.h"

#include <sdpgl.h>

///////////////////////////////////////////////////////////////
// CParticleEmitter

CParticleEmitter::CParticleEmitter()
{
	ReserveLocations(1);
}

///////////////////////////////////////////////////////////////
// k3dIUnknown overrides

sdpResult CParticleEmitter::QueryInterface(const sdpIID& InterfaceID, void** Interface)
{
	if(sdpSupportInterface<k3dIAnimate>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dIEditorRender>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dIEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dILocationServer>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dIObject>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dIPersist>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dITarget>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dITimeServer>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dITransform>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<k3dIUserEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;										
	if(sdpSupportInterface<sdpIUnknown>(static_cast<k3dIObject*>(this), InterfaceID, Interface)) return SDP_S_OK;										

	return SDP_E_NO_INTERFACE;
}

gulong CParticleEmitter::AddRef()
{
	return sdpComAddRef(m_ReferenceCount);
}

gulong CParticleEmitter::Release()
{
	return sdpComRelease(this, m_ReferenceCount);
}

///////////////////////////////////////////////////////////////
// k3dIObject overrides

sdpResult CParticleEmitter::QueryRenameObject()
{
	// We don't allow ourselves to be renamed because our name is determined
	// by the name of our owner (the particle burst) ...
	return SDP_S_FALSE;
}

sdpResult CParticleEmitter::QueryDeleteObject()
{
	// We don't allow ourselves to be deleted because our lifetime should
	// match our owner's (the particle burst) ...
	return SDP_S_FALSE;
}

///////////////////////////////////////////////////////////////
// k3dIEditorRender overrides

sdpResult CParticleEmitter::EditorRender(k3dIEditorEngine* Engine)
{
	glDisable(GL_LIGHTING);
	glDisable(GL_TEXTURE_1D);
	glDisable(GL_TEXTURE_2D);

	sdpVector3 coords = m_Locations[0].m_AnimXYZ;

	// Draw our little wand ...
	glColor3d(1.0, 1.0, 0.0);

	glBegin(GL_LINES);
	glVertex3dv(coords);
	glVertex3d(coords[0], coords[1] - 3.0, coords[2]);
	glEnd();

	// Draw our little sunburst ...
	glColor3d(1.0, 1.0, 0.0);

	glBegin(GL_LINES);

	glVertex3d(coords[0] + 0.7, coords[1], coords[2]);
	glVertex3d(coords[0] - 0.7, coords[1], coords[2]);
	glVertex3d(coords[0], coords[1] + 0.7, coords[2]);
	glVertex3d(coords[0], coords[1] - 0.0, coords[2]);	// Avoid covering the wand handle
	glVertex3d(coords[0], coords[1], coords[2] + 0.7);
	glVertex3d(coords[0], coords[1], coords[2] - 0.7);

	glVertex3d(coords[0] + 0.3, coords[1] + 0.3, coords[2] + 0.3);
	glVertex3d(coords[0] - 0.3, coords[1] - 0.3, coords[2] - 0.3);
	glVertex3d(coords[0] - 0.3, coords[1] + 0.3, coords[2] + 0.3);
	glVertex3d(coords[0] + 0.3, coords[1] - 0.3, coords[2] - 0.3);
	glVertex3d(coords[0] + 0.3, coords[1] + 0.3, coords[2] - 0.3);
	glVertex3d(coords[0] - 0.3, coords[1] - 0.3, coords[2] + 0.3);
	glVertex3d(coords[0] - 0.3, coords[1] + 0.3, coords[2] - 0.3);
	glVertex3d(coords[0] + 0.3, coords[1] - 0.3, coords[2] + 0.3);

	glEnd();

	DrawTransformationHandles(1.0);

	return SDP_S_OK;
}

///////////////////////////////////////////////////////////////
// k3dIPersist overrides

sdpResult CParticleEmitter::Save(sdpXmlIElement* Element, k3dIDependencies* Dependencies)
{
	// Sanity checks ...
	g_assert(Element);
	g_assert(Dependencies);

	g_return_val_if_fail(sdpSucceeded(k3dTransformObject::Save(Element, Dependencies)), SDP_E_FAIL);
	g_return_val_if_fail(sdpSucceeded(k3dLocationServer::Save(Element, Dependencies)), SDP_E_FAIL);

	return SDP_S_OK;
}

sdpResult CParticleEmitter::Load(sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(Element);

	g_return_val_if_fail(sdpSucceeded(k3dTransformObject::Load(Element)), SDP_E_FAIL);
	g_return_val_if_fail(sdpSucceeded(k3dLocationServer::Load(Element)), SDP_E_FAIL);

	return SDP_S_OK;
}



