#ifndef LISSACURVE_H
#define LISSACURVE_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CLissaCurve K-3D tool, which generates geometry composed of Lissajous curves
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dprimtool.h>
#include <k3dprimtooldialog.h>

// Forward class references
class CLissaCurve;

/////////////////////////////////////////////////////////////////////////////
// CLissaCurveDialog

class CLissaCurveDialog : public k3dPrimToolDialog
{
public:
	CLissaCurveDialog();

	bool Create(CLissaCurve* Object);

protected:
	CLissaCurve* m_Object;

	// Event handling ...	
	void OnEvent(sdpGtkEvent* Event);

	void OnEdgeCount();
	void OnXFreq();
	void OnXAmp();
	void OnXPhase();
	void OnYFreq();
	void OnYAmp();
	void OnYPhase();
	void OnZFreq();
	void OnZAmp();
	void OnZPhase();
	void OnMFreq();
	void OnMAmp();
	void OnMPhase();

	// Update controls ...
	void UpdateControls();
};

/////////////////////////////////////////////////////////////////////////////
// CLissaCurve

class CLissaCurve : public k3dPrimTool
{
friend class CLissaCurveDialog;
public:
	CLissaCurve();

	CLissaCurveDialog m_UserInterface;

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIObject overrides
	sdpResult EditObject();
	sdpResult ObjectDeleted();

	// k3dIUserEventClient overrides
//	sdpResult OnLButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
//	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
//	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);

//	KObjectContainer<&sdpChannel, &IID_k3dIChannel, k3dIChannel> m_Mwaveform;
//	KObjectContainer<&sdpChannel, &IID_k3dIChannel, k3dIChannel> m_Xwaveform;
//	KObjectContainer<&sdpChannel, &IID_k3dIChannel, k3dIChannel> m_Ywaveform;
//	KObjectContainer<&sdpChannel, &IID_k3dIChannel, k3dIChannel> m_Zwaveform;

	gulong	m_EdgeCount;

	double	m_MAmp;
	double	m_MFreq;
	double	m_MPhase;
	bool		m_CustomM;
	double	m_XAmp;
	double	m_XFreq;
	double	m_XPhase;
	bool		m_CustomX;
	double	m_YAmp;
	double	m_YFreq;
	double	m_YPhase;
	bool		m_CustomY;
	double	m_ZAmp;
	double	m_ZFreq;
	double	m_ZPhase;
	bool		m_CustomZ;

	void OnModify(bool SaveAction);
};



#endif // LISSACURVE_H
