#ifndef SMOOTH_H
#define SMOOTH_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CSmooth K-3D tool, which "smooths" geometry by averaging point locations
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dtool.h>
#include <k3ddialog.h>

// Forward class references
class CSmooth;

/////////////////////////////////////////////////////////////////////////////
// CSmoothDialog

class CSmoothDialog : public k3dDialog
{
friend class CSmooth;
public:
	CSmoothDialog();

	bool Create(CSmooth* Object);

protected:
	CSmooth* m_Object;

	// Update controls ...
	void UpdateControls();

	// Event handling ...	
	void OnEvent(sdpGtkEvent* Event);
	void OnSmoothingFactor();
};

/////////////////////////////////////////////////////////////////////////////
// CSmooth

class CSmooth : public k3dTool
{
friend class CSmoothDialog;
public:
	CSmooth();

	CSmoothDialog m_UserInterface;

	// k3dTool overrides ...
	void OnModify(bool SaveAction);

	// k3dIUnknown overrides ...
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIObject overrides ...
	sdpResult EditObject();
	sdpResult ObjectDeleted();
	
	// k3dIUserEventClient overrides
	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);

protected:
	gdouble m_SmoothingFactor;
};


#endif // SMOOTH_H
