#ifndef SPHEREIZE_H
#define SPHEREIZE_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CSphereize K-3D tool, which "sphereizes" objects by making them conform to the surface of a sphere
		\author Timothy M. Shead (tshead@k-3d.com)
		\author Romain Behar (romainbehar@yahoo.com)
*/

#include <k3dtool.h>
#include <k3ddialog.h>

// Forward class references
class CSphereize;

/////////////////////////////////////////////////////////////////////////////
// CSphereizeDialog

class CSphereizeDialog : public k3dDialog
{
friend class CSphereize;
public:
	CSphereizeDialog();

	bool Create(CSphereize* Object);

	// Update controls ...
	void UpdateControls(); // protected ?

protected:
	CSphereize* m_Object;

	// Event handling ...
	void OnEvent(sdpGtkEvent* Event);
	void OnSphereFactor();
};

/////////////////////////////////////////////////////////////////////////////
// CSphereize

class CSphereize : public k3dTool
{
friend class CSphereizeDialog;
public:
	CSphereize();

	CSphereizeDialog m_UserInterface;

	// k3dTool overrides ...
	void OnModify(bool SaveAction);
	void ModifyObject(k3dIObject* Object); //?
	void ModifyLocationServer(k3dIObject* Object); //?
	void ModifyLocation(k3dILocation* Location); //?

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	//k3dIObject overrides
	sdpResult EditObject();
	sdpResult ObjectDeleted();

	//k3dIUserEventClient overrides
	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);

	gdouble m_SphereizeFactor;

};


#endif // SPHEREIZE_H
