#ifndef TRIANGULATE_H
#define TRIANGULATE_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares CTriangulate, a K-3D tool for tessellating polygons
		\author Tim Shead (tshead@k-3d.com)
		\author Romain Behar (romainbehar@yahoo.com)
*/

#include <k3dprimtool.h>
#include <k3dprimtooldialog.h>

// Forward class references
class CTriangulate;

/////////////////////////////////////////////////////////////////////////////
// CTriangulateDialog

/// Provides the user interface for the Triangulate tool
class CTriangulateDialog : public k3dPrimToolDialog
{
friend class CTriangulate;
public:
	CTriangulateDialog();

	/// Called to create the UI
	bool Create(CTriangulate* Object);

protected:
	/// Stores a pointer to the owning Triangulate object
	CTriangulate* m_Object;
};


/////////////////////////////////////////////////////////////////////////////
// CTriangulate

class CTriangulate : public k3dPrimTool
{
friend class CTriangulateDialog;
public:
	CTriangulate();

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dTool overrides ...
	void OnModify(bool SaveAction);

	//k3dIObject overrides
	sdpResult EditObject();
	sdpResult ObjectDeleted();

protected:
	CTriangulateDialog m_UserInterface;
};


#endif // TRIANGULATE_H
