#ifndef UVMAP_H
#define UVMAP_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Defines the CUVMap K-3D tool, which is used to modify the texture-mapping coordinates of geometry
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <k3dtool.h>
#include <k3ddialog.h>

// Forward class declarations
class CUVMap;

/////////////////////////////////////////////////////////////////////////////
// CUVMapDialog

class CUVMapDialog : public k3dDialog
{
public:
	CUVMapDialog();

	bool Create(CUVMap* Object);

	// Update controls ...
	void UpdateControls();

protected:
	CUVMap* m_Object;

	// Event handling ...	
	void OnEvent(sdpGtkEvent* Event);

	void OnProjection();
	void OnCylindrical();
	void OnSpherical();
	void OnCubic();
	void OnConic();
	void OnPolygon();
	void OnImplicit();
	void OnApply();
};

/////////////////////////////////////////////////////////////////////////////
// CUVMap

class CUVMap : public k3dTool
{
friend class CUVMapDialog;
public:
	CUVMap();

	CUVMapDialog m_UserInterface;

	// k3dTool overrides ...
	void OnModify(bool SaveAction);

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIObject overrides
	sdpResult EditObject();
	sdpResult ObjectDeleted();
	
	typedef enum
	{
		PROJECTION,
		CYLINDRICAL,
		SPHERICAL,
		CUBIC,
		CONIC,
		POLYGON,
		IMPLICIT,
	} Type;

	Type m_Type;
};


#endif // UVMAP_H
