#include "stdafx.h"

#include "tlbtext.h"
#include "toollib.h"




CTextPage::CTextPage(CTextSheet* Sheet)
	: KPage(IDD, Sheet, "Text")
{
	//{{AFX_DATA_INIT(CTextPage)
	//}}AFX_DATA_INIT

	m_Sheet = Sheet;

}





BEGIN_MESSAGE_MAP(CTextPage, KPage)
	//{{AFX_MSG_MAP(CTextPage)
	ON_WM_DESTROY()
	ON_EN_CHANGE(IDC_TEXT, OnChangeText)
	ON_EN_k3dILLFOCUS(IDC_SCALEX, OnKillfocusScalex)
	ON_EN_k3dILLFOCUS(IDC_SCALEY, OnKillfocusScaley)
	ON_EN_k3dILLFOCUS(IDC_SLANTX, OnKillfocusSlantx)
	ON_EN_k3dILLFOCUS(IDC_SLANTY, OnKillfocusSlanty)
	ON_EN_k3dILLFOCUS(IDC_CURVESUBDIVISION, OnKillfocusCurvesubdivision)
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_CHOOSEFONT, OnChoosefont)

END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CTextPage message handlers





BOOL CTextPage::OnInitDialog()
{
	KPage::OnInitDialog();

	// Create a default font.
	GetObject(GetStockObject(SYSTEM_FONT), sizeof(m_Sheet->m_Object->m_LogFont), &m_Sheet->m_Object->m_LogFont);
	m_Sheet->m_Object->m_LogFont.lfHeight = 32;
	m_Sheet->m_Object->m_LogFont.lfWidth = 0;
	strcpy(&m_Sheet->m_Object->m_LogFont.lfFaceName[0], "Times New Roman");
	m_Sheet->m_Object->m_Font.CreateFontIndirect(&m_Sheet->m_Object->m_LogFont);

	SetDlgItemText(IDC_TEXT, m_Sheet->m_Object->m_Text);
	GetDlgItem(IDC_TEXT)->SetFont(&m_Sheet->m_Object->m_Font, TRUE);

	k3dSetDouble(k3dIChannel::DATA_SCALAR, GetDlgItem(IDC_SCALEX)->m_hWnd, m_Sheet->m_Object->m_ScaleX);
	k3dSetDouble(k3dIChannel::DATA_SCALAR, GetDlgItem(IDC_SCALEY)->m_hWnd, m_Sheet->m_Object->m_ScaleY);
	k3dSetDouble(k3dIChannel::DATA_SCALAR, GetDlgItem(IDC_SLANTX)->m_hWnd, m_Sheet->m_Object->m_SlantX);
	k3dSetDouble(k3dIChannel::DATA_SCALAR, GetDlgItem(IDC_SLANTY)->m_hWnd, m_Sheet->m_Object->m_SlantY);
	k3dSetLong(GetDlgItem(IDC_CURVESUBDIVISION)->m_hWnd, m_Sheet->m_Object->m_CurveSubdivision);

	OnModify();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


void CTextPage::OnChoosefont()
{
	CHOOSEFONT choosefont;

	memset(&choosefont, 0, sizeof(choosefont));
	choosefont.lStructSize = sizeof(choosefont);
	choosefont.hwndOwner = m_hWnd;
	choosefont.lpLogFont = &m_Sheet->m_Object->m_LogFont;
	choosefont.Flags = CF_SCREENFONTS | CF_TTONLY | CF_INITTOLOGFONTSTRUCT;
	choosefont.lpfnHook = NULL;
	if(ChooseFont(&choosefont))
		{
			m_Sheet->m_Object->m_Font.DeleteObject();
			m_Sheet->m_Object->m_Font.CreateFontIndirect(&m_Sheet->m_Object->m_LogFont);
			GetDlgItem(IDC_TEXT)->SetFont(&m_Sheet->m_Object->m_Font, TRUE);

			OnModify();
		}
}



void CTextPage::OnChangeText()
{
	CString temp;
	GetDlgItemText(IDC_TEXT, temp);
	m_Sheet->m_Object->m_Text = (SDPCSTRING)temp;
	OnModify();
}



void CTextPage::OnKillfocusScalex()
{
	KUpdateDouble(k3dIChannel::DATA_SCALAR, GetDlgItem(IDC_SCALEX)->m_hWnd, m_Sheet->m_Object->m_ScaleX);
	OnModify();
}

void CTextPage::OnKillfocusScaley()
{
	KUpdateDouble(k3dIChannel::DATA_SCALAR, GetDlgItem(IDC_SCALEY)->m_hWnd, m_Sheet->m_Object->m_ScaleY);
	OnModify();
}

void CTextPage::OnKillfocusSlantx()
{
	KUpdateDouble(k3dIChannel::DATA_SCALAR, GetDlgItem(IDC_SLANTX)->m_hWnd, m_Sheet->m_Object->m_SlantX);
	OnModify();
}

void CTextPage::OnKillfocusSlanty()
{
	KUpdateDouble(k3dIChannel::DATA_SCALAR, GetDlgItem(IDC_SLANTY)->m_hWnd, m_Sheet->m_Object->m_SlantY);
	OnModify();
}

void CTextPage::OnKillfocusCurvesubdivision()
{
	KValidateLong(GetDlgItem(IDC_CURVESUBDIVISION)->m_hWnd, m_Sheet->m_Object->m_CurveSubdivision, 2, LONG_MAX);
	OnModify();
}







/////////////////////////////////////////////////////////////////////////////
// CText

CText::CText() : k3dPrimTool()
{
	m_Text = "K-3D";

	m_ScaleX = 0.25;
	m_ScaleY = 0.25;
	m_SlantX = 0.0;
	m_SlantY = 0.0;

	m_CurveSubdivision = 4;

	m_Sheet = NULL;
}


CText::~CText()
{
	if(m_Sheet)
		m_Sheet->DestroyWindow();
}



////////////////////////////////////////////////////////////////////////////
// Helper functions for fixed-point and spline math

FIXED CText::FixedFromDouble(double d)
{
	long l;

	l = (long)(d * 65536L);
	return *(FIXED*)&l;
}




double CText::DoubleFromFixed(FIXED Arg)
{
	return (double)Arg.value + ((double)Arg.fract * 0.00001525879);
}





FIXED CText::fxDiv2(FIXED fxVal1, FIXED fxVal2)
{
	long l;

	l = (*((long*)&(fxVal1)) + *((long*)&(fxVal2)))/2;
	return *(FIXED*)&l;
}


double CText::GetXCoord(double Arg)
{
	return ((Arg + m_XOffset) * m_ScaleX);
}



double CText::GetYCoord(double Arg)
{
	return (Arg * m_ScaleY);
}



///////////////////////////////////////////////////////////////
// k3dIUnknown overrides

sdpResult CText::QueryInterface(const sdpIID& InterfaceID, void** Interface)
{
	if(sdpSupportInterface<sdpIUnknown>(static_cast<k3dIObject*>(this), InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIObject>(this, InterfaceID, Interface)) return SDP_S_OK;

	if(sdpSupportInterface<k3dIDocumentEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIUserEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	return SDP_E_NO_INTERFACE;
}


gulong CText::AddRef()
{
	return sdpComAddRef(m_ReferenceCount);
}


gulong CText::Release()
{
	return sdpComRelease(this, m_ReferenceCount);
}




///////////////////////////////////////////////////////////////
// k3dIObject overrides

sdpResult CText::EditObject()
{
	if(!KOpenDialog(m_Sheet, this)) KFAIL(SDP_E_FAIL);
	return SDP_S_OK;
}


///////////////////////////////////////////////////////////////
// k3dIUserEventClient overrides

sdpResult CText::OnLButtonStartDrag(long Flags, long X, long Y)
{
	KUserEvent::OnLButtonStartDrag(Flags, X, Y);
	return SDP_S_OK;
}

sdpResult CText::OnLButtonDrag(long Flags, long X, long Y)
{
	double xfactor, yfactor;

	KUserEvent::OnLButtonDrag(Flags, X, Y);

	xfactor = ((double)(CurrentMouse()[0] - LastMouse()[0])) * 0.01;
	yfactor = ((double)(CurrentMouse()[1] - LastMouse()[1])) * 0.01;

	if(ShiftDown())
		{
			m_SlantX += xfactor;
			m_SlantY -= yfactor;
		}
	else if(ControlDown())
		{
		}
	else
		{
			m_ScaleX += xfactor;
			m_ScaleY -= yfactor;
		}

	k3dSetDouble(k3dIChannel::DATA_SCALAR, m_Sheet->m_TextPage->GetDlgItem(IDC_SCALEX)->m_hWnd, m_ScaleX);
	k3dSetDouble(k3dIChannel::DATA_SCALAR, m_Sheet->m_TextPage->GetDlgItem(IDC_SCALEY)->m_hWnd, m_ScaleY);

	k3dSetDouble(k3dIChannel::DATA_SCALAR, m_Sheet->m_TextPage->GetDlgItem(IDC_SLANTX)->m_hWnd, m_SlantX);
	k3dSetDouble(k3dIChannel::DATA_SCALAR, m_Sheet->m_TextPage->GetDlgItem(IDC_SLANTY)->m_hWnd, m_SlantY);

	OnModify();

	return SDP_S_OK;
}


sdpResult CText::OnLButtonEndDrag(long Flags, long X, long Y)
{
	KUserEvent::OnLButtonEndDrag(Flags, X, Y);

	return SDP_S_OK;
}


void CTextSheet::OnHelp()
{
	m_Object->StandardHelp("tools", "text.html");
}



/////////////////////////////////////////////////////////////////////////////
// CText message handlers

void CText::OnModify()
{
	GLYPHMETRICS gm;
	DWORD size;

	CDC* cdc;
	MAT2 mat;

	void* buffer;

	LPTTPOLYGONHEADER lpStart, lpHeader;
	LPTTPOLYCURVE lpCurve;
	POINTFX spline[3];
	double coords[3];

	char c;
	int pointindex;
	int i, j, k;
	double alphax, alphay, betax, betay;
	double t;



	if(m_Text.size())
		{
			// Setup point / poly arrays
			KillAll();
			pointindex = 0;

			// Create a group and name it
			m_GroupList->Create();
			m_GroupArray.push_back(m_Group->GetHandle());
			sdpString name;
			name.format("Text: %s", m_Text.c_str());
			m_GroupList->Object()->ObjectContext()->SetName(name);

			// Create HDC and select font into it
			cdc = m_Sheet->GetDC();
			cdc->SelectObject(m_Font);

			// make the 2x2 TrueType matrix
			mat.eM11 = FixedFromDouble(1.0);
			mat.eM12 = FixedFromDouble(m_SlantY);
			mat.eM21 = FixedFromDouble(m_SlantX);
			mat.eM22 = FixedFromDouble(1.0);

			m_XOffset = 0.0;

			for(j = 0; j < m_Text.size(); j++)
				{
					c = m_Text[(sdpString::size_type)j];

					// allocate space for the bitmap/outline
					buffer = NULL;
					size = cdc->GetGlyphOutline(c, GGO_NATIVE, &gm, 0, NULL, &mat);
					buffer = malloc(size);
					if(buffer)
						{
							if(cdc->GetGlyphOutline(c, GGO_NATIVE, &gm, size, buffer, &mat) == size)
								{
									lpStart = lpHeader = (LPTTPOLYGONHEADER)buffer;
									while((DWORD)lpHeader < (DWORD)(((SDPSTRING)lpStart) + size))
										{
											if(lpHeader->dwType == TT_POLYGON_TYPE)
												{
													// Create polygon
													m_GroupPolygonList->Create();
													m_PolygonArray.push_back(m_GroupPolygon->GetHandle());
													m_GroupPolygon->SetSelectable(FALSE);
													m_GroupPolygon->SetSaveable(FALSE);

													// Create first point & attach to polygon
													m_GroupPointList->Create();
													m_PointArray.push_back(m_GroupPoint->GetHandle());
													m_GroupPoint->SetSelectable(FALSE);
													m_GroupPoint->SetSaveable(FALSE);
													m_GroupPolygon->AttachPointAfter(m_PointArray[pointindex++]);

													coords[0] = (float)GetXCoord(DoubleFromFixed(lpHeader->pfxStart.x));
													coords[1] = (float)GetYCoord(DoubleFromFixed(lpHeader->pfxStart.y));
													coords[2] = 0.0;

													locationserver->GetLocation(location)->SetOldXYZ(coords);
													locationserver->GetLocation(location)->SetLocalXYZ(coords);

													// Get to first curve.
													lpCurve = (LPTTPOLYCURVE)(lpHeader + 1);

													while((DWORD)lpCurve < (DWORD)(((SDPSTRING)lpHeader) + lpHeader->cb))
														{
															// Straight line segment
															if(TT_PRIM_LINE == lpCurve->wType)
																{
																	for(i = 0; i < lpCurve->cpfx; i++)
																		{
																			// Create point & attach to polygon
																			m_GroupPointList->Create();
																			m_PointArray.push_back(m_GroupPoint->GetHandle());
																			m_GroupPoint->SetSelectable(FALSE);
																			m_GroupPoint->SetSaveable(FALSE);
																			m_GroupPolygon->AttachPointAfter(m_PointArray[pointindex++]);

																			coords[0] = (float)GetXCoord(DoubleFromFixed(lpCurve->apfx[i].x));
																			coords[1] = (float)GetYCoord(DoubleFromFixed(lpCurve->apfx[i].y));
																			coords[2] = 0.0;

																			locationserver->GetLocation(location)->SetOldXYZ(coords);
																			locationserver->GetLocation(location)->SetLocalXYZ(coords);
																		}
																}
															else if(TT_PRIM_QSPLINE == lpCurve->wType)
																{
																	//**********************************************
																	// format assumption:
																	//   The bytes immediately preceding a POLYCURVE
																	//   structure contain a valid POINTFX.
																	//
																	//   If this is first curve, this points to the
																	//      pfxStart of the POLYGONHEADER.
																	//   Otherwise, this points to the last point of
																	//      the previous POLYCURVE.
																	//
																	//	 In either case, this is representative of the
																	//      previous curve's last point.
																	//**********************************************

																	spline[0] = *(LPPOINTFX)((SDPSTRING)lpCurve - sizeof(POINTFX));

																	for(i = 0; i < lpCurve->cpfx;)
																		{
																			// The B point.
																			spline[1] = lpCurve->apfx[i++];

																			// Calculate the C point.
																			if(i == (lpCurve->cpfx - 1))
																				{
																					spline[2] = lpCurve->apfx[i++];
																				}
																			else
																				{
																					// C is midpoint between B and next point.
																					spline[2].x = fxDiv2(lpCurve->apfx[i-1].x, lpCurve->apfx[i].x);
																					spline[2].y = fxDiv2(lpCurve->apfx[i-1].y, lpCurve->apfx[i].y);
																				}

																			// Setup constants to subdivide spline
																			alphax = DoubleFromFixed(spline[0].x) - 2.0 * DoubleFromFixed(spline[1].x) + DoubleFromFixed(spline[2].x);
																			alphay = DoubleFromFixed(spline[0].y) - 2.0 * DoubleFromFixed(spline[1].y) + DoubleFromFixed(spline[2].y);
																			betax = 2.0 * DoubleFromFixed(spline[1].x) - 2.0 * DoubleFromFixed(spline[0].x);
																			betay = 2.0 * DoubleFromFixed(spline[1].y) - 2.0 * DoubleFromFixed(spline[0].y);

																			for(k = 1; k <= m_CurveSubdivision; k++)
																				{
																					t = (double)k / m_CurveSubdivision;
																					// Create point & attach to polygon
																					m_GroupPointList->Create();
																					m_PointArray.push_back(m_GroupPoint->GetHandle());
																					m_GroupPoint->SetSelectable(FALSE);
																					m_GroupPoint->SetSaveable(FALSE);
																					m_GroupPolygon->AttachPointAfter(m_PointArray[pointindex++]);

																					coords[0] = (float)GetXCoord(t * t * alphax + t * betax + DoubleFromFixed(spline[0].x));
																					coords[1] = (float)GetYCoord(t * t * alphay + t * betay + DoubleFromFixed(spline[0].y));
																					coords[2] = 0.0;

																					locationserver->GetLocation(location)->SetOldXYZ(coords);
																					locationserver->GetLocation(location)->SetLocalXYZ(coords);
																				}

																			// New A point for next slice of spline.
																			spline[0] = spline[2];
																		}
																}
															else
																{
																	::MessageBox(NULL, "Unexpected curve type", "Text:", NULL);
																}

															// Move on to next curve.
															lpCurve = (LPTTPOLYCURVE)&(lpCurve->apfx[i]);
														}

													// Move on to next polygon.
													lpHeader = (LPTTPOLYGONHEADER)(((SDPSTRING)lpHeader) + lpHeader->cb);
												}
											else
												{
													::MessageBox(NULL, "Unexpected header type", "Text:", NULL);
												}
										}
								}
							free(buffer);

							m_XOffset += gm.gmCellIncX;
						}
				}

			m_Sheet->ReleaseDC(cdc);
		}

	// Center text on the origin
	CenterNewGeometry();

	// Update our sheet
	m_Sheet->UpdateNumbers();

	// Bypass the base class version of OnModify();
	KUserObject::OnModify();
}





////////////////////////////////////////////////////////////////////////////
// CTextSheet

CTextSheet::CTextSheet(CText* Object)
: KPrimSheet(Object, theApp.m_hInstance, "Text", VERTICAL, "Text", TRUE)
{
	m_TextPage = NULL;
	m_Object = Object;
}


CTextSheet::~CTextSheet()
{
	m_Object->m_Sheet = NULL;
}



bool CTextSheet::Create()
{
	m_TextPage = new CTextPage(this);

	if(m_TextPage)
		{
			AddPage(m_TextPage);

			Create(IDD);
			return TRUE;
		}

	KFAIL(FALSE);
}


BEGIN_MESSAGE_MAP(CTextSheet, KPrimSheet)
	//{{AFX_MSG_MAP(CTextSheet)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


