#include "stdafx.h"
#include "tlbboolean.h"
#include "toollib.h"





CBoolean::CBoolean() : k3dTool()
{
	m_BooleanPage = NULL;

	m_Polygon2 = NULL;
	m_PolygonList2 = NULL;

	m_Point2 = NULL;
	m_PointList2 = NULL;
	m_PointLocation2 = NULL;
}


CBoolean::~CBoolean()
{
	Onclear(&m_APolys);
	Onclear(&m_BPolys);

	if(m_Polygon2) m_Polygon2->Release();
	if(m_PolygonList2) m_PolygonList2->Release();

	if(m_Point2) m_Point2->Release();
	if(m_PointList2) m_PointList2->Release();
	if(m_PointLocation2) m_PointLocation2->Release();
}


bool CBoolean::Create()
{
	if(sdpSucceeded(m_Object->m_Document->CreateObject(KPolygonEnumerator, IID_k3dIPolygon, (void**)&m_Polygon2)))
		{
			if(sdpSucceeded(m_Polygon2->QueryInterface(IID_k3dIList, (void**)&m_PolygonList2)))
				{
					if(sdpSucceeded(m_Object->m_Document->CreateObject(KPointEnumerator, IID_k3dIPoint, (void**)&m_Point2)))
						{
							if(sdpSucceeded(m_Point2->QueryInterface(IID_k3dIList, (void**)&m_PointList2)))
								{
									if(sdpSucceeded(m_Point2->QueryInterface(IID_k3dILocation, (void**)&m_PointLocation2)))
										{
											return TRUE;
										}
								}
						}
				}
		}
	KFAIL(FALSE);
}



///////////////////////////////////////////////////////////////
// k3dIUnknown overrides

sdpResult CBoolean::QueryInterface(const sdpIID& InterfaceID, void** Interface)
{
	if(sdpSupportInterface<sdpIUnknown>(static_cast<k3dIObject*>(this), InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIObject>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIDocumentEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;

	return SDP_E_NO_INTERFACE;
}


gulong CBoolean::AddRef()
{
	return sdpComAddRef(m_ReferenceCount);
}


gulong CBoolean::Release()
{
	return sdpComRelease(this, m_ReferenceCount);
}




///////////////////////////////////////////////////////////////
// k3dIObject overrides

sdpResult CBoolean::EditObject()
{
	m_BooleanPage = new CBooleanPage(this);
	if(m_BooleanPage)
		{
			AddPage(m_BooleanPage);
			Create(IDD);
		}
	KFAIL(SDP_E_FAIL);
}


///////////////////////////////////////////////////////////////
// k3dIDocumentEventClient overrides

sdpResult CBoolean::OnPolygonsDeparting()
{
	sdpResult hr;
	KHANDLE handle;
	POSITION position;

	CList<CPolygon*, CPolygon*> newlist;


	k3dIPolygon* polygon;

	hr = m_Document->CreateObject(KPolygonEnumerator, IID_k3dIList, (void**)&polygonlist);
	if(sdpSucceeded(hr))
		{
			hr = polygonlist->QueryInterface(IID_k3dIPolygon, (void**)&polygon);
			if(sdpSucceeded(hr))
				{
					for(position = m_APolys.GetHeadPosition(); position; m_APolys.GetNext(position))
						{
							handle = m_APolys.GetAt(position)->m_Handle;

							for(hr = polygonlist->SetFirst(); SDP_S_OK == hr; hr = polygonlist->SetNext())
								{
									if(polygon->GetHandle() == handle)
										{
											newlist.push_back(new CPolygon(handle));
											break;
										}
								}
						}

					Onclear(&m_APolys);
					m_APolys.push_back(&newlist);
					newlist.clear();

					for(position = m_BPolys.GetHeadPosition(); position; m_BPolys.GetNext(position))
						{
							handle = m_BPolys.GetAt(position);

							for(hr = polygonlist->SetFirst(); SDP_S_OK == hr; hr = polygonlist->SetNext())
								{
									if(polygon->GetHandle() == handle)
										{
											newlist.push_back(new CPolygon(handle));
											break;
										}
								}
						}

					Onclear(&m_BPolys);
					m_BPolys.push_back(&newlist);
					newlist.clear();

					polygon->Release();
				}
			polygonlist->Release();
		}

	m_BooleanPage->OnUpdateNumbers();

	return SDP_S_OK;
}



void CBooleanSheet::OnHelp()
{
	m_Object->StandardHelp("tools", "boolean.html");
}



void CBoolean::OnAdd(CList<CPolygon*, CPolygon*>*List)
{
	sdpResult hr;
	KHANDLE handle;

	CList<CPolygon*, CPolygon*> newpolylist;

	POSITION position;

	m_PolygonMatchList->ClearCriteria();
	m_PolygonMatchList->SetShowCriteria(TRUE, TRUE);
	m_PolygonMatchList->SetSelectedCriteria(TRUE, TRUE);

	for(hr = m_PolygonMatchList->SetFirstMatch(); SDP_S_OK == hr; hr = m_PolygonMatchList->SetNextMatch())
		{
			handle = m_Polygon->GetHandle();

			for(position = List->GetHeadPosition(); position; List->GetNext(position))
				{
					if(List->GetAt(position) == handle)
						{
							continue;
						}
				}
			newpolylist.push_back(new CPolygon(handle));
		}

	List->push_back(&newpolylist);

	m_BooleanPage->OnUpdateNumbers();
}


void CBoolean::OnRemove(CList<CPolygon*, CPolygon*>* List)
{
	sdpResult hr;
	KHANDLE handle;

	POSITION position;

	m_PolygonMatchList->ClearCriteria();
	m_PolygonMatchList->SetShowCriteria(TRUE, TRUE);
	m_PolygonMatchList->SetSelectedCriteria(TRUE, TRUE);

	for(hr = m_PolygonMatchList->SetFirstMatch(); SDP_S_OK == hr; hr = m_PolygonMatchList->SetNextMatch())
		{
			handle = m_Polygon->GetHandle();

			for(position = List->GetHeadPosition();	position; List->GetNext(position))
				{
					if(List->GetAt(position)->m_Handle == handle)
						{
							delete List->GetAt(position);
							List->RemoveAt(position);
							break;
						}
					List->GetNext(position);
				}
		}
	m_BooleanPage->OnUpdateNumbers();
}


void CBoolean::Onclear(CList<CPolygon*, CPolygon*>* List)
{
	POSITION position;

	for(position = List->GetHeadPosition(); position; List->GetNext(position))
		{
			delete List->GetAt(position);
		}

	List->clear();
}



void CBoolean::OnSelect(CList<CPolygon*, CPolygon*>* List)
{
	POSITION position;
	sdpResult hr;

	for(position = List->GetHeadPosition(); position; List->GetNext(position))
		{
			hr = m_PolygonList->SetHandle(List->GetAt(position)->m_Handle);
			if(sdpSucceeded(hr))
				{
					m_Polygon->SetSelected(TRUE);
					m_Polygon->SetShow(TRUE);
				}
		}

	OnModify();
}


void CBoolean::CalculateBoundingBox(CList<CPolygon*, CPolygon*>* Polys, k3dBoundingBox3* Result)
{
	sdpResult hr;
	POSITION position;
	sdpVector3 coords;

	CPolygon* poly;

	Result->Init();

	for(position = Polys->GetHeadPosition(); position; Polys->GetNext(position))
		{
			poly = Polys->GetAt(position);

			poly->m_BBox.Init();
			m_PolygonList->SetHandle(poly->m_Handle);
			m_Polygon->SetSelected(FALSE);

			for(hr = m_Polygon->SetFirstPoint(); SDP_S_OK == hr; hr = m_Polygon->SetNextPoint())
				{
					m_PolygonLocation->GetOldXYZ(coords);
					Result->Expand(coords);
					poly->m_BBox.Expand(coords);
				}
		}
}


void CBoolean::IntersectPolysWithBoundingBox(CList<CPolygon*, CPolygon*>* Polys, k3dBoundingBox3* Box, CList<CPolygon*, CPolygon*>* Result)
{
	POSITION position;
	CPolygon* poly;
	k3dBoundingBox3 intersection;

	for(position = Polys->GetHeadPosition(); position; Polys->GetNext(position))
		{
			poly = Polys->GetAt(position);

			intersection = ::Intersect(poly->m_BBox, *Box);
			if(intersection.IsDefined())	// If the polygon intersects
				{
					Result->push_back(poly);
				}
		}
}


void CBoolean::BBoxIntersectPolysWithPolys(CList<CPolygon*, CPolygon*>* APolys, CList<CPolygon*, CPolygon*>* BPolys, CList<CPolygon*, CPolygon*>* Result)
{
	POSITION position, position2;

	CPolygon* polya;
	CPolygon* polyb;
	k3dBoundingBox3 intersection;

	for(position = APolys->GetHeadPosition(); position; APolys->GetNext(position))
		{
			polya = APolys->GetAt(position);

			for(position2 = BPolys->GetHeadPosition(); position2; BPolys->GetNext(position2))
				{
					polyb = BPolys->GetAt(position2);

					intersection = ::Intersect(polya->m_BBox, polyb->m_BBox);
					if(intersection.IsDefined())	// If the polygon intersects
						{
							m_PolygonList->SetHandle(polya->m_Handle);
							m_Polygon->SetSelected(TRUE);
							Result->push_back(polya);
							break;
						}
				}
		}
}



void CBoolean::CalculatePlaneEquations(CList<CPolygon*, CPolygon*>* Polys)
{
	sdpResult hr;
	POSITION position;
	CPolygon* poly;
	gulong count;

	sdpVector3 coords;
	sdpVector3 p1, p2, p3;

	for(position = Polys->GetHeadPosition(); position; Polys->GetNext(position))
		{
			poly = Polys->GetAt(position);

			count = 0;
			poly->m_Origin = 0.0;

			m_PolygonList->SetHandle(poly->m_Handle);

			for(hr = m_Polygon->SetFirstPoint(); SDP_S_OK == hr; hr = m_Polygon->SetNextPoint())
				{
					count++;
					m_PolygonLocation->GetOldXYZ(coords);
					poly->m_Origin += coords;

					switch(count)
						{
							case 1:
								p1 = coords;
								break;

							case 2:
								p2 = coords;
								break;

							case 3:
								p3 = coords;
								break;

							default:
								break;
						}
				}
			poly->m_Origin /= count;	// Calculate origin by averaging

			poly->m_Normal = (p2 - p1) ^ (p3 - p1);
		}
}



void CBoolean::IntersectPolysWithPolys(CList<CPolygon*, CPolygon*>* APolys, CList<CPolygon*, CPolygon*>* BPolys)
{
	POSITION position, position2;

	CPolygon* polya;
	CPolygon* polyb;

	// Calculate plane equations for faces
	CalculatePlaneEquations(APolys);
	CalculatePlaneEquations(BPolys);

	// Calculate epsilon for faces
#pragma SDPPOSTIT(*** Calculate face epsilons\n)
//	CalculateEpsilon(APolys);
//	CalculateEpsilon(BPolys);

	// Test faces against each other
	for(position = APolys->GetHeadPosition(); position; APolys->GetNext(position))
		{
			polya = APolys->GetAt(position);

			for(position2 = BPolys->GetHeadPosition(); position2; BPolys->GetNext(position2))
				{
					polyb = BPolys->GetAt(position2);

					IntersectPolyWithPoly(polya, polyb);
				}
		}
}


/*
// findEps - determine tolerance for a face by subtituting all its vertices
// into its plane equation.  Vertex tolerances are also taken into account.

findEps(face)
FACE *face;
{
    extern REAL faceEps, maxFaceEps;	// Defined in "newmain.c"
    CONTOUR *contur;
    ELIST *elptr, *first;
    VERTEX *vert;
    REAL maxerr = 0.0;
    REAL error;
    REAL planeDist();

    face->eps = 0.0;
    for(contur=face->fcontlist; contur!=NULL; contur=contur->nextcn)
    {
	first = elptr = contur->cnelist;
	do {
	    vert = ISON(elptr->elflags,REVDIRECT) ? elptr->eledge->vert2 :
							elptr->eledge->vert1;
#ifdef NEVER
	    error = planeDist(face,vert);
#endif
	    PLANEDIST(error,face,vert);
	    error += vert->veps;
	    if(error < 0.0) error = -error;
	    if(error > maxerr) maxerr = error;
	    elptr=elptr->nextel;
	} while (elptr != first);
    }
    face->eps = maxerr;
    if(face->eps < faceEps)
	face->eps = faceEps;
    if(face->eps > maxFaceEps)
    {
	fprintf(stderr,"Face: %s: Tolerance exceeds maximum.\n",face->fid);
	fprintf(stderr,"face: %g max: %g\n",face->eps, maxFaceEps);
    }

}

// planeDist - find out how far away from a plane a certain point is.
// the distance is positive, zero, or negative.  The distance will be
// zero if the calculated distance is less than the face's epsilon tolerance.


REAL planeDist(face,pt)
FACE *face;
VERTEX *pt;
{
    REAL dist;

    dist = dotp(face->planeq.normal,makeVect(pt->p,face->planeq.origin));
    if(ABS(dist) < face->eps)
	 return(0);
    else return(dist);
}

*/





void CBoolean::IntersectPolyWithPoly(CPolygon* APoly, CPolygon* BPoly)
{
/*
	PLANE n[3];
	REAL d[3];
	REAL x,y,z;
	REAL val;
	bool updown;
	int i,j;
	int cuts;

	bool inface1, inface2;
	VERTEX *verta, *vertb;

	int tcomp();
	VERTEX* makevert();

	facePairs++;

	facea = NULL;
	faceb = NULL;
	nInsects = 0;
*/

	CPolygon line(NULL);
	double da, db, du;
	double val;

	// Test face normals against each other
	line.m_Normal = APoly->m_Normal ^ APoly->m_Normal;
	val = line.m_Normal.Length();

	// If magnitude is on the order of the machine precision, the faces MUST be considered coplanar
	if(val < 1e-7)
		{
#pragma SDPPOSTIT(*** Handle coplanar case here\n)
			return;
			// return coPlanarCase(PolyA, PolyB);
		}

	// Face planes intersect - get intersection line. We already have the line direction (from the crossproduct)
	line.m_Normal /= val;	// Normalize the vector
	line.m_Origin = 0.0;

	da = APoly->m_Normal * APoly->m_Origin;
	db = BPoly->m_Normal * BPoly->m_Origin;
	du = line.m_Normal * line.m_Origin;

	// Orthonormalize A wrt B
	val = APoly->m_Normal * BPoly->m_Normal;	// Orthogonalize
	da -= val * db;
	BPoly->m_Normal *= val;
	APoly->m_Normal -= BPoly->m_Normal;

	val = APoly->m_Normal.Length();
	if(val < 1e-4)
		{
#pragma SDPPOSTIT(*** Handle possibly inaccurate line equation\n)
		}

	da /= val;
	APoly->m_Normal /= val;	// Normalize

	// Now get our point
	line.m_Origin -= APoly->m_Normal * da;
	line.m_Origin -= BPoly->m_Normal * db;
	line.m_Origin -= line.m_Normal * du;

/*
	switch(findCrossings(face1, face2))
		{
			case NOCROSS:
				resetPhantom();
				return(0);
			case SAMEPLANE:
				resetPhantom();
				return(coPlanarCase(face1, face2));
			case TOUCHING:
				if(!mergevert)
					{
						resetPhantom();
						return(0);
					}
		}
	switch(findCrossings(face2, face1))
		{
			case NOCROSS:
				resetPhantom();
				return(0);
			case TOUCHING:
				if(!mergevert)
					{
						resetPhantom();
						return(0);
					}
		}

	// Now we have the intersection line.  Trace around each face
	//and find the intersections between face edges and this line.

	findCrossings(face1, face2);
	findCrossings(face2, face1);
	resetPhantom();

	// Now sort the intersection points (and associated information)
	//into order along the intersection line. Sorting order is:
	//	1st by value of "t" along intersection line
	//	2nd by face (and then by contour) which intersection occurs with
	//	3rd on whether the intersection represents a right or left edge.


	qsort(pInsects, nInsects, sizeof(INTITEM *), tcomp);

	if(debug)
		{
			fprintf(stderr,"Sorted intersections:\n");
			for(i = 0; i < nInsects; i++)
				{
					fprintf(stderr,"%d) Vertex: %s  Face: %s; comp: %d  t: %f ptr %lx flags %d %d\n",	i,pInsects[i]->ivert->vid,pInsects[i]->iface->fid,pInsects[i]->icontur,	pInsects[i]->t,pInsects[i],pInsects[i]->ivert->vflags,pInsects[i]->iflags);
				}
		}
	mergeVerts(face1, face2);
	removeTouches(pInsects, &nInsects);

	if(debug)
		{
			fprintf(stderr,"Sorted intersections:\n");
			for(i = 0; i < nInsects; i++)
				{
 					fprintf(stderr,"%d) Vertex: %s  Face: %s; comp: %d  t: %f ptr %lx flags %d %d\n",	i,pInsects[i]->ivert->vid,pInsects[i]->iface->fid,pInsects[i]->icontur,	pInsects[i]->t,pInsects[i],pInsects[i]->ivert->vflags,pInsects[i]->iflags);
				}
		}

	cuts = lineproc(pInsects, nInsects, face1, face2);

	for(i = 0; i < nInsects; i++)	pInsects[i] == NULL;

	if(cuts != 0) nOfIfaces++;
	numCuts += cuts;
	return(cuts!=0);
*/
}



void CBoolean::Intersect()
{
	k3dBoundingBox3 polyabbox;
	k3dBoundingBox3 polybbbox;
	k3dBoundingBox3 polyintersection;

	sdpVector3 coords;

	// Reset lists
	m_APolys1.clear();
	m_BPolys1.clear();
	m_APolys2.clear();
	m_BPolys2.clear();

	// Calculate bounding boxes for A & B
	CalculateBoundingBox(&m_APolys, &m_APolysBBox);
	CalculateBoundingBox(&m_BPolys, &m_BPolysBBox);

	// Calculate intersection of A & B bounding boxes
	m_Intersection = ::Intersect(m_APolysBBox, m_BPolysBBox);
	if(m_Intersection.IsDefined())	// Do we have a valid intersection?
		{
			// Test individual polygons against the intersection
			IntersectPolysWithBoundingBox(&m_APolys, &m_Intersection, &m_APolys1);
			IntersectPolysWithBoundingBox(&m_BPolys, &m_Intersection, &m_BPolys1);

			// Did we have anything left?
			if(m_APolys1.GetCount() && m_BPolys1.GetCount())
				{
					// Test our candidate A polys against candidate B polys
					BBoxIntersectPolysWithPolys(&m_APolys1, &m_BPolys1, &m_APolys2);
					if(m_APolys2.GetCount())
						{
							// Test our candidate B polys against remaining candidate A polys
							BBoxIntersectPolysWithPolys(&m_BPolys1, &m_APolys2, &m_BPolys2);

							// Detect true poly/poly intersections
							IntersectPolysWithPolys(&m_APolys2, &m_BPolys2);

							OnModify();
							return;
						}
				}
		}

	m_IO->Error("Objects don't overlap!", "Boolean:");
}


BEGIN_MESSAGE_MAP(CBoolean, KSheet)
	//{{AFX_MSG_MAP(CBoolean)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CBoolean message handlers

/*
void CBoolean::OnModify(bool UpdateView)
{
	POSITION position1, position2;
	double coords1[3], coords2[3];
	double range;
	double dist, distx, disty, distz;
	double strength;
	SBooleanActivePoint activepoint;
	SBooleanGridPoint gridpoint;

	range = m_BooleanPage->m_Range * m_BooleanPage->m_Range;

	if(CBooleanPage::BONES == m_BooleanPage->m_Type)
		{
			// Calculate & store grid point deltas
			for(position1 = m_GridPoints.GetHeadPosition(); position1; m_GridPoints.GetNext(position1))
				{
					gridpoint = m_GridPoints.GetAt(position1);

					m_Point->SetHandle(gridpoint.m_PointHandle);
					m_PointLocation->GetLocalXYZ(coords1);
					gridpoint.m_DX = (coords1[0] - gridpoint.m_OriginalX);
					gridpoint.m_DY = (coords1[1] - gridpoint.m_OriginalY);
					gridpoint.m_DZ = (coords1[2] - gridpoint.m_OriginalZ);

					m_GridPoints.GetAt(position1) = gridpoint;
				}

			// Enumerate active points
			for(position2 = m_ActivePoints.GetHeadPosition(); position2; m_ActivePoints.GetNext(position2))
				{
					activepoint = m_ActivePoints.GetAt(position2);
					coords2[0] = activepoint.m_OriginalX;
					coords2[1] = activepoint.m_OriginalY;
					coords2[2] = activepoint.m_OriginalZ;

					m_Point->SetHandle(activepoint.m_PointHandle);

					// Enumerate grid points
					for(position1 = m_GridPoints.GetHeadPosition(); position1; m_GridPoints.GetNext(position1))
						{
							gridpoint = m_GridPoints.GetAt(position1);

							distx = gridpoint.m_OriginalX - activepoint.m_OriginalX;
							disty = gridpoint.m_OriginalY - activepoint.m_OriginalY;
							distz = gridpoint.m_OriginalZ - activepoint.m_OriginalZ;

							dist = distx * distx + disty * disty + distz * distz;
							if(dist > range) continue;

							strength = sqrt(dist) / m_BooleanPage->m_Range;
							if(m_BooleanPage->m_CustomStrength)
								{
									strength = m_Strengthfalloff->GetValue1(strength, .001) * m_BooleanPage->m_Strength;
								}
							else
								{
									strength = (1.0 - strength) * m_BooleanPage->m_Strength;
								}

							coords2[0] += strength * gridpoint.m_DX;
							coords2[1] += strength * gridpoint.m_DY;
							coords2[2] += strength * gridpoint.m_DZ;
						}
					m_PointLocation->SetLocalXYZ(coords2);
				}
		}
	else if(CBooleanPage::MAGNETS == m_BooleanPage->m_Type)
		{
			// Calculate & store grid point locations
			for(position1 = m_GridPoints.GetHeadPosition(); position1; m_GridPoints.GetNext(position1))
				{
					gridpoint = m_GridPoints.GetAt(position1);

					m_Point->SetHandle(gridpoint.m_PointHandle);
					m_PointLocation->GetLocalXYZ(coords1);
					gridpoint.m_DX = coords1[0];
					gridpoint.m_DY = coords1[1];
					gridpoint.m_DZ = coords1[2];

					m_GridPoints.GetAt(position1) = gridpoint;
				}

			// Enumerate active points
			for(position2 = m_ActivePoints.GetHeadPosition(); position2; m_ActivePoints.GetNext(position2))
				{
					activepoint = m_ActivePoints.GetAt(position2);
					coords2[0] = activepoint.m_OriginalX;
					coords2[1] = activepoint.m_OriginalY;
					coords2[2] = activepoint.m_OriginalZ;

					m_Point->SetHandle(m_ActivePoints.GetAt(position2).m_PointHandle);

					// Enumerate grid points
					for(position1 = m_GridPoints.GetHeadPosition(); position1; m_GridPoints.GetNext(position1))
						{
							gridpoint = m_GridPoints.GetAt(position1);

							distx = gridpoint.m_DX - coords2[0];
							disty = gridpoint.m_DY - coords2[1];
							distz = gridpoint.m_DZ - coords2[2];

							dist = distx * distx + disty * disty + distz * distz;
							if(dist > range) continue;

							strength = sqrt(dist) / m_BooleanPage->m_Range;
							if(m_BooleanPage->m_CustomStrength)
								{
									strength = m_Strengthfalloff->GetValue1(strength, .001) * m_BooleanPage->m_Strength;
								}
							else
								{
									strength = (1.0 - strength) * m_BooleanPage->m_Strength;
								}

							coords2[0] += strength * distx;
							coords2[1] += strength * disty;
							coords2[2] += strength * distz;
						}
					m_PointLocation->SetLocalXYZ(coords2);
				}
		}

	if(UpdateView)
		{
			m_Document->ASyncUpdateAllViews();
		}
}




void CBoolean::OnReset()
{
	POSITION position;
	double coords[3];

	SBooleanActivePoint activepoint;
	SBooleanGridPoint gridpoint;

	// Reset grid points
	for(position = m_GridPoints.GetHeadPosition(); position; m_GridPoints.GetNext(position))
		{
			gridpoint = m_GridPoints.GetAt(position);

			coords[0] = gridpoint.m_OriginalX;
			coords[1] = gridpoint.m_OriginalY;
			coords[2] = gridpoint.m_OriginalZ;

			m_Point->SetHandle(gridpoint.m_PointHandle);
			m_PointLocation->SetOldXYZ(coords);
			m_PointLocation->SetLocalXYZ(coords);
		}

	// Reset active points
	for(position = m_ActivePoints.GetHeadPosition(); position; m_ActivePoints.GetNext(position))
		{
			activepoint = m_ActivePoints.GetAt(position);

			coords[0] = activepoint.m_OriginalX;
			coords[1] = activepoint.m_OriginalY;
			coords[2] = activepoint.m_OriginalZ;

			m_Point->SetHandle(activepoint.m_PointHandle);
			m_PointLocation->SetOldXYZ(coords);
			m_PointLocation->SetLocalXYZ(coords);
		}

	KSheet::OnModify();

}


void CBoolean::OnSet()
{
	POSITION position;
	double coords[3];

	SBooleanActivePoint activepoint;
	SBooleanGridPoint gridpoint;

	if(SDP_S_OK == m_IO->Query("Make all changes to grid and active points permanent? (No Undo)", "Boolean:"))
		{
			// Reset grid points
			for(position = m_GridPoints.GetHeadPosition(); position; m_GridPoints.GetNext(position))
				{
					gridpoint = m_GridPoints.GetAt(position);
					m_Point->SetHandle(gridpoint.m_PointHandle);

					m_PointLocation->GetLocalXYZ(coords);
					m_PointLocation->SetOldXYZ(coords);

					gridpoint.m_OriginalX = coords[0];
					gridpoint.m_OriginalY = coords[1];
					gridpoint.m_OriginalZ = coords[2];

					m_GridPoints.GetAt(position) = gridpoint;
				}

			// Reset active points
			for(position = m_ActivePoints.GetHeadPosition(); position; m_ActivePoints.GetNext(position))
				{
					activepoint = m_ActivePoints.GetAt(position);
					m_Point->SetHandle(activepoint.m_PointHandle);

					m_PointLocation->GetLocalXYZ(coords);
					m_PointLocation->SetOldXYZ(coords);

					activepoint.m_OriginalX = coords[0];
					activepoint.m_OriginalY = coords[1];
					activepoint.m_OriginalZ = coords[2];

					m_ActivePoints.GetAt(position) = activepoint;
				}

			KSheet::OnSet();
		}
}

*/

/////////////////////////////////////////////////////////////////////////////
// CBooleanPage property page

CBooleanPage::CBooleanPage(CBoolean* Parent)
  : KPage(IDD, Parent, "Boolean")
{
	//{{AFX_DATA_INIT(CBooleanPage)
	//}}AFX_DATA_INIT

	m_Sheet = Parent;
}




BEGIN_MESSAGE_MAP(CBooleanPage, KPage)
	//{{AFX_MSG_MAP(CBooleanPage)
	ON_BN_CLICKED(IDC_AADD, OnAadd)
	ON_BN_CLICKED(IDC_ADD, OnAdd)
	ON_BN_CLICKED(IDC_AND, OnAnd)
	ON_BN_CLICKED(IDC_AREMOVE, OnAremove)
	ON_BN_CLICKED(IDC_Aclear, OnAclear)
	ON_BN_CLICKED(IDC_ASELECT, OnAselect)
	ON_BN_CLICKED(IDC_BADD, OnBadd)
	ON_BN_CLICKED(IDC_BREMOVE, OnBremove)
	ON_BN_CLICKED(IDC_Bclear, OnBclear)
	ON_BN_CLICKED(IDC_BSELECT, OnBselect)
	ON_BN_CLICKED(IDC_SUBTRACT, OnSubtract)
	ON_BN_CLICKED(IDC_XOR, OnXor)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CBooleanPage message handlers

BOOL CBooleanPage::OnInitDialog()
{
	KPage::OnInitDialog();

	OnUpdateNumbers();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}




void CBooleanPage::OnUpdateNumbers()
{
	k3dSetLong(GetDlgItem(IDC_APOLYS)->m_hWnd, m_Sheet->m_APolys.GetCount());
	k3dSetLong(GetDlgItem(IDC_BPOLYS)->m_hWnd, m_Sheet->m_BPolys.GetCount());
}





void CBooleanPage::OnAadd()
{
	m_Sheet->OnAdd(&m_Sheet->m_APolys);
}

void CBooleanPage::OnBadd()
{
	m_Sheet->OnAdd(&m_Sheet->m_BPolys);
}

void CBooleanPage::OnAremove()
{
	m_Sheet->OnRemove(&m_Sheet->m_APolys);
}

void CBooleanPage::OnBremove()
{
	m_Sheet->OnRemove(&m_Sheet->m_BPolys);
}

void CBooleanPage::OnAclear()
{
	m_Sheet->Onclear(&m_Sheet->m_APolys);
	OnUpdateNumbers();
}

void CBooleanPage::OnBclear()
{
	m_Sheet->Onclear(&m_Sheet->m_BPolys);
	OnUpdateNumbers();
}

void CBooleanPage::OnAselect()
{
	m_Sheet->OnSelect(&m_Sheet->m_APolys);
}

void CBooleanPage::OnBselect()
{
	m_Sheet->OnSelect(&m_Sheet->m_BPolys);
}





void CBooleanPage::OnAdd()
{
	m_Sheet->Intersect();
}

void CBooleanPage::OnAnd()
{
	m_Sheet->Intersect();
}

void CBooleanPage::OnSubtract()
{
	m_Sheet->Intersect();
}

void CBooleanPage::OnXor()
{
	m_Sheet->Intersect();
}
