#include "stdafx.h"
#include "tlbbreakpoly.h"
#include "toollib.h"





CBreakPolyPage::CBreakPolyPage(CBreakPolySheet* Sheet)
	: KPage(IDD, Sheet, "BreakPoly")
{
	//{{AFX_DATA_INIT(CBreakPolyPage)
	//}}AFX_DATA_INIT

	m_Sheet = Sheet;
}




BEGIN_MESSAGE_MAP(CBreakPolyPage, KPage)
	//{{AFX_MSG_MAP(CBreakPolyPage)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CBreakPolyPage message handlers




////////////////////////////////////////////////////////////////////////////
// CBreakPolySheet

CBreakPolySheet::CBreakPolySheet(CBreakPoly* Object)
: KPrimSheet(Object, theApp.m_hInstance, "BreakPoly", VERTICAL, "BreakPoly", TRUE)
{
	m_BreakPolyPage = NULL;
	m_Object = Object;
}


CBreakPolySheet::~CBreakPolySheet()
{
	m_Object->m_Sheet = NULL;
}



bool CBreakPolySheet::Create()
{
	m_BreakPolyPage = new CBreakPolyPage(this);

	if(m_BreakPolyPage)
		{
			AddPage(m_BreakPolyPage);

			Create(IDD);
			return TRUE;
		}

	KFAIL(FALSE);
}


BEGIN_MESSAGE_MAP(CBreakPolySheet, KPrimSheet)
	//{{AFX_MSG_MAP(CBreakPolySheet)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()





/////////////////////////////////////////////////////////////////////////////
// CBreakPoly

CBreakPoly::CBreakPoly() : k3dPrimTool()
{
	m_Point1 = NULL;
	m_Point2 = NULL;
	m_Polygon = NULL;

	m_Sheet = NULL;
}


CBreakPoly::~CBreakPoly()
{
	if(m_Sheet)
		m_Sheet->DestroyWindow();
}


///////////////////////////////////////////////////////////////
// k3dIUnknown overrides

sdpResult CBreakPoly::QueryInterface(const sdpIID& InterfaceID, void** Interface)
{
	if(sdpSupportInterface<sdpIUnknown>(static_cast<k3dIObject*>(this), InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIObject>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIDocumentEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIUserEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIOpenGLRender>(this, InterfaceID, Interface)) return SDP_S_OK;

	return SDP_E_NO_INTERFACE;
}


gulong CBreakPoly::AddRef()
{
	return sdpComAddRef(m_ReferenceCount);
}


gulong CBreakPoly::Release()
{
	return sdpComRelease(this, m_ReferenceCount);
}




///////////////////////////////////////////////////////////////
// k3dIObject overrides

sdpResult CBreakPoly::SetObjectContext(k3dIApplication* Server, k3dIObjectContext* Context)
{
	if(sdpSucceeded(k3dPrimTool::SetObjectContext(Server, Context)))
		return SDP_S_OK;

	KFAIL(SDP_E_FAIL);
}



sdpResult CBreakPoly::EditObject()
{
	if(!KOpenDialog(m_Sheet, this)) KFAIL(SDP_E_FAIL);
	return SDP_S_OK;
}




///////////////////////////////////////////////////////////////
// k3dIDocumentEventClient overrides

sdpResult CBreakPoly::OnUndo(k3dIObject* Sender)
{
	KillAll();

	m_Polygon = NULL;
	m_Point1 = NULL;
	m_Point2 = NULL;

	OnModify();

	return SDP_S_OK;
}




///////////////////////////////////////////////////////////////
// k3dIOpenGLRender overrides

sdpResult CBreakPoly::OpenGLRender(k3dICamera* Camera, k3dIOpenGLEngine* Engine)
{
	kglPushAttrib(GL_ALL_ATTRIB_BITS);

	kglDisable(GL_LIGHTING);
	kglDisable(GL_TEXTURE_1D);
	kglDisable(GL_TEXTURE_2D);

	kglColor3d(1.0, 0.0, 0.0);
	kglLineWidth(1.0f);
	kglPointSize(4.0f);

	sdpVector3 coords;

	// Draw selected polygon
	if(m_Polygon)
		{
			kglBegin(GL_LINE_LOOP);

			k3dIPolyPointServer* points = m_Polygon->Points();
			for(K3DPOLYPOINTHANDLE polypointhandle = points->GetFirstPolyPoint(); polypointhandle; polypointhandle = points->GetNextPolyPoint(polypointhandle))
				{
					points->GetPolyPoint(polypointhandle)->Location()->GetAnimXYZ(coords);
					kglVertex3dv(coords);
				}

			kglEnd();
		}

	// Draw selected points
	kglBegin(GL_POINTS);

	if(m_Point1)
		{
			m_Point1->Location()->GetAnimXYZ(coords);
			kglVertex3dv(coords);
		}
	if(m_Point2)
		{
			m_Point2->Location()->GetAnimXYZ(coords);
			kglVertex3dv(coords);
		}

	kglEnd();
	kglPopAttrib();

	return SDP_S_OK;
}




///////////////////////////////////////////////////////////////
// k3dIUserEventClient overrides

void CBreakPolySheet::OnHelp()
{
	m_Object->StandardHelp("tools", "breakpoly.html");
}





/////////////////////////////////////////////////////////////////////////////
// CBreakPoly message handlers


sdpResult CBreakPoly::OnLButtonClick(long Flags, long X, long Y)
{
	static const long clicksensitivity = 4;

	KUserEvent::OnLButtonClick(Flags, X, Y);

	// Calculate screen coordinates for document points ...
	m_Document->GetActiveView()->Camera()->SnapshotScreenCoordinates();

	sdpVector3 screencoords;

	// For each polygon in the document ...
	for(k3dIPolygon* poly = m_Document->GetFirstPolygon(); poly; poly = m_Document->GetNextPolygon(poly))
		{
			// If the polygon isn't visible and selectable, skip it ...
			if(SDP_S_OK != poly->State()->QueryState(k3dIState::VISIBLE | k3dIState::SELECTABLE)) continue;

			// For each poly point ...
			k3dIPolyPointServer* points = poly->Points();
			for(K3DPOLYPOINTHANDLE polypointhandle = points->GetFirstPolyPoint(); polypointhandle; polypointhandle = points->GetNextPolyPoint(polypointhandle))
				{
					k3dIPolyPoint* polypoint = points->GetPolyPoint(polypointhandle);
					polypoint->Location()->GetScreenXYZ(screencoords);

					// If the point isn't close to the mouse click, skip it ...
					if((abs(screencoords[0] - CurrentMouse()[0]) > clicksensitivity) ||
						 (abs(screencoords[1] - CurrentMouse()[1]) > clicksensitivity)) continue;

					if(m_Polygon)
						{
							if(m_Polygon != poly)
								{
									m_IO->Error("Must select a point on the same polygon", "BreakPoly:");
									return SDP_S_OK;
								}
							else
								{
									if(m_Point2) m_Point1 = m_Point2;

									m_Point2 = polypoint->Point();
									CreatePolygons();
									OnModify();
									return SDP_S_OK;
								}
						}
					else
						{
							m_Polygon = poly;
							m_Point1 = polypoint->Point();
							OnModify();
							return SDP_S_OK;
						}
				}
		}

	OnModify();

	return SDP_S_OK;
}


void CBreakPoly::CreatePolygons()
{
	// Sanity check ...
	g_assert(m_Polygon && m_Point1 && m_Point2);

	bool found1 = FALSE;
	bool found2 = FALSE;

#pragma SDPPOSTIT(*** CBreakPoly::CreatePolygons())
/*
	// Allocate first new polygon
	m_PolygonList->Create();
	m_PolygonArray.push_back(m_Polygon->GetHandle());
	m_Polygon->SetSelectable(FALSE);

	// Add points to first polygon
	polygonlist->SetHandle(m_Polygon);
	for(hr = polygon->SetFirstPoint(); SDP_S_OK == hr;)
		{
			m_Polygon->AttachPointAfter(polygon->GetPointHandle());

			if(polygon->GetPointHandle() == m_Point1 && !found2)
				{
					found1 = TRUE;
					while(polygon->GetPointHandle() != m_Point2)
						{
							polygon->SetNextPoint();
						}
				}
			else if(polygon->GetPointHandle() == m_Point2 && !found1)
				{
					found2 = TRUE;
					while(polygon->GetPointHandle() != m_Point1)
						{
							polygon->SetNextPoint();
						}
				}
			else
				{
					hr = polygon->SetNextPoint();
				}
		}

	// Allocate second new polygon
	m_PolygonList->Create();
	m_PolygonArray.push_back(m_Polygon->GetHandle());
	m_Polygon->SetSelectable(FALSE);

	// Add points to second poly
	if(found1)
		{
			polygonlist->SetHandle(m_Polygon);

			for(hr = polygon->SetFirstPoint(); SDP_S_OK == hr; hr = polygon->SetNextPoint())
				{
					if(polygon->GetPointHandle() == m_Point1) break;
				}

			for(; SDP_S_OK == hr;)
				{
					m_Polygon->AttachPointAfter(polygon->GetPointHandle());

					if(polygon->GetPointHandle() == m_Point2)
						{
							break;
						}

					hr = polygon->SetNextPoint();
				}
		}
	else if(found2)
		{
			polygonlist->SetHandle(m_Polygon);

			for(hr = polygon->SetFirstPoint(); SDP_S_OK == hr; hr = polygon->SetNextPoint())
				{
					if(polygon->GetPointHandle() == m_Point2) break;
				}

			for(; SDP_S_OK == hr;)
				{
					m_Polygon->AttachPointAfter(polygon->GetPointHandle());

					if(polygon->GetPointHandle() == m_Point1)
						{
							break;
						}

					hr = polygon->SetNextPoint();
				}
		}
*/

	m_Polygon = NULL;
	m_Point1 = NULL;
	m_Point2 = NULL;
}


void CBreakPoly::OnModify()
{
	// Update the display, and skip the implementation of k3dTool::OnModify()
	KUserObject::OnModify();

	if(m_Sheet)
		m_Sheet->UpdateNumbers();
}


