#include "stdafx.h"
#include "tlbjoinpolys.h"
#include "toollib.h"




CJoinPolysPage::CJoinPolysPage(CJoinPolysSheet* Sheet)
	: KPage(IDD, Sheet, "JoinPolys")
{
	//{{AFX_DATA_INIT(CJoinPolysPage)
	//}}AFX_DATA_INIT
	m_Sheet = Sheet;
}




BEGIN_MESSAGE_MAP(CJoinPolysPage, KPage)
	//{{AFX_MSG_MAP(CJoinPolysPage)
	ON_BN_CLICKED(IDC_2POINT, On2Point)
	ON_BN_CLICKED(IDC_4POINT, On4Point)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CJoinPolysPage message handlers

BOOL CJoinPolysPage::OnInitDialog()
{
	KPage::OnInitDialog();

	if(CJoinPolys::TWOPOINT == m_Sheet->m_Object->m_JoinType)
		{
			CheckRadioButton(IDC_2POINT, IDC_4POINT, IDC_2POINT);
		}
	else if(CJoinPolys::FOURPOINT == m_Sheet->m_Object->m_JoinType)
		{
			CheckRadioButton(IDC_2POINT, IDC_4POINT, IDC_4POINT);
		}

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}



void CJoinPolysPage::On2Point()
{
	m_Sheet->m_Object->m_JoinType = CJoinPolys::TWOPOINT;
	m_Sheet->m_Object->OnReset();
}


void CJoinPolysPage::On4Point()
{
	m_Sheet->m_Object->m_JoinType = CJoinPolys::FOURPOINT;
	m_Sheet->m_Object->OnReset();
}


/////////////////////////////////////////////////////////////////////////////
// CJoinPolys

CJoinPolys::CJoinPolys() : k3dPrimTool()
{
	m_Point1 = NULL;
	m_Point2 = NULL;
	m_Point3 = NULL;
	m_Point4 = NULL;
	m_Polygon1 = NULL;
	m_Polygon2 = NULL;

	m_JoinType = TWOPOINT;

	m_Sheet = NULL;
}


CJoinPolys::~CJoinPolys()
{
	if(m_Sheet)
		m_Sheet->DestroyWindow();
}



bool CJoinPolys::Create()
{
	if(!k3dPrimTool::Create()) KFAIL(FALSE);

	m_PointMatchList->ClearCriteria();
	m_PointMatchList->SetShowCriteria(TRUE, TRUE);
	m_PointMatchList->SetSelectedCriteria(TRUE, TRUE);

	return TRUE;
}



///////////////////////////////////////////////////////////////
// k3dIUnknown overrides

sdpResult CJoinPolys::QueryInterface(const sdpIID& InterfaceID, void** Interface)
{
	if(sdpSupportInterface<sdpIUnknown>(static_cast<k3dIObject*>(this), InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIObject>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIDocumentEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIUserEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIEventClient>(this, InterfaceID, Interface)) return SDP_S_OK;
	if(sdpSupportInterface<k3dIOpenGLRender>(this, InterfaceID, Interface)) return SDP_S_OK;

	return SDP_E_NO_INTERFACE;
}


gulong CJoinPolys::AddRef()
{
	return sdpComAddRef(m_ReferenceCount);
}


gulong CJoinPolys::Release()
{
	return sdpComRelease(this, m_ReferenceCount);
}





///////////////////////////////////////////////////////////////
// k3dIObject overrides

sdpResult CJoinPolys::SetObjectContext(k3dIApplication* Server, k3dIObjectContext* Context)
{
	if(sdpSucceeded(k3dPrimTool::SetObjectContext(Server, Context)))
		{
			return SDP_S_OK;
		}
	KFAIL(SDP_E_FAIL);
}



sdpResult CJoinPolys::EditObject()
{
	if(!KOpenDialog(m_Sheet, this)) KFAIL(SDP_E_FAIL);
	return SDP_S_OK;
}




///////////////////////////////////////////////////////////////
// k3dIDocumentEventClient overrides

sdpResult CJoinPolys::OnUndo(k3dIObject* Sender)
{
	KillAll();
	m_Polygon1 = NULL;
	m_Polygon2 = NULL;
	m_Point1 = NULL;
	m_Point2 = NULL;

	m_Sheet->OnModify();

	return SDP_S_OK;
}




///////////////////////////////////////////////////////////////
// k3dIOpenGLRender overrides

sdpResult CJoinPolys::OpenGLRender(k3dICamera* Camera, k3dIOpenGLEngine* Engine)
{
	sdpResult hr;
	double coords[3];

	kglPushAttrib(GL_ALL_ATTRIB_BITS);
	kglDisable(GL_LIGHTING);

	kglColor3d(1.0, 0.0, 0.0);
	kglLineWidth(1.0f);
	kglPointSize(4.0f);

	// Draw first polygon
	kglColor3d(1.0, 0.0, 0.0);

	if(m_Polygon1)
		{
			m_PolygonList->SetHandle(m_Polygon1);

			kglBegin(GL_LINE_LOOP);

			for(hr = m_Polygon->SetFirstPoint(); SDP_S_OK == hr; hr = m_Polygon->SetNextPoint())
				{
					m_PointList->SetHandle(m_Polygon->GetPointHandle());
					m_PointLocation->GetAnimXYZ(coords);
					kglVertex3d(coords[0], coords[1], coords[2]);
				}

			kglEnd();
		}

	// Draw second polygon
	kglColor3d(1.0, 0.5, 0.0);

	if(m_Polygon2)
		{
			m_PolygonList->SetHandle(m_Polygon2);

			kglBegin(GL_LINE_LOOP);

			for(hr = m_Polygon->SetFirstPoint(); SDP_S_OK == hr; hr = m_Polygon->SetNextPoint())
				{
					m_PolygonLocation->GetAnimXYZ(coords);
					kglVertex3d(coords[0], coords[1], coords[2]);
				}

			kglEnd();
		}

	// Draw selected points
	kglBegin(GL_POINTS);

	kglColor3d(1.0, 0.0, 0.0);
	if(m_Point1)
		{
			m_PointList->SetHandle(m_Point1);
			m_PointLocation->GetAnimXYZ(coords);
			kglVertex3d(coords[0], coords[1], coords[2]);
		}
	if(m_Point3)
		{
			m_PointList->SetHandle(m_Point3);
			m_PointLocation->GetAnimXYZ(coords);
			kglVertex3d(coords[0], coords[1], coords[2]);
		}

	kglColor3d(1.0, 0.5, 0.0);
	if(m_Point2)
		{
			m_PointList->SetHandle(m_Point2);
			m_PointLocation->GetAnimXYZ(coords);
			kglVertex3d(coords[0], coords[1], coords[2]);
		}
	if(m_Point4)
		{
			m_PointList->SetHandle(m_Point4);
			m_PointLocation->GetAnimXYZ(coords);
			kglVertex3d(coords[0], coords[1], coords[2]);
		}

	kglEnd();

	kglPopAttrib();

	return SDP_S_OK;
}




///////////////////////////////////////////////////////////////
// k3dIUserEventClient overrides

void CJoinPolysSheet::OnHelp()
{
	m_Object->StandardHelp("tools", "joinpolys.html");
}

/////////////////////////////////////////////////////////////////////////////
// CJoinPolys message handlers


sdpResult CJoinPolys::OnLButtonClick(long Flags, long X, long Y)
{
	sdpResult hr;
	sdpVector3 screencoords;

	KUserEvent::OnLButtonClick(Flags, X, Y);

	m_Document->GetActiveView()->Camera()->SnapshotScreenCoordinates();

	long clicksensitivity = 4;

	for(k3dIPolygon* poly = m_Document->GetFirstPolygon(); poly; poly = m_Document->GetNextPolygon(poly))
		{
			if(SDP_S_OK == m_Polygon->QueryShow() && SDP_S_OK == m_Polygon->QuerySelectable())
				{
					for(hr = m_Polygon->SetFirstPoint(); SDP_S_OK == hr; hr = m_Polygon->SetNextPoint())
						{
							m_PolygonLocation->GetScreenXYZ(screencoords);

							if((abs(screencoords[0] - CurrentMouse()[0]) < clicksensitivity) &&
								 (abs(screencoords[1] - CurrentMouse()[1]) < clicksensitivity))
								{
									if(TWOPOINT == m_JoinType)
										{
											if(m_Polygon1)
												{
													if(m_Polygon->GetHandle() == m_Polygon1)
														{
															m_IO->Error("Must select a point on a different polygon", "JoinPolys:");
															return SDP_S_OK;
														}
													else
														{
															m_Polygon2 = m_Polygon->GetHandle();
															m_Point2 = m_Polygon->GetPointHandle();
															CreatePolygon();
															m_Sheet->OnModify();
															return SDP_S_OK;
														}
												}
											else
												{
													m_Polygon1 = m_Polygon->GetHandle();
													m_Point1 = m_Polygon->GetPointHandle();
													m_Sheet->OnModify();
													return SDP_S_OK;
												}
										}
									else if(FOURPOINT == m_JoinType)
										{
											if(m_Point3)
												{
													if(m_Polygon->GetHandle() != m_Polygon2)
														{
															m_IO->Error("Must select a point on the second polygon", "JoinPolys:");
															return SDP_S_OK;
														}
													else
														{
															m_Point4 = m_Polygon->GetPointHandle();
															CreatePolygon();
															m_Sheet->OnModify();
															return SDP_S_OK;
														}
												}
											else if(m_Point2)
												{
													if(m_Polygon->GetHandle() != m_Polygon1)
														{
															m_IO->Error("Must select a point on the first polygon", "JoinPolys:");
															return SDP_S_OK;
														}
													else
														{
															m_Point3 = m_Polygon->GetPointHandle();
															m_Sheet->OnModify();
															return SDP_S_OK;
														}
												}
											else if(m_Point1)
												{
													if(m_Polygon->GetHandle() == m_Polygon1)
														{
															m_IO->Error("Must select a point on a different polygon", "JoinPolys:");
															return SDP_S_OK;
														}
													else
														{
															m_Polygon2 = m_Polygon->GetHandle();
															m_Point2 = m_Polygon->GetPointHandle();
															m_Sheet->OnModify();
															return SDP_S_OK;
														}
												}
											else
												{
													m_Polygon1 = m_Polygon->GetHandle();
													m_Point1 = m_Polygon->GetPointHandle();
													m_Sheet->OnModify();
													return SDP_S_OK;
												}
										}
								}
						}
				}
		}

	m_Sheet->OnModify();

	return SDP_S_OK;
}


void CJoinPolys::CreatePolygon()
{
	sdpResult hr;

	k3dIPolygon* polygon;


	if(TWOPOINT == m_JoinType)
		{
			if(m_Polygon1 && m_Polygon2 && m_Point1 && m_Point2)
				{
					if(sdpSucceeded(m_Document->CreateObject(KPolygonEnumerator, IID_k3dIPolygon, FALSE, FALSE, FALSE, (void**)&polygon)))
						{
							if(sdpSucceeded(polygon->QueryInterface(IID_k3dIList, (void**)&polygonlist)))
								{
									// Allocate new polygon
									m_PolygonList->Create();
									m_PolygonArray.push_back(m_Polygon->GetHandle());
									m_Polygon->SetSelectable(FALSE);

									// Add points to polygon
									polygonlist->SetHandle(m_Polygon1);
									for(hr = polygon->SetFirstPoint(); SDP_S_OK == hr; hr = polygon->SetNextPoint())
										{
											m_Polygon->AttachPointAfter(polygon->GetPointHandle());

											if(polygon->GetPointHandle() == m_Point1) break;
										}
									polygonlist->SetHandle(m_Polygon2);
									for(hr = polygon->SetFirstPoint(); SDP_S_OK == hr; hr = polygon->SetNextPoint())
										{
											if(polygon->GetPointHandle() == m_Point2) break;
										}
									for(; SDP_S_OK == hr; hr = polygon->SetNextPoint())
										{
											m_Polygon->AttachPointAfter(polygon->GetPointHandle());
										}
									for(hr = polygon->SetFirstPoint(); SDP_S_OK == hr; hr = polygon->SetNextPoint())
										{
											m_Polygon->AttachPointAfter(polygon->GetPointHandle());

											if(polygon->GetPointHandle() == m_Point2) break;
										}
									polygonlist->SetHandle(m_Polygon1);
									for(hr = polygon->SetFirstPoint(); SDP_S_OK == hr; hr = polygon->SetNextPoint())
										{
											if(polygon->GetPointHandle() == m_Point1) break;
										}
									for(; SDP_S_OK == hr; hr = polygon->SetNextPoint())
										{
											m_Polygon->AttachPointAfter(polygon->GetPointHandle());
										}

									m_Polygon1 = NULL;
									m_Polygon2 = NULL;
									m_Point1 = NULL;
									m_Point2 = NULL;

									polygonlist->Release();
								}
							polygon->Release();
						}
				}
		}
	else if(FOURPOINT == m_JoinType)
		{
			if(m_Polygon1 && m_Polygon2 && m_Point1 && m_Point2 && m_Point3 && m_Point4)
				{
					hr = m_Document->CreateObject(KPolygonEnumerator, IID_k3dIPolygon, FALSE, FALSE, FALSE, (void**)&polygon);
					if(sdpSucceeded(hr))
						{
							// Allocate new polygon
							m_PolygonList->Create();
							m_PolygonArray.push_back(m_Polygon->GetHandle());
							m_Polygon->SetSelectable(FALSE);

							// Add points to polygon
							polygonlist->SetHandle(m_Polygon1);
							for(hr = polygon->SetFirstPoint(); SDP_S_OK == hr; hr = polygon->SetNextPoint())
								{
									m_Polygon->AttachPointAfter(polygon->GetPointHandle());

									if(polygon->GetPointHandle() == m_Point1) break;
								}
							polygonlist->SetHandle(m_Polygon2);
							for(hr = polygon->SetFirstPoint(); SDP_S_OK == hr; hr = polygon->SetNextPoint())
								{
									if(polygon->GetPointHandle() == m_Point2) break;
								}
							for(; SDP_S_OK == hr; hr = polygon->SetNextPoint())
								{
									m_Polygon->AttachPointAfter(polygon->GetPointHandle());

									if(polygon->GetPointHandle() == m_Point4) break;
								}
							if(SDP_S_OK != hr)
								{
									for(hr = polygon->SetFirstPoint(); SDP_S_OK == hr; hr = polygon->SetNextPoint())
										{
											m_Polygon->AttachPointAfter(polygon->GetPointHandle());

											if(polygon->GetPointHandle() == m_Point4) break;
										}
								}
							polygonlist->SetHandle(m_Polygon1);
							for(hr = polygon->SetFirstPoint(); SDP_S_OK == hr; hr = polygon->SetNextPoint())
								{
									if(polygon->GetPointHandle() == m_Point3) break;
								}
							for(; SDP_S_OK == hr; hr = polygon->SetNextPoint())
								{
									m_Polygon->AttachPointAfter(polygon->GetPointHandle());
								}


							m_Polygon1 = NULL;
							m_Polygon2 = NULL;
							m_Point1 = NULL;
							m_Point2 = NULL;
							m_Point3 = NULL;
							m_Point4 = NULL;

							polygon->Release();
						}
				}
		}
	m_Sheet->OnModify();
}


void CJoinPolys::OnModify()
{
	// Skip the k3dPrimTool implementation
	k3dTool::OnModify();
}



void CJoinPolys::OnReset()
{
	m_Point1 = NULL;
	m_Point2 = NULL;
	m_Point3 = NULL;
	m_Point4 = NULL;
	m_Polygon1 = NULL;
	m_Polygon2 = NULL;

	m_Sheet->OnModify();
}



////////////////////////////////////////////////////////////////////////////
// CJoinPolysSheet

CJoinPolysSheet::CJoinPolysSheet(CJoinPolys* Object)
: KPrimSheet(Object, theApp.m_hInstance, "JoinPolys", VERTICAL, "JoinPolys", TRUE)
{
	m_JoinPolysPage = NULL;
	m_Object = Object;
}


CJoinPolysSheet::~CJoinPolysSheet()
{
	m_Object->m_Sheet = NULL;
}



bool CJoinPolysSheet::Create()
{
	m_JoinPolysPage = new CJoinPolysPage(this);

	if(m_JoinPolysPage)
		{
			AddPage(m_JoinPolysPage);

			Create(IDD);
			return TRUE;
		}

	KFAIL(FALSE);
}


BEGIN_MESSAGE_MAP(CJoinPolysSheet, KPrimSheet)
	//{{AFX_MSG_MAP(CJoinPolysSheet)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



