#ifndef WAVEIZER_H
#define WAVEIZER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Defines the CWaveizer K-3D object, which applies an animated "wave" transformation to geometry
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <k3dpersistobject.h>
#include <k3ddialog.h>
#include <k3dchannelbutton.h>
#include <k3dvariable.h>

// Forward class references ...
class CWaveizer;

/////////////////////////////////////////////////////////////////////////////////////
// CWaveizerDialog

/// Encapsulates the Waveizer UI
class CWaveizerDialog : public k3dDialog
{
friend class CWaveizer;
public:
	CWaveizerDialog();
	
	/// Called to initialize the UI
	bool Create(CWaveizer* Object);

protected:
	/// Stores a pointer to the owning object
	CWaveizer* m_Object;

	/// Handles our custom events
	void OnEvent(sdpGtkEvent* Event);

	void OnDisplaceX();
	void OnDisplaceY();
	void OnDisplaceZ();
	
	void OnAlongX();
	void OnAlongY();
	void OnAlongZ();

	/// Called to initialize controls at creation time
	bool InitializeControls();

	/// Called to update the state of any UI controls
	void UpdateControls();
};

////////////////////////////////////////////////////////////////////////////////////
// CWaveizer

/// Encapsulates the Waveizer object
class CWaveizer : public k3dPersistObject
{
friend class CWaveizerDialog;
friend class CWaveizerWaveizeObject;
public:
	CWaveizer();
	~CWaveizer();

	// k3dIUnknown overrides
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface);
	gulong AddRef();
	gulong Release();

	// k3dIObject overrides
	sdpResult EditObject();
	sdpResult ObjectRenamed(SDPCSTRING NewName);
	sdpResult ObjectDeleted();

	// k3dIAnimate overrides
	sdpResult PopAnimate();

	// k3dIPersist overrides
	sdpResult InitNew();

protected:
	typedef enum
	{
		X,
		Y,
		Z,
	} Axis;
	
	class VariableAxis : public k3dCustomVariable<Axis>
	{
	protected:
		sdpString ConvertToString() { return sdpFormatString("%d", m_Value); }
		bool ConvertFromString(SDPCSTRING String) { m_Value = Axis(atoi(String)); return true; }
	};

	k3dVariable<bool> m_DisplaceX;
	k3dVariable<bool> m_DisplaceY;
	k3dVariable<bool> m_DisplaceZ;

	VariableAxis m_Along;

//	k3dVariable<bool> m_FiniteRange;

	k3dScalarChannel m_Amplitude;
	k3dScalarChannel m_Wavelength;
	k3dScalarChannel m_Phase;
//	k3dScalarChannel m_Range;
//	k3dScalarChannel m_Falloff;
	
	CWaveizerDialog m_UserInterface;
};

#endif // WAVEIZER_H
