#include <iostream>
#include <string>

#include "test.h"

//////////////////////////////////////////////////////////////////////
// CTestDocument

CTestDocument::CTestDocument()
{
	std::cout << "CTestDocument::CTestDocument()" << std::endl;
}

CTestDocument::~CTestDocument()
{
	std::cout << "CTestDocument::~CTestDocument()" << std::endl;
}

const char* CTestDocument::Path()
{
	return "/tmp/test.k3d";
}

//////////////////////////////////////////////////////////////////////
// The JavaScript TestDocument object ...

JSClass CJavaScriptTestDocument::Class =
{
	"TestDocument", JSCLASS_HAS_PRIVATE,
	AddProperty, DeleteProperty, GetProperty, SetProperty, Enumerate, Resolve, Convert, Finalize
};

JSPropertySpec CJavaScriptTestDocument::Properties[] =
{
	{ 0 }
};

JSFunctionSpec CJavaScriptTestDocument::Methods[] =
{
	{ "Path", Path, 1, 0, 0 },
	{ 0 }
};

JSObject* CJavaScriptTestDocument::Create(CTestDocument* TestDocument, JSContext* Context, JSObject* TestApplicationObject)
{
	JSObject* result = JS_NewObject(Context, &Class, 0, 0);
	JS_SetPrivate(Context, result, TestDocument);
	JS_DefineProperties(Context, result, Properties);
	JS_DefineFunctions(Context, result, Methods);

	return result;
}

CTestDocument* CJavaScriptTestDocument::TestDocument(JSContext* Context, JSObject* Object)
{
	return reinterpret_cast<CTestDocument*>(JS_GetPrivate(Context, Object));
}

JSBool CJavaScriptTestDocument::AddProperty(JSContext* Context, JSObject* Object, jsval ID, jsval* Value)
{
	return JS_TRUE;
}

JSBool CJavaScriptTestDocument::DeleteProperty(JSContext* Context, JSObject* Object, jsval ID, jsval* Value)
{
	return JS_TRUE;
}

JSBool CJavaScriptTestDocument::GetProperty(JSContext* Context, JSObject* Object, jsval ID, jsval* Value)
{
	return JS_TRUE;
}

JSBool CJavaScriptTestDocument::SetProperty(JSContext* Context, JSObject* Object, jsval ID, jsval* Value)
{
	return JS_TRUE;
}

JSBool CJavaScriptTestDocument::Enumerate(JSContext* Context, JSObject* Object)
{
	return JS_TRUE;
}

JSBool CJavaScriptTestDocument::Resolve(JSContext* Context, JSObject* Object, jsval ID)
{
	return JS_TRUE;
}

JSBool CJavaScriptTestDocument::Convert(JSContext* Context, JSObject* Object, JSType Type, jsval* Value)
{
	return JS_TRUE;
}

void CJavaScriptTestDocument::Finalize(JSContext* Context, JSObject* Object)
{
std::cout << "jsdoc::finalize" << std::endl;
}

JSBool CJavaScriptTestDocument::Path(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result)
{
	*Result = STRING_TO_JSVAL(JS_NewStringCopyZ(Context, TestDocument(Context, Object)->Path()));
	return JS_TRUE;
}


//////////////////////////////////////////////////////////////////////
// CTestApplication

CTestApplication::CTestApplication()
{
	std::cout << "CTestApplication::CTestApplication()" << std::endl;
}

CTestApplication::~CTestApplication()
{
	std::cout << "CTestApplication::~CTestApplication()" << std::endl;

	for(TestDocumentIterator document = m_TestDocuments.begin(); document != m_TestDocuments.end(); document++)
		delete(*document);
}

CTestDocument* CTestApplication::CreateTestDocument()
{
	CTestDocument* result = new CTestDocument();
	m_TestDocuments.push_back(result);
	
	return result;
}

CTestDocument* CTestApplication::ActiveTestDocument()
{
	if(m_TestDocuments.size())
		return m_TestDocuments.back();
	else
		return 0;
}

void CTestApplication::Message(const char* Message)
{
	std::cout << Message << std::endl;
}

CTestApplication TestApplication;

//////////////////////////////////////////////////////////////////////
// The JavaScript TestApplication object ...

JSClass CJavaScriptTestApplication::Class =
{
	"TestApplication", JSCLASS_HAS_PRIVATE,
	AddProperty, DeleteProperty, GetProperty, SetProperty, Enumerate, Resolve, Convert, Finalize
};

JSPropertySpec CJavaScriptTestApplication::Properties[] =
{
	{ 0 }
};

JSFunctionSpec CJavaScriptTestApplication::Methods[] =
{
	{ "CreateTestDocument", CreateTestDocument, 0, 0, 0 },
	{ "ActiveTestDocument", ActiveTestDocument, 0, 0, 0 },
	{ "Message", Message, 1, 0, 0 },
	{ 0 }
};

JSObject* CJavaScriptTestApplication::Create(CTestApplication* TestApplication, JSContext* Context, JSObject* GlobalObject)
{
	JSObject* result = JS_DefineObject(Context, GlobalObject, "TestApplication", &Class, NULL, 0);
	JS_SetPrivate(Context, result, TestApplication);
	JS_DefineProperties(Context, result, Properties);
	JS_DefineFunctions(Context, result, Methods);

	return result;
}

CTestApplication* CJavaScriptTestApplication::TestApplication(JSContext* Context, JSObject* Object)
{
	return reinterpret_cast<CTestApplication*>(JS_GetPrivate(Context, Object));
}

JSBool CJavaScriptTestApplication::AddProperty(JSContext* Context, JSObject* Object, jsval ID, jsval* Value)
{
	return JS_TRUE;
}

JSBool CJavaScriptTestApplication::DeleteProperty(JSContext* Context, JSObject* Object, jsval ID, jsval* Value)
{
	return JS_TRUE;
}

JSBool CJavaScriptTestApplication::GetProperty(JSContext* Context, JSObject* Object, jsval ID, jsval* Value)
{
	return JS_TRUE;
}

JSBool CJavaScriptTestApplication::SetProperty(JSContext* Context, JSObject* Object, jsval ID, jsval* Value)
{
	return JS_TRUE;
}

JSBool CJavaScriptTestApplication::Enumerate(JSContext* Context, JSObject* Object)
{
	return JS_TRUE;
}

JSBool CJavaScriptTestApplication::Resolve(JSContext* Context, JSObject* Object, jsval ID)
{
	return JS_TRUE;
}

JSBool CJavaScriptTestApplication::Convert(JSContext* Context, JSObject* Object, JSType Type, jsval* Value)
{
	return JS_TRUE;
}

void CJavaScriptTestApplication::Finalize(JSContext* Context, JSObject* Object)
{
}

JSBool CJavaScriptTestApplication::CreateTestDocument(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result)
{
	CTestDocument* document = TestApplication(Context, Object)->CreateTestDocument();
	*Result = OBJECT_TO_JSVAL(CJavaScriptTestDocument::Create(document, Context, Object));

	return JS_TRUE;
}

JSBool CJavaScriptTestApplication::ActiveTestDocument(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result)
{
	CTestDocument* document = TestApplication(Context, Object)->ActiveTestDocument();
	*Result = OBJECT_TO_JSVAL(CJavaScriptTestDocument::Create(document, Context, Object));

	return JS_TRUE;
}

JSBool CJavaScriptTestApplication::Message(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result)
{
	TestApplication(Context, Object)->Message(JS_GetStringBytes(JS_ValueToString(Context, argv[0])));
	return JS_TRUE;
}


/////////////////////////////////////////////////////////////////////
// main()

int main(int argc, char* argv[])
{
	// Fire up the interpreter ...
	JSRuntime* runtime = JS_NewRuntime(0x100000);
	JSContext* context = JS_NewContext(runtime, 0x1000); 

	// Create the global object and core JS classes ...
	JSClass global_class = { "global", 0, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub };
	JSObject* global = JS_NewObject(context, &global_class, NULL, NULL); 
	JS_InitStandardClasses(context, global);

	// Create the application object ...
	CJavaScriptTestApplication::Create(&TestApplication, context, global);

	// Evaluate a simple script ...
	jsval result;
	std::string script("var app = TestApplication; var doc = app.CreateTestDocument(); app.Message(doc.Path()); app.Message(doc.Path())");
	JS_EvaluateScript(context, global, script.c_str(), script.size(), "test", 0, &result); 

//	std::cout << "script result: " << JS_GetStringBytes(JS_ValueToString(context, result)) << std::endl;

	// Shut down the interpreter
	JS_DestroyContext(context);
	JS_Finish(runtime);

	return 0;
}
