#ifndef TEST_H
#define TEST_H

#include <jsapi.h>

#include <vector>

// Forward class declarations
class CTestApplication;
class CTestDocument;

///////////////////////////////////////////////////////////////////////////
// CTestDocument

class CTestDocument
{
public:
	CTestDocument();
	~CTestDocument();

	const char* Path();
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptTestDocument

class CJavaScriptTestDocument
{
public:
	static JSObject* Create(CTestDocument* TestDocument, JSContext* Context, JSObject* TestApplicationObject);
	static CTestDocument* TestDocument(JSContext* Context, JSObject* Object);

protected:
	typedef enum
	{
	} PropertyID;

	static JSClass Class;
	static JSPropertySpec Properties[];
	static JSFunctionSpec Methods[];

	static JSBool AddProperty(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool DeleteProperty(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetProperty(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool SetProperty(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool Enumerate(JSContext* Context, JSObject* Object);
	static JSBool Resolve(JSContext* Context, JSObject* Object, jsval ID);
	static JSBool Convert(JSContext* Context, JSObject* Object, JSType Type, jsval* Value);
	static void Finalize(JSContext* Context, JSObject* Object);

	static JSBool Path(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

///////////////////////////////////////////////////////////////////////////
// CTestApplication

class CTestApplication
{
public:
	CTestApplication();
	~CTestApplication();

	CTestDocument* CreateTestDocument();
	CTestDocument* ActiveTestDocument();
	void Message(const char* Message);

protected:
	typedef std::vector<CTestDocument*> TestDocuments;
	typedef TestDocuments::iterator TestDocumentIterator;
	TestDocuments m_TestDocuments;
};

extern CTestApplication TestApplication;

///////////////////////////////////////////////////////////////////////////
// CJavaScriptTestApplication

class CJavaScriptTestApplication
{
public:
	static JSObject* Create(CTestApplication* TestApplication, JSContext* Context, JSObject* GlobalObject);
	static CTestApplication* TestApplication(JSContext* Context, JSObject* Object);

protected:
	typedef enum
	{
	} PropertyID;

	static JSClass Class;
	static JSPropertySpec Properties[];
	static JSFunctionSpec Methods[];

	static JSBool AddProperty(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool DeleteProperty(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetProperty(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool SetProperty(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool Enumerate(JSContext* Context, JSObject* Object);
	static JSBool Resolve(JSContext* Context, JSObject* Object, jsval ID);
	static JSBool Convert(JSContext* Context, JSObject* Object, JSType Type, jsval* Value);
	static void Finalize(JSContext* Context, JSObject* Object);

	static JSBool CreateTestDocument(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool ActiveTestDocument(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool Message(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};


#endif // TEST_H
