// SDPGL
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares SDPGL functions that ARE related to OpenGL tracing
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPGLTRACEPROTOTYPES_H
#define SDPGLTRACEPROTOTYPES_H

/// Sets the desired output stream for tracing (std::cout is the default)
void sdpglSetStream(const std::ostream* Stream);	
/// Pushes a new trace state onto the stack (tracing is enabled by default)
void sdpglPushTrace(bool State);						
/// Pops a trace state off the stack ...
void sdpglPopTrace();
/// Sends a string to the output stream (obeys tracing state)
void sdpglTrace(SDPCSTRING Text);						

void sdpglAccum (GLenum op, GLfloat value);
void sdpglAlphaFunc (GLenum func, GLclampf ref);
GLboolean sdpglAreTexturesResident (GLsizei n, const GLuint *textures, GLboolean *residences);
void sdpglArrayElement (GLint i);
void sdpglBegin (GLenum mode);
void sdpglBindTexture (GLenum target, GLuint texture);
void sdpglBitmap (GLsizei width, GLsizei height, GLfloat xorig, GLfloat yorig, GLfloat xmove, GLfloat ymove, const GLubyte *bitmap);
void sdpglBlendFunc (GLenum sfactor, GLenum dfactor);
void sdpglCallList (GLuint list);
void sdpglCallLists (GLsizei n, GLenum type, const GLvoid *lists);
void sdpglClear (GLbitfield mask);
void sdpglClearAccum (GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha);
void sdpglClearColor (GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha);
void sdpglClearDepth (GLclampd depth);
void sdpglClearIndex (GLfloat c);
void sdpglClearStencil (GLint s);
void sdpglClipPlane (GLenum plane, const GLdouble *equation);
void sdpglColor3b (GLbyte red, GLbyte green, GLbyte blue);
void sdpglColor3bv (const GLbyte *v);
void sdpglColor3d (GLdouble red, GLdouble green, GLdouble blue);
void sdpglColor3dv (const GLdouble *v);
void sdpglColor3f (GLfloat red, GLfloat green, GLfloat blue);
void sdpglColor3fv (const GLfloat *v);
void sdpglColor3i (GLint red, GLint green, GLint blue);
void sdpglColor3iv (const GLint *v);
void sdpglColor3s (GLshort red, GLshort green, GLshort blue);
void sdpglColor3sv (const GLshort *v);
void sdpglColor3ub (GLubyte red, GLubyte green, GLubyte blue);
void sdpglColor3ubv (const GLubyte *v);
void sdpglColor3ui (GLuint red, GLuint green, GLuint blue);
void sdpglColor3uiv (const GLuint *v);
void sdpglColor3us (GLushort red, GLushort green, GLushort blue);
void sdpglColor3usv (const GLushort *v);
void sdpglColor4b (GLbyte red, GLbyte green, GLbyte blue, GLbyte alpha);
void sdpglColor4bv (const GLbyte *v);
void sdpglColor4d (GLdouble red, GLdouble green, GLdouble blue, GLdouble alpha);
void sdpglColor4dv (const GLdouble *v);
void sdpglColor4f (GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha);
void sdpglColor4fv (const GLfloat *v);
void sdpglColor4i (GLint red, GLint green, GLint blue, GLint alpha);
void sdpglColor4iv (const GLint *v);
void sdpglColor4s (GLshort red, GLshort green, GLshort blue, GLshort alpha);
void sdpglColor4sv (const GLshort *v);
void sdpglColor4ub (GLubyte red, GLubyte green, GLubyte blue, GLubyte alpha);
void sdpglColor4ubv (const GLubyte *v);
void sdpglColor4ui (GLuint red, GLuint green, GLuint blue, GLuint alpha);
void sdpglColor4uiv (const GLuint *v);
void sdpglColor4us (GLushort red, GLushort green, GLushort blue, GLushort alpha);
void sdpglColor4usv (const GLushort *v);
void sdpglColorMask (GLboolean red, GLboolean green, GLboolean blue, GLboolean alpha);
void sdpglColorMaterial (GLenum face, GLenum mode);
void sdpglColorPointer (GLint size, GLenum type, GLsizei stride, const GLvoid *pointer);
void sdpglCopyPixels (GLint x, GLint y, GLsizei width, GLsizei height, GLenum type);
void sdpglCopyTexImage1D (GLenum target, GLint level, GLenum internalFormat, GLint x, GLint y, GLsizei width, GLint border);
void sdpglCopyTexImage2D (GLenum target, GLint level, GLenum internalFormat, GLint x, GLint y, GLsizei width, GLsizei height, GLint border);
void sdpglCopyTexSubImage1D (GLenum target, GLint level, GLint xoffset, GLint x, GLint y, GLsizei width);
void sdpglCopyTexSubImage2D (GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height);
void sdpglCullFace (GLenum mode);
void sdpglDeleteLists (GLuint list, GLsizei range);
void sdpglDeleteTextures (GLsizei n, const GLuint *textures);
void sdpglDepthFunc (GLenum func);
void sdpglDepthMask (GLboolean flag);
void sdpglDepthRange (GLclampd zNear, GLclampd zFar);
void sdpglDisable (GLenum cap);
void sdpglDisableClientState (GLenum array);
void sdpglDrawArrays (GLenum mode, GLint first, GLsizei count);
void sdpglDrawBuffer (GLenum mode);
void sdpglDrawElements (GLenum mode, GLsizei count, GLenum type, const GLvoid *indices);
void sdpglDrawPixels (GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid *pixels);
void sdpglEdgeFlag (GLboolean flag);
void sdpglEdgeFlagPointer (GLsizei stride, const GLvoid *pointer);
void sdpglEdgeFlagv (const GLboolean *flag);
void sdpglEnable (GLenum cap);
void sdpglEnableClientState (GLenum array);
void sdpglEnd (void);
void sdpglEndList (void);
void sdpglEvalCoord1d (GLdouble u);
void sdpglEvalCoord1dv (const GLdouble *u);
void sdpglEvalCoord1f (GLfloat u);
void sdpglEvalCoord1fv (const GLfloat *u);
void sdpglEvalCoord2d (GLdouble u, GLdouble v);
void sdpglEvalCoord2dv (const GLdouble *u);
void sdpglEvalCoord2f (GLfloat u, GLfloat v);
void sdpglEvalCoord2fv (const GLfloat *u);
void sdpglEvalMesh1 (GLenum mode, GLint i1, GLint i2);
void sdpglEvalMesh2 (GLenum mode, GLint i1, GLint i2, GLint j1, GLint j2);
void sdpglEvalPoint1 (GLint i);
void sdpglEvalPoint2 (GLint i, GLint j);
void sdpglFeedbackBuffer (GLsizei size, GLenum type, GLfloat *buffer);
void sdpglFinish (void);
void sdpglFlush (void);
void sdpglFogf (GLenum pname, GLfloat param);
void sdpglFogfv (GLenum pname, const GLfloat *params);
void sdpglFogi (GLenum pname, GLint param);
void sdpglFogiv (GLenum pname, const GLint *params);
void sdpglFrontFace (GLenum mode);
void sdpglFrustum (GLdouble left, GLdouble right, GLdouble bottom, GLdouble top, GLdouble zNear, GLdouble zFar);
GLuint sdpglGenLists (GLsizei range);
void sdpglGenTextures (GLsizei n, GLuint *textures);
void sdpglGetBooleanv (GLenum pname, GLboolean *params);
void sdpglGetClipPlane (GLenum plane, GLdouble *equation);
void sdpglGetDoublev (GLenum pname, GLdouble *params);
GLenum sdpglGetError (void);
void sdpglGetFloatv (GLenum pname, GLfloat *params);
void sdpglGetIntegerv (GLenum pname, GLint *params);
void sdpglGetLightfv (GLenum light, GLenum pname, GLfloat *params);
void sdpglGetLightiv (GLenum light, GLenum pname, GLint *params);
void sdpglGetMapdv (GLenum target, GLenum query, GLdouble *v);
void sdpglGetMapfv (GLenum target, GLenum query, GLfloat *v);
void sdpglGetMapiv (GLenum target, GLenum query, GLint *v);
void sdpglGetMaterialfv (GLenum face, GLenum pname, GLfloat *params);
void sdpglGetMaterialiv (GLenum face, GLenum pname, GLint *params);
void sdpglGetPixelMapfv (GLenum map, GLfloat *values);
void sdpglGetPixelMapuiv (GLenum map, GLuint *values);
void sdpglGetPixelMapusv (GLenum map, GLushort *values);
void sdpglGetPointerv (GLenum pname, GLvoid* *params);
void sdpglGetPolygonStipple (GLubyte *mask);
const char* sdpglGetString (GLenum name);
void sdpglGetTexEnvfv (GLenum target, GLenum pname, GLfloat *params);
void sdpglGetTexEnviv (GLenum target, GLenum pname, GLint *params);
void sdpglGetTexGendv (GLenum coord, GLenum pname, GLdouble *params);
void sdpglGetTexGenfv (GLenum coord, GLenum pname, GLfloat *params);
void sdpglGetTexGeniv (GLenum coord, GLenum pname, GLint *params);
void sdpglGetTexImage (GLenum target, GLint level, GLenum format, GLenum type, GLvoid *pixels);
void sdpglGetTexLevelParameterfv (GLenum target, GLint level, GLenum pname, GLfloat *params);
void sdpglGetTexLevelParameteriv (GLenum target, GLint level, GLenum pname, GLint *params);
void sdpglGetTexParameterfv (GLenum target, GLenum pname, GLfloat *params);
void sdpglGetTexParameteriv (GLenum target, GLenum pname, GLint *params);
void sdpglHint (GLenum target, GLenum mode);
void sdpglIndexMask (GLuint mask);
void sdpglIndexPointer (GLenum type, GLsizei stride, const GLvoid *pointer);
void sdpglIndexd (GLdouble c);
void sdpglIndexdv (const GLdouble *c);
void sdpglIndexf (GLfloat c);
void sdpglIndexfv (const GLfloat *c);
void sdpglIndexi (GLint c);
void sdpglIndexiv (const GLint *c);
void sdpglIndexs (GLshort c);
void sdpglIndexsv (const GLshort *c);
void sdpglIndexub (GLubyte c);
void sdpglIndexubv (const GLubyte *c);
void sdpglInitNames (void);
void sdpglInterleavedArrays (GLenum format, GLsizei stride, const GLvoid *pointer);
GLboolean sdpglIsEnabled (GLenum cap);
GLboolean sdpglIsList (GLuint list);
GLboolean sdpglIsTexture (GLuint texture);
void sdpglLightModelf (GLenum pname, GLfloat param);
void sdpglLightModelfv (GLenum pname, const GLfloat *params);
void sdpglLightModeli (GLenum pname, GLint param);
void sdpglLightModeliv (GLenum pname, const GLint *params);
void sdpglLightf (GLenum light, GLenum pname, GLfloat param);
void sdpglLightfv (GLenum light, GLenum pname, const GLfloat *params);
void sdpglLighti (GLenum light, GLenum pname, GLint param);
void sdpglLightiv (GLenum light, GLenum pname, const GLint *params);
void sdpglLineStipple (GLint factor, GLushort pattern);
void sdpglLineWidth (GLfloat width);
void sdpglListBase (GLuint base);
void sdpglLoadIdentity (void);
void sdpglLoadMatrixd (const GLdouble *m);
void sdpglLoadMatrixf (const GLfloat *m);
void sdpglLoadName (GLuint name);
void sdpglLogicOp (GLenum opcode);
void sdpglMap1d (GLenum target, GLdouble u1, GLdouble u2, GLint stride, GLint order, const GLdouble *points);
void sdpglMap1f (GLenum target, GLfloat u1, GLfloat u2, GLint stride, GLint order, const GLfloat *points);
void sdpglMap2d (GLenum target, GLdouble u1, GLdouble u2, GLint ustride, GLint uorder, GLdouble v1, GLdouble v2, GLint vstride, GLint vorder, const GLdouble *points);
void sdpglMap2f (GLenum target, GLfloat u1, GLfloat u2, GLint ustride, GLint uorder, GLfloat v1, GLfloat v2, GLint vstride, GLint vorder, const GLfloat *points);
void sdpglMapGrid1d (GLint un, GLdouble u1, GLdouble u2);
void sdpglMapGrid1f (GLint un, GLfloat u1, GLfloat u2);
void sdpglMapGrid2d (GLint un, GLdouble u1, GLdouble u2, GLint vn, GLdouble v1, GLdouble v2);
void sdpglMapGrid2f (GLint un, GLfloat u1, GLfloat u2, GLint vn, GLfloat v1, GLfloat v2);
void sdpglMaterialf (GLenum face, GLenum pname, GLfloat param);
void sdpglMaterialfv (GLenum face, GLenum pname, const GLfloat *params);
void sdpglMateriali (GLenum face, GLenum pname, GLint param);
void sdpglMaterialiv (GLenum face, GLenum pname, const GLint *params);
void sdpglMatrixMode (GLenum mode);
void sdpglMultMatrixd (const GLdouble *m);
void sdpglMultMatrixf (const GLfloat *m);
void sdpglNewList (GLuint list, GLenum mode);
void sdpglNormal3b (GLbyte nx, GLbyte ny, GLbyte nz);
void sdpglNormal3bv (const GLbyte *v);
void sdpglNormal3d (GLdouble nx, GLdouble ny, GLdouble nz);
void sdpglNormal3dv (const GLdouble *v);
void sdpglNormal3f (GLfloat nx, GLfloat ny, GLfloat nz);
void sdpglNormal3fv (const GLfloat *v);
void sdpglNormal3i (GLint nx, GLint ny, GLint nz);
void sdpglNormal3iv (const GLint *v);
void sdpglNormal3s (GLshort nx, GLshort ny, GLshort nz);
void sdpglNormal3sv (const GLshort *v);
void sdpglNormalPointer (GLenum type, GLsizei stride, const GLvoid *pointer);
void sdpglOrtho (GLdouble left, GLdouble right, GLdouble bottom, GLdouble top, GLdouble zNear, GLdouble zFar);
void sdpglPassThrough (GLfloat token);
void sdpglPixelMapfv (GLenum map, GLsizei mapsize, const GLfloat *values);
void sdpglPixelMapuiv (GLenum map, GLsizei mapsize, const GLuint *values);
void sdpglPixelMapusv (GLenum map, GLsizei mapsize, const GLushort *values);
void sdpglPixelStoref (GLenum pname, GLfloat param);
void sdpglPixelStorei (GLenum pname, GLint param);
void sdpglPixelTransferf (GLenum pname, GLfloat param);
void sdpglPixelTransferi (GLenum pname, GLint param);
void sdpglPixelZoom (GLfloat xfactor, GLfloat yfactor);
void sdpglPointSize (GLfloat size);
void sdpglPolygonMode (GLenum face, GLenum mode);
void sdpglPolygonOffset (GLfloat factor, GLfloat units);
void sdpglPolygonStipple (const GLubyte *mask);
void sdpglPopAttrib (void);
void sdpglPopClientAttrib (void);
void sdpglPopMatrix (void);
void sdpglPopName (void);
void sdpglPrioritizeTextures (GLsizei n, const GLuint *textures, const GLclampf *priorities);
void sdpglPushAttrib (GLbitfield mask);
void sdpglPushClientAttrib (GLbitfield mask);
void sdpglPushMatrix (void);
void sdpglPushName (GLuint name);
void sdpglRasterPos2d (GLdouble x, GLdouble y);
void sdpglRasterPos2dv (const GLdouble *v);
void sdpglRasterPos2f (GLfloat x, GLfloat y);
void sdpglRasterPos2fv (const GLfloat *v);
void sdpglRasterPos2i (GLint x, GLint y);
void sdpglRasterPos2iv (const GLint *v);
void sdpglRasterPos2s (GLshort x, GLshort y);
void sdpglRasterPos2sv (const GLshort *v);
void sdpglRasterPos3d (GLdouble x, GLdouble y, GLdouble z);
void sdpglRasterPos3dv (const GLdouble *v);
void sdpglRasterPos3f (GLfloat x, GLfloat y, GLfloat z);
void sdpglRasterPos3fv (const GLfloat *v);
void sdpglRasterPos3i (GLint x, GLint y, GLint z);
void sdpglRasterPos3iv (const GLint *v);
void sdpglRasterPos3s (GLshort x, GLshort y, GLshort z);
void sdpglRasterPos3sv (const GLshort *v);
void sdpglRasterPos4d (GLdouble x, GLdouble y, GLdouble z, GLdouble w);
void sdpglRasterPos4dv (const GLdouble *v);
void sdpglRasterPos4f (GLfloat x, GLfloat y, GLfloat z, GLfloat w);
void sdpglRasterPos4fv (const GLfloat *v);
void sdpglRasterPos4i (GLint x, GLint y, GLint z, GLint w);
void sdpglRasterPos4iv (const GLint *v);
void sdpglRasterPos4s (GLshort x, GLshort y, GLshort z, GLshort w);
void sdpglRasterPos4sv (const GLshort *v);
void sdpglReadBuffer (GLenum mode);
void sdpglReadPixels (GLint x, GLint y, GLsizei width, GLsizei height, GLenum format, GLenum type, GLvoid *pixels);
void sdpglRectd (GLdouble x1, GLdouble y1, GLdouble x2, GLdouble y2);
void sdpglRectdv (const GLdouble *v1, const GLdouble *v2);
void sdpglRectf (GLfloat x1, GLfloat y1, GLfloat x2, GLfloat y2);
void sdpglRectfv (const GLfloat *v1, const GLfloat *v2);
void sdpglRecti (GLint x1, GLint y1, GLint x2, GLint y2);
void sdpglRectiv (const GLint *v1, const GLint *v2);
void sdpglRects (GLshort x1, GLshort y1, GLshort x2, GLshort y2);
void sdpglRectsv (const GLshort *v1, const GLshort *v2);
GLint sdpglRenderMode (GLenum mode);
void sdpglRotated (GLdouble angle, GLdouble x, GLdouble y, GLdouble z);
void sdpglRotatef (GLfloat angle, GLfloat x, GLfloat y, GLfloat z);
void sdpglScaled (GLdouble x, GLdouble y, GLdouble z);
void sdpglScalef (GLfloat x, GLfloat y, GLfloat z);
void sdpglScissor (GLint x, GLint y, GLsizei width, GLsizei height);
void sdpglSelectBuffer (GLsizei size, GLuint *buffer);
void sdpglShadeModel (GLenum mode);
void sdpglStencilFunc (GLenum func, GLint ref, GLuint mask);
void sdpglStencilMask (GLuint mask);
void sdpglStencilOp (GLenum fail, GLenum zfail, GLenum zpass);
void sdpglTexCoord1d (GLdouble s);
void sdpglTexCoord1dv (const GLdouble *v);
void sdpglTexCoord1f (GLfloat s);
void sdpglTexCoord1fv (const GLfloat *v);
void sdpglTexCoord1i (GLint s);
void sdpglTexCoord1iv (const GLint *v);
void sdpglTexCoord1s (GLshort s);
void sdpglTexCoord1sv (const GLshort *v);
void sdpglTexCoord2d (GLdouble s, GLdouble t);
void sdpglTexCoord2dv (const GLdouble *v);
void sdpglTexCoord2f (GLfloat s, GLfloat t);
void sdpglTexCoord2fv (const GLfloat *v);
void sdpglTexCoord2i (GLint s, GLint t);
void sdpglTexCoord2iv (const GLint *v);
void sdpglTexCoord2s (GLshort s, GLshort t);
void sdpglTexCoord2sv (const GLshort *v);
void sdpglTexCoord3d (GLdouble s, GLdouble t, GLdouble r);
void sdpglTexCoord3dv (const GLdouble *v);
void sdpglTexCoord3f (GLfloat s, GLfloat t, GLfloat r);
void sdpglTexCoord3fv (const GLfloat *v);
void sdpglTexCoord3i (GLint s, GLint t, GLint r);
void sdpglTexCoord3iv (const GLint *v);
void sdpglTexCoord3s (GLshort s, GLshort t, GLshort r);
void sdpglTexCoord3sv (const GLshort *v);
void sdpglTexCoord4d (GLdouble s, GLdouble t, GLdouble r, GLdouble q);
void sdpglTexCoord4dv (const GLdouble *v);
void sdpglTexCoord4f (GLfloat s, GLfloat t, GLfloat r, GLfloat q);
void sdpglTexCoord4fv (const GLfloat *v);
void sdpglTexCoord4i (GLint s, GLint t, GLint r, GLint q);
void sdpglTexCoord4iv (const GLint *v);
void sdpglTexCoord4s (GLshort s, GLshort t, GLshort r, GLshort q);
void sdpglTexCoord4sv (const GLshort *v);
void sdpglTexCoordPointer (GLint size, GLenum type, GLsizei stride, const GLvoid *pointer);
void sdpglTexEnvf (GLenum target, GLenum pname, GLfloat param);
void sdpglTexEnvfv (GLenum target, GLenum pname, const GLfloat *params);
void sdpglTexEnvi (GLenum target, GLenum pname, GLint param);
void sdpglTexEnviv (GLenum target, GLenum pname, const GLint *params);
void sdpglTexGend (GLenum coord, GLenum pname, GLdouble param);
void sdpglTexGendv (GLenum coord, GLenum pname, const GLdouble *params);
void sdpglTexGenf (GLenum coord, GLenum pname, GLfloat param);
void sdpglTexGenfv (GLenum coord, GLenum pname, const GLfloat *params);
void sdpglTexGeni (GLenum coord, GLenum pname, GLint param);
void sdpglTexGeniv (GLenum coord, GLenum pname, const GLint *params);
void sdpglTexImage1D (GLenum target, GLint level, GLint internalformat, GLsizei width, GLint border, GLenum format, GLenum type, const GLvoid *pixels);
void sdpglTexImage2D (GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, const GLvoid *pixels);
void sdpglTexParameterf (GLenum target, GLenum pname, GLfloat param);
void sdpglTexParameterfv (GLenum target, GLenum pname, const GLfloat *params);
void sdpglTexParameteri (GLenum target, GLenum pname, GLint param);
void sdpglTexParameteriv (GLenum target, GLenum pname, const GLint *params);
void sdpglTexSubImage1D (GLenum target, GLint level, GLint xoffset, GLsizei width, GLenum format, GLenum type, const GLvoid *pixels);
void sdpglTexSubImage2D (GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid *pixels);
void sdpglTranslated (GLdouble x, GLdouble y, GLdouble z);
void sdpglTranslatef (GLfloat x, GLfloat y, GLfloat z);
void sdpglVertex2d (GLdouble x, GLdouble y);
void sdpglVertex2dv (const GLdouble *v);
void sdpglVertex2f (GLfloat x, GLfloat y);
void sdpglVertex2fv (const GLfloat *v);
void sdpglVertex2i (GLint x, GLint y);
void sdpglVertex2iv (const GLint *v);
void sdpglVertex2s (GLshort x, GLshort y);
void sdpglVertex2sv (const GLshort *v);
void sdpglVertex3d (GLdouble x, GLdouble y, GLdouble z);
void sdpglVertex3dv (const GLdouble *v);
void sdpglVertex3f (GLfloat x, GLfloat y, GLfloat z);
void sdpglVertex3fv (const GLfloat *v);
void sdpglVertex3i (GLint x, GLint y, GLint z);
void sdpglVertex3iv (const GLint *v);
void sdpglVertex3s (GLshort x, GLshort y, GLshort z);
void sdpglVertex3sv (const GLshort *v);
void sdpglVertex4d (GLdouble x, GLdouble y, GLdouble z, GLdouble w);
void sdpglVertex4dv (const GLdouble *v);
void sdpglVertex4f (GLfloat x, GLfloat y, GLfloat z, GLfloat w);
void sdpglVertex4fv (const GLfloat *v);
void sdpglVertex4i (GLint x, GLint y, GLint z, GLint w);
void sdpglVertex4iv (const GLint *v);
void sdpglVertex4s (GLshort x, GLshort y, GLshort z, GLshort w);
void sdpglVertex4sv (const GLshort *v);
void sdpglVertexPointer (GLint size, GLenum type, GLsizei stride, const GLvoid *pointer);
void sdpglViewport (GLint x, GLint y, GLsizei width, GLsizei height);

/*
int   kwglChoosePixelFormat       (HDC a, CONST PIXELFORMATDESCRIPTOR *b);
BOOL  kwglCopyContext             (HGLRC a, HGLRC b, UINT c);
HGLRC kwglCreateContext           (HDC a);
HGLRC kwglCreateLayerContext      (HDC a, int b);
BOOL  kwglDeleteContext           (HGLRC a);
BOOL  kwglDescribeLayerPlane      (HDC a, int b, int c, UINT d, LPLAYERPLANEDESCRIPTOR e);
int   kwglDescribePixelFormat     (HDC a, int b, UINT c, LPPIXELFORMATDESCRIPTOR d);
HGLRC kwglGetCurrentContext       (void);
HDC   kwglGetCurrentDC            (void);
PROC  kwglGetDefaultProcAddress   (LPCSTR a);
int   kwglGetLayerPaletteEntries  (HDC a, int b, int c, int d, COLORREF *e);
int   kwglGetPixelFormat          (HDC a);
PROC  kwglGetProcAddress          (LPCSTR a);
BOOL  kwglMakeCurrent             (HDC a, HGLRC b);
BOOL  kwglRealizeLayerPalette     (HDC a, int b, BOOL c);
int   kwglSetLayerPaletteEntries  (HDC a, int b, int c, int d, CONST COLORREF *e);
BOOL  kwglSetPixelFormat          (HDC a, int b, CONST PIXELFORMATDESCRIPTOR *c);
BOOL  kwglShareLists              (HGLRC a, HGLRC b);
BOOL  kwglSwapBuffers             (HDC a);
BOOL  kwglSwapLayerBuffers        (HDC a, UINT b);
BOOL  kwglUseFontBitmapsA         (HDC a, DWORD b, DWORD c, DWORD d);
BOOL  kwglUseFontBitmapsW         (HDC a, DWORD b, DWORD c, DWORD d);
BOOL  kwglUseFontOutlinesA        (HDC a, DWORD b, DWORD c, DWORD d, FLOAT e, FLOAT f, int g, LPGLYPHMETRICSFLOAT h);
BOOL  kwglUseFontOutlinesW        (HDC a, DWORD b, DWORD c, DWORD d, FLOAT e, FLOAT f, int g, LPGLYPHMETRICSFLOAT h);

*/

#endif // SDPGLTRACEPROTOTYPES_H
