// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the sampleclient application, a separate process whose UI is "grabbed" by the SDPGTK sample application
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <sdpgtkobjectcontainer.h>
#include <iostream>

// sdpGtkObjectContainer is the main SDPGTK class - without alteration,
// it encapsulates all the functionality required to load GTKML documents.
//
// We derive our own classes from it so we can override OnEvent() and
// handle our SDPGTK events.

//////////////////////////////////////////////////////////////////////////
// ClientWindow

/// The main window for the application
class ClientWindow : public sdpGtkObjectContainer
{
public:
	ClientWindow();

	/// Loads the GTKML template
	void Load();

protected:
	/// Handles events for this template
	void OnEvent(sdpGtkEvent* Event);
};

ClientWindow::ClientWindow()
{
}

void ClientWindow::Load()
{
	// Load the GTKML document ...
	sdpGtkObjectContainer::LoadFile("client.gtkml");

	// Make everything visible ...
	RootWidget().Show();
}

void ClientWindow::OnEvent(sdpGtkEvent* Event)
{
	// Handle events ...
	sdpGtkObjectContainer::OnEvent(Event);
}

//////////////////////////////////////////////////////////////////////////////////
// main

/// Initializes GTK+ and loads the main window
gint main(gint argc, gchar* argv[])
{
	if(argc != 2)
		{
			std::cerr << "You must specify the window ID on the command line!" << std::endl;
			return 1;
		}
		
	guint32 socketid = atol(argv[1]);

	gtk_init(&argc, &argv);

	sdpGtkPlug plug;
	plug.Create(socketid);

	ClientWindow content;
	content.Load();
	
	plug.Attach(content.RootWidget());
	plug.Show();

	gtk_main();

	return 0;
}


