
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the sdpGtkCalendar class, which encapsulates the GtkCalendar widget
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <sdpgtkcalendar.h>
#include <sdpxmlutility.h>
#include <sdpgtkutility.h>

sdpGtkCalendar::sdpGtkCalendar() : sdpGtkWidget()
{
}

sdpGtkCalendar::sdpGtkCalendar(GtkCalendar* Calendar) : sdpGtkWidget(GTK_WIDGET(Calendar))
{
}

bool sdpGtkCalendar::Create()
{
	m_Object = GTK_OBJECT(gtk_calendar_new());
	return Attached();
}

bool sdpGtkCalendar::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	return Create();
}

bool sdpGtkCalendar::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	if(sdpXmlFindAttribute(Element, "displayoptions"))
		{
			int displayoptions = 0;

			sdpString textdisplayoptions;
			sdpXmlGetAttribute(Element, "displayoptions", textdisplayoptions);
			sdpGtkMarkAttribute(Element, "displayoptions");

			if(textdisplayoptions.find("heading") != sdpString::npos)
				displayoptions |= GTK_CALENDAR_SHOW_HEADING;
			if(textdisplayoptions.find("daynames") != sdpString::npos)
				displayoptions |= GTK_CALENDAR_SHOW_DAY_NAMES;
			if(textdisplayoptions.find("nomonthchanges") != sdpString::npos)
				displayoptions |= GTK_CALENDAR_NO_MONTH_CHANGE;
			if(textdisplayoptions.find("weeknumbers") != sdpString::npos)
				displayoptions |= GTK_CALENDAR_SHOW_WEEK_NUMBERS;
			if(textdisplayoptions.find("startmonday") != sdpString::npos)
				displayoptions |= GTK_CALENDAR_WEEK_START_MONDAY;

			SetDisplayOptions(GtkCalendarDisplayOptions(displayoptions));
		}

	return sdpGtkWidget::CreateOptions(ObjectContainer, Element);
}

gint sdpGtkCalendar::SelectMonth(guint Month, guint Year)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return gtk_calendar_select_month(GTK_CALENDAR(m_Object), Month, Year);
}

void sdpGtkCalendar::SelectDay(guint Day)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_calendar_select_day(GTK_CALENDAR(m_Object), Day);
}

gint sdpGtkCalendar::MarkDay(guint Day)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return gtk_calendar_mark_day(GTK_CALENDAR(m_Object), Day);
}

gint sdpGtkCalendar::UnmarkDay(guint Day)
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return gtk_calendar_unmark_day(GTK_CALENDAR(m_Object), Day);
}

void sdpGtkCalendar::ClearMarks()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_calendar_clear_marks(GTK_CALENDAR(m_Object));
}

void sdpGtkCalendar::SetDisplayOptions(GtkCalendarDisplayOptions Options)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_calendar_display_options(GTK_CALENDAR(m_Object), Options);
}

void sdpGtkCalendar::GetDate(guint& Year, guint& Month, guint& Day)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_calendar_get_date(GTK_CALENDAR(m_Object), &Year, &Month, &Day);
}

void sdpGtkCalendar::Freeze()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_calendar_freeze(GTK_CALENDAR(m_Object));
}

void sdpGtkCalendar::Thaw()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_calendar_thaw(GTK_CALENDAR(m_Object));
}

