#include "sdpgtkevent.h"
#include "sdpgtkloopevents.h"
#include "sdpgtkevents.h"

#include <iostream>

////////////////////////////////////////////////////////////////////
// sdpGtkEvent

sdpGtkEvent* sdpGtkEvent::CreateEvent(SDPCSTRING EventSignal, SDPCSTRING EventName, sdpGtkObjectContainer* EventContainer, GtkObject* EventObject)
{
	// Create a new event object, based on the type ...
	const sdpString signal(EventSignal);

	// Our "special" objects ...
	if(signal == "quit")
		return new sdpGtkEventQuit(EventSignal, EventName, EventContainer);
	if(signal == "timeout")
		return new sdpGtkEventTimeout(EventSignal, EventName, EventContainer);
	if(signal == "idle")
		return new sdpGtkEventIdle(EventSignal, EventName, EventContainer);

// Machine-generated code that instantiates "normal" event objects ...
#include "sdpgtkeventcreation.h"

	std::cout << "sdpGtkEvent::CreateEvent(): Unknown event type \"" << signal << "\"!!!" << std::endl;

	return 0;
}

sdpGtkEvent::sdpGtkEvent(SDPCSTRING EventSignal, SDPCSTRING EventName, sdpGtkObjectContainer* EventContainer)
{
	// Sanity checks ...
	g_assert_string(EventSignal);
	g_assert_string(EventName);
	g_assert(EventContainer);

	// Assign data ...
	m_EventSignal = EventSignal;
	m_EventName = EventName;
	m_EventContainer = EventContainer;
	m_EventHandler = 0;
}


sdpGtkEvent::~sdpGtkEvent()
{
}

bool sdpGtkEvent::IsObjectEvent()
{
	return !IsLoopEvent();
}

bool sdpGtkEvent::IsLoopEvent()
{
	return (m_EventSignal == "quit" || m_EventSignal == "timeout" || m_EventSignal == "idle");
}

/////////////////////////////////////////////////////////////////////
// sdpGtkObjectEvent

sdpGtkObjectEvent::sdpGtkObjectEvent(SDPCSTRING EventSignal, SDPCSTRING EventName, sdpGtkObjectContainer* EventContainer, GtkObject* EventObject) : 
	sdpGtkEvent(EventSignal, EventName, EventContainer)
{
	// Sanity checks ...
	g_assert(EventObject);

	m_EventObject = EventObject;
}

void sdpGtkObjectEvent::Disconnect()
{
	// Sanity checks ...
	g_assert(m_EventHandler);

	gtk_signal_disconnect(m_EventObject, m_EventHandler);
	m_EventHandler = 0;
}

void sdpGtkObjectEvent::Block()
{
	// Sanity checks ...
	g_assert(m_EventHandler);

	gtk_signal_handler_block(m_EventObject, m_EventHandler);
}

void sdpGtkObjectEvent::Unblock()
{
	// Sanity checks ...
	g_assert(m_EventHandler);

	gtk_signal_handler_unblock(m_EventObject, m_EventHandler);
}


///////////////////////////////////////////////////////////////////////
// sdpGtkLoopEvent

sdpGtkLoopEvent::sdpGtkLoopEvent(SDPCSTRING EventSignal, SDPCSTRING EventName, sdpGtkObjectContainer* EventContainer) : 
	sdpGtkEvent(EventSignal, EventName, EventContainer)
{
}

gint sdpGtkLoopEvent::Event()
{
	m_EventContainer->OnEvent(this);
	return m_Result;
}

gint sdpGtkLoopEvent::RawEvent(gpointer EventData)
{
	return ((sdpGtkLoopEvent*)EventData)->Event();
}

bool sdpGtkLoopEvent::Result()
{
	return m_Result;
}

void sdpGtkLoopEvent::SetResult(bool Result)
{
	m_Result = Result;
}

