
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include <sdpgtkfixed.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>
#include <sdpmath.h>

sdpGtkFixed::sdpGtkFixed() : sdpGtkContainer()
{
}

sdpGtkFixed::sdpGtkFixed(GtkFixed* Fixed) : sdpGtkContainer(GTK_CONTAINER(Fixed))
{
}

bool sdpGtkFixed::Create()
{
	m_Object = GTK_OBJECT(gtk_fixed_new());
	return Attached();
}

bool sdpGtkFixed::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Create the widget ...
	return Create();
}

bool sdpGtkFixed::AttachChild(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* ChildElement, sdpGtkObject* Child)
{
	sdpVector2 position(0, 0);
	sdpXmlGetAttribute(ChildElement, "position", position);
	sdpGtkMarkAttribute(ChildElement, "position");

	Attach(GTK_WIDGET(Child->Object()), position[0], position[1]);
	return true;
}

void sdpGtkFixed::Attach(GtkWidget* Widget, gint16 X, gint16 Y)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(Widget);

	gtk_fixed_put(GTK_FIXED(m_Object), Widget, X, Y);
}

void sdpGtkFixed::Move(GtkWidget* Widget, gint16 X, gint16 Y)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(Widget);

	gtk_fixed_move(GTK_FIXED(m_Object), Widget, X, Y);
}
