
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtkhandlebox.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>

sdpGtkHandleBox::sdpGtkHandleBox() : sdpGtkBin()
{
}

sdpGtkHandleBox::sdpGtkHandleBox(GtkHandleBox* HandleBox) : sdpGtkBin(GTK_BIN(HandleBox))
{
}

bool sdpGtkHandleBox::Create()
{
	m_Object = GTK_OBJECT(gtk_handle_box_new());
	return Attached();
}

bool sdpGtkHandleBox::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Create the control ...
	return Create();
}

bool sdpGtkHandleBox::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Shadow type ...
	SetShadowType(sdpGtkGetShadowType(GTK_SHADOW_ETCHED_IN, Element));

	// Handle position ...
	sdpString handlepositiontext = "top";
	sdpXmlGetAttribute(Element, "handleposition", handlepositiontext);
	sdpGtkMarkAttribute(Element, "handleposition");

	GtkPositionType handleposition;
	if(handlepositiontext == "left")
		handleposition = GTK_POS_LEFT;
	else if(handlepositiontext == "right")
		handleposition = GTK_POS_RIGHT;
	else if(handlepositiontext == "top")
		handleposition = GTK_POS_TOP;
	else if(handlepositiontext == "bottom")
		handleposition = GTK_POS_BOTTOM;
	else
		g_assert_not_reached(); // Unknown shadow type!

	SetHandlePosition(handleposition);

	// Snap Edge ...
	sdpString snapedgetext = "top";
	sdpXmlGetAttribute(Element, "snapedge", snapedgetext);
	sdpGtkMarkAttribute(Element, "snapedge");

	GtkPositionType snapedge;
	if(snapedgetext == "left")
		snapedge = GTK_POS_LEFT;
	else if(snapedgetext == "right")
		snapedge = GTK_POS_RIGHT;
	else if(snapedgetext == "top")
		snapedge = GTK_POS_TOP;
	else if(snapedgetext == "bottom")
		snapedge = GTK_POS_BOTTOM;
	else
		g_assert_not_reached(); // Unknown shadow type!

	SetSnapEdge(snapedge);

	return sdpGtkBin::CreateOptions(ObjectContainer, Element);
}

void sdpGtkHandleBox::SetShadowType(GtkShadowType ShadowType)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_handle_box_set_shadow_type(GTK_HANDLE_BOX(m_Object), ShadowType);
}

void sdpGtkHandleBox::SetHandlePosition(GtkPositionType PositionType)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_handle_box_set_handle_position(GTK_HANDLE_BOX(m_Object), PositionType);
}

void sdpGtkHandleBox::SetSnapEdge(GtkPositionType SnapEdge)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_handle_box_set_snap_edge(GTK_HANDLE_BOX(m_Object), SnapEdge);
}

