
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtkmenu.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>

//////////////////////////////////////////////////////////////
// sdpGtkMenu

sdpGtkMenu::sdpGtkMenu() : sdpGtkMenuShell()
{
}

sdpGtkMenu::sdpGtkMenu(GtkMenu* Menu) : sdpGtkMenuShell(GTK_MENU_SHELL(Menu))
{
}

bool sdpGtkMenu::Create()
{
	m_Object = GTK_OBJECT(gtk_menu_new());
	return Attached();
}

bool sdpGtkMenu::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	return Create();
}

bool sdpGtkMenu::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	sdpString title;
	sdpXmlGetAttribute(Element, "title", title);
	sdpGtkMarkAttribute(Element, "title");
	if(strlen(title))
		SetTitle(title);

	return sdpGtkMenuShell::CreateOptions(ObjectContainer, Element);
}

bool sdpGtkMenu::AttachChild(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* ChildElement, sdpGtkObject* Child)
{
	Append(GTK_WIDGET(Child->Object()));
	return true;
}

void sdpGtkMenu::Append(GtkWidget* Item)
{
	// Sanity checks ..
	g_assert(Item);
	g_return_if_fail(Attached());

	gtk_menu_append(GTK_MENU(m_Object), Item);
}

void sdpGtkMenu::Prepend(GtkWidget* Item)
{
	// Sanity checks ..
	g_assert(Item);
	g_return_if_fail(Attached());

	gtk_menu_prepend(GTK_MENU(m_Object), Item);
}

void sdpGtkMenu::Insert(GtkWidget* Item, gint Position)
{
	// Sanity checks ..
	g_assert(Item);
	g_return_if_fail(Attached());

	gtk_menu_insert(GTK_MENU(m_Object), Item, Position);
}

void sdpGtkMenu::Popup(GtkWidget* ParentShell, GtkWidget* ParentItem, GtkMenuPositionFunc PositionFunction, gpointer Data, guint Button, guint32 ActivateTime)
{
	// Sanity checks ..
	g_assert(ParentShell);
	g_assert(ParentItem);
	g_return_if_fail(Attached());

	gtk_menu_popup(GTK_MENU(m_Object), ParentShell, ParentItem, PositionFunction, Data, Button, ActivateTime);
}

void sdpGtkMenu::SetTitle(SDPCSTRING Title)
{
	// Sanity checks ...
	g_assert_string(Title);
	g_return_if_fail(Attached());

	gtk_menu_set_title(GTK_MENU(m_Object), Title);
}


