
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtkmisc.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>
#include <sdpmath.h>

//////////////////////////////////////////////////////////////////////
// sdpGtkMisc

sdpGtkMisc::sdpGtkMisc() : sdpGtkWidget()
{
}

sdpGtkMisc::sdpGtkMisc(GtkMisc* Misc) : sdpGtkWidget(GTK_WIDGET(Misc))
{
}

bool sdpGtkMisc::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Alignment ...
	sdpVector2 alignment(0.5, 0.5);
	sdpXmlGetAttribute(Element, "alignment", alignment);
	sdpGtkMarkAttribute(Element, "alignment");

	SetAlignment(alignment[0], alignment[1]);

	sdpVector2 labelpadding;
	sdpXmlGetAttribute(Element, "labelpadding", labelpadding);
	sdpGtkMarkAttribute(Element, "labelpadding");
	
	SetPadding(labelpadding[0], labelpadding[1]);

	return sdpGtkWidget::CreateOptions(ObjectContainer, Element);
}

void sdpGtkMisc::SetAlignment(gfloat XAlignment, gfloat YAlignment)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_misc_set_alignment(GTK_MISC(m_Object), XAlignment, YAlignment);
}

void sdpGtkMisc::SetPadding(gint XPad, gint YPad)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_misc_set_padding(GTK_MISC(m_Object), XPad, YPad);
}
