
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtknotebook.h>
#include <sdpgtklabel.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>

sdpGtkNotebook::sdpGtkNotebook() : sdpGtkContainer()
{
}

sdpGtkNotebook::sdpGtkNotebook(GtkNotebook* Notebook) : sdpGtkContainer(GTK_CONTAINER(Notebook))
{
}

bool sdpGtkNotebook::Create()
{
	m_Object = GTK_OBJECT(gtk_notebook_new());
	return Attached();
}

bool sdpGtkNotebook::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Create our notebook ...
	return Create();
}

bool sdpGtkNotebook::AttachChild(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* ChildElement, sdpGtkObject* Child)
{
	// Sanity checks ...
	g_assert(ChildElement);
	g_assert(Child);

	sdpString tablabel;
	sdpXmlGetAttribute(ChildElement, "tablabel", tablabel);
	sdpGtkMarkAttribute(ChildElement, "tablabel");

	sdpGtkLabel label;
	label.Create(tablabel);

	AppendPage(GTK_WIDGET(Child->Object()), GTK_WIDGET(label.Object()));
	return true;
}

void sdpGtkNotebook::LeftTabs()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(m_Object), GTK_POS_LEFT);
}

void sdpGtkNotebook::RightTabs()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(m_Object), GTK_POS_RIGHT);
}

void sdpGtkNotebook::TopTabs()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(m_Object), GTK_POS_TOP);
}

void sdpGtkNotebook::BottomTabs()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(m_Object), GTK_POS_BOTTOM);
}

void sdpGtkNotebook::ShowTabs()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(m_Object), true);
}

void sdpGtkNotebook::HideTabs()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(m_Object), false);
}

void sdpGtkNotebook::ShowBorder()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_notebook_set_show_border(GTK_NOTEBOOK(m_Object), true);
}

void sdpGtkNotebook::HideBorder()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_notebook_set_show_border(GTK_NOTEBOOK(m_Object), false);
}

void sdpGtkNotebook::AppendPage(GtkWidget* Page, GtkWidget* Label)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(Page);

	gtk_notebook_append_page(GTK_NOTEBOOK(m_Object), Page, Label);
}

void sdpGtkNotebook::PrependPage(GtkWidget* Page, GtkWidget* Label)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(Page);

	gtk_notebook_prepend_page(GTK_NOTEBOOK(m_Object), Page, Label);
}

void sdpGtkNotebook::InsertPage(GtkWidget* Page, GtkWidget* Label, gint Position)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_assert(Page);

	gtk_notebook_insert_page(GTK_NOTEBOOK(m_Object), Page, Label, Position);
}

void sdpGtkNotebook::RemovePage(gint Position)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_notebook_remove_page(GTK_NOTEBOOK(m_Object), Position);
}

gint sdpGtkNotebook::CurrentPage()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), -1);

	return gint(gtk_notebook_get_current_page(GTK_NOTEBOOK(m_Object)));
}

void sdpGtkNotebook::SetPage(gint Page)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_notebook_set_page(GTK_NOTEBOOK(m_Object), Page);
}

void sdpGtkNotebook::NextPage()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_notebook_next_page(GTK_NOTEBOOK(m_Object));
}

void sdpGtkNotebook::PreviousPage()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_notebook_prev_page(GTK_NOTEBOOK(m_Object));
}
