
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtkrange.h>

sdpGtkRange::sdpGtkRange() : sdpGtkWidget()
{
}

sdpGtkRange::sdpGtkRange(GtkRange* Range) : sdpGtkWidget(GTK_WIDGET(Range))
{
}

void sdpGtkRange::SetAdjustment(GtkAdjustment* Adjustment)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_range_set_adjustment(GTK_RANGE(m_Object), Adjustment);
}

sdpGtkAdjustment sdpGtkRange::Adjustment()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), sdpGtkAdjustment());

	return sdpGtkAdjustment(gtk_range_get_adjustment(GTK_RANGE(m_Object)));
}

void sdpGtkRange::SetUpdatePolicy(GtkUpdateType Policy)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_range_set_update_policy(GTK_RANGE(m_Object), Policy);
}

bool sdpGtkRange::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	return sdpGtkWidget::CreateOptions(ObjectContainer, Element);
}

bool sdpGtkRange::AttachChild(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* ChildElement, sdpGtkObject* Child)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(ChildElement);
	g_assert(Child);

	// Is the child an adjustment?
	if(ChildElement->Name() == sdpString("adjustment"))
		{
			SetAdjustment(GTK_ADJUSTMENT(Child->Object()));
			return true;
		}

	// Default behavior ...
	return sdpGtkWidget::AttachChild(ObjectContainer, ChildElement, Child);
}

