
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the sdpGtkRuler class, which wraps GtkRuler
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <sdpgtkruler.h>
#include <sdpmath.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>

sdpGtkRuler::sdpGtkRuler() : sdpGtkWidget()
{
}

sdpGtkRuler::sdpGtkRuler(GtkRuler* Ruler) : sdpGtkWidget(GTK_WIDGET(Ruler))
{
}

bool sdpGtkRuler::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Metric ...
	sdpString metrictext = "pixels";
	sdpXmlGetAttribute(Element, "metric", metrictext);
	sdpGtkMarkAttribute(Element, "metric");

	GtkMetricType metric;
	if(metrictext == "pixels")
		metric = GTK_PIXELS;
	else if(metrictext == "inches")
		metric = GTK_INCHES;
	else if(metrictext == "cm")
		metric = GTK_CENTIMETERS;
	else
		g_assert_not_reached(); // Unknown metric type!

	SetMetric(metric);

	// Ruler range ...
	sdpVector2 range;
	sdpXmlGetAttribute(Element, "range", range);
	sdpGtkMarkAttribute(Element, "range");

	// Current position ...
	gdouble position = 0.0;
	sdpXmlGetAttribute(Element, "position", position);
	sdpGtkMarkAttribute(Element, "position");

	// Maximum size ...
	gdouble maxsize = 0.0;
	sdpXmlGetAttribute(Element, "maxsize", maxsize);
	sdpGtkMarkAttribute(Element, "maxsize");

	SetRange(range[0], range[1], position, maxsize);

	return sdpGtkWidget::CreateOptions(ObjectContainer, Element);
}

void sdpGtkRuler::SetMetric(GtkMetricType Metric)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_ruler_set_metric(GTK_RULER(m_Object), Metric);
}

void sdpGtkRuler::SetRange(gdouble Lower, gdouble Upper, gdouble Position, gdouble MaxSize)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_ruler_set_range(GTK_RULER(m_Object), Lower, Upper, Position, MaxSize);
}

