
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the sdpGtkScale class, which wraps GtkScale
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <sdpgtkscale.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>

sdpGtkScale::sdpGtkScale() : sdpGtkRange()
{
}

sdpGtkScale::sdpGtkScale(GtkScale* Scale) : sdpGtkRange(GTK_RANGE(Scale))
{
}

bool sdpGtkScale::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Number of digits to display ...
	gulong digits = 0;
	sdpXmlGetAttribute(Element, "digits", digits);
	sdpGtkMarkAttribute(Element, "digits");
	SetDigits(digits);

	// Show the value?
	bool drawvalue = true;
	sdpXmlGetAttribute(Element, "drawvalue", drawvalue);
	sdpGtkMarkAttribute(Element, "drawvalue");
	SetDrawValue(drawvalue);

	// Value position ...
	sdpString valuepositiontext = "right";
	sdpXmlGetAttribute(Element, "valueposition", valuepositiontext);
	sdpGtkMarkAttribute(Element, "valueposition");

	GtkPositionType valueposition;
	if(valuepositiontext == "left")
		valueposition = GTK_POS_LEFT;
	else if(valuepositiontext == "right")
		valueposition = GTK_POS_RIGHT;
	else if(valuepositiontext == "top")
		valueposition = GTK_POS_TOP;
	else if(valuepositiontext == "bottom")
		valueposition = GTK_POS_BOTTOM;
	else
		g_assert_not_reached(); // Unknown shadow type!

	SetValuePosition(valueposition);

	return sdpGtkRange::CreateOptions(ObjectContainer, Element);
}

void sdpGtkScale::SetDigits(gulong Digits)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_scale_set_digits(GTK_SCALE(m_Object), Digits);
}

void sdpGtkScale::SetDrawValue(bool Visible)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_scale_set_draw_value(GTK_SCALE(m_Object), Visible);
}

void sdpGtkScale::SetValuePosition(GtkPositionType Position)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_scale_set_value_pos(GTK_SCALE(m_Object), Position);
}

gint sdpGtkScale::GetValueWidth()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), 0);

	return gtk_scale_get_value_width(GTK_SCALE(m_Object));
}

