
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtkspinbutton.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>

sdpGtkSpinButton::sdpGtkSpinButton() : sdpGtkEntry()
{
}

sdpGtkSpinButton::sdpGtkSpinButton(GtkSpinButton* Button) : sdpGtkEntry(GTK_ENTRY(Button))
{
}

bool sdpGtkSpinButton::Create(gdouble ClimbRate, guint Digits)
{
	return Create(0, ClimbRate, Digits);
}

bool sdpGtkSpinButton::Create(GtkAdjustment* Adjustment, gdouble ClimbRate, guint Digits)
{
	m_Object = GTK_OBJECT(gtk_spin_button_new(Adjustment, ClimbRate, Digits));
	return Attached();
}

bool sdpGtkSpinButton::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	gdouble climbrate = 1.0;
	sdpXmlGetAttribute(Element, "climbrate", climbrate);
	sdpGtkMarkAttribute(Element, "climbrate");

	guint digits = 0;
	sdpXmlGetAttribute(Element, "digits", digits);
	sdpGtkMarkAttribute(Element, "digits");

	g_return_val_if_fail(Create(climbrate, digits), false);
	
	// Initialize our contents with the element text ...
	SetText(Element->Text());
	
	return true;
}

bool sdpGtkSpinButton::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Shadow type ...
	SetShadowType(sdpGtkGetShadowType(GTK_SHADOW_ETCHED_IN, Element));

	return sdpGtkEntry::CreateOptions(ObjectContainer, Element);
}

bool sdpGtkSpinButton::AttachChild(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* ChildElement, sdpGtkObject* Child)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(ChildElement);
	g_assert(Child);

	// Is the child an adjustment?
	if(ChildElement->Name() == sdpString("adjustment"))
		{
			SetAdjustment(GTK_ADJUSTMENT(Child->Object()));
			return true;
		}

	// Default behavior ...
	return sdpGtkEntry::AttachChild(ObjectContainer, ChildElement, Child);
}

void sdpGtkSpinButton::Configure(GtkAdjustment* Adjustment, gfloat ClimbRate, guint Digits)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_spin_button_configure(GTK_SPIN_BUTTON(m_Object), Adjustment, ClimbRate, Digits);
}

void sdpGtkSpinButton::SetAdjustment(GtkAdjustment* Adjustment)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_spin_button_set_adjustment(GTK_SPIN_BUTTON(m_Object), Adjustment);
}

sdpGtkAdjustment sdpGtkSpinButton::Adjustment()
{
	// Sanity checks ...
	g_return_val_if_fail(Attached(), sdpGtkAdjustment());

	return sdpGtkAdjustment(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(m_Object)));
}

void sdpGtkSpinButton::SetDigits(guint Digits)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(m_Object), Digits);
}

void sdpGtkSpinButton::GetValue(gdouble& Value)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	Value = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(m_Object));
}

void sdpGtkSpinButton::GetValue(gint& Value)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	Value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(m_Object));
}

void sdpGtkSpinButton::SetValue(gdouble Value)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_Object), Value);
}

void sdpGtkSpinButton::SetUpdatePolicy(GtkSpinButtonUpdatePolicy Policy)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(m_Object), Policy);
}

void sdpGtkSpinButton::SetNumeric(bool Numeric)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(m_Object), Numeric);
}

void sdpGtkSpinButton::Spin(GtkSpinType Direction, gfloat Increment)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_spin_button_spin(GTK_SPIN_BUTTON(m_Object), Direction, Increment);
}

void sdpGtkSpinButton::SetWrap(bool Wrap)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(m_Object), Wrap);
}

void sdpGtkSpinButton::SetShadowType(GtkShadowType ShadowType)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(m_Object), ShadowType);
}

void sdpGtkSpinButton::SetSnapToTicks(bool SnapToTicks)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(m_Object), SnapToTicks);
}

void sdpGtkSpinButton::Update()
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_spin_button_update(GTK_SPIN_BUTTON(m_Object));
}
