
// SDPGTK Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sdpgtkwindow.h>
#include <sdpgtkutility.h>
#include <sdpxmlutility.h>
#include <sdpmath.h>

sdpGtkWindow::sdpGtkWindow() : sdpGtkBin()
{
}

sdpGtkWindow::sdpGtkWindow(GtkWindow* Window) : sdpGtkBin(GTK_BIN(Window))
{
}

bool sdpGtkWindow::Create(GtkWindowType Type)
{
	m_Object = GTK_OBJECT(gtk_window_new(Type));
	return Attached();
}

bool sdpGtkWindow::Create(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Create the window ...
	sdpString type = "toplevel";
	sdpXmlGetAttribute(Element, "type", type);
	sdpGtkMarkAttribute(Element, "type");

	GtkWindowType windowtype = GTK_WINDOW_TOPLEVEL;
	if(type == "toplevel")
		windowtype = GTK_WINDOW_TOPLEVEL;
	else if(type == "dialog")
		windowtype = GTK_WINDOW_DIALOG;
	else if(type == "popup")
		windowtype = GTK_WINDOW_POPUP;
	else
		g_return_val_if_fail(0, false);	// Unknown window type!

	g_return_val_if_fail(Create(windowtype), false);

	// Default size ...
	sdpGtkMarkAttribute(Element, "defaultsize");
	if(sdpXmlFindAttribute(Element, "defaultsize"))
		{
			sdpVector2 defaultsize;
			sdpXmlGetAttribute(Element, "defaultsize", defaultsize);
			SetDefaultSize(gint(defaultsize[0]), gint(defaultsize[1]));
		}

	return true;
}


bool sdpGtkWindow::CreateOptions(sdpGtkIObjectContainer* ObjectContainer, sdpXmlIElement* Element)
{
	// Sanity checks ...
	g_assert(ObjectContainer);
	g_assert(Element);

	// Set the window title ...
	sdpString title;
	sdpXmlGetAttribute(Element, "title", title);
	sdpGtkMarkAttribute(Element, "title");
	SetTitle(title);

	// Set window modality ...
	bool modal = false;
	sdpXmlGetAttribute(Element, "modal", modal);
	sdpGtkMarkAttribute(Element, "modal");
	SetModal(modal);

	// Handle window resizing policy ...
	bool allowshrink = false;
	sdpXmlGetAttribute(Element, "allowshrink", allowshrink);
	sdpGtkMarkAttribute(Element, "allowshrink");

	bool allowgrow = true;
	sdpXmlGetAttribute(Element, "allowgrow", allowgrow);
	sdpGtkMarkAttribute(Element, "allowgrow");

	bool autoshrink = true;
	sdpXmlGetAttribute(Element, "autoshrink", autoshrink);
	sdpGtkMarkAttribute(Element, "autoshrink");

	SetPolicy(allowshrink, allowgrow, autoshrink);

	return sdpGtkBin::CreateOptions(ObjectContainer, Element);
}

void sdpGtkWindow::SetTitle(SDPCSTRING Title)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_window_set_title(GTK_WINDOW(m_Object), Title);
}

void sdpGtkWindow::SetFocus(GtkWidget* Widget)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	g_return_if_fail(Widget);
	
	gtk_window_set_focus(GTK_WINDOW(m_Object), Widget);
}

void sdpGtkWindow::SetPolicy(gint AllowShrink, gint AllowGrow, gint AutoShrink)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_window_set_policy(GTK_WINDOW(m_Object), AllowShrink, AllowGrow, AutoShrink);
}

void sdpGtkWindow::SetPosition(GtkWindowPosition Position)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_window_set_position(GTK_WINDOW(m_Object), Position);
}

void sdpGtkWindow::SetDefaultSize(gint Width, gint Height)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_window_set_default_size(GTK_WINDOW(m_Object), Width, Height);
}

void sdpGtkWindow::SetModal(bool Modal)
{
	// Sanity checks ...
	g_return_if_fail(Attached());

	gtk_window_set_modal(GTK_WINDOW(m_Object), Modal);
}

void sdpGtkWindow::SetTransientFor(GtkWindow* Window)
{
	// Sanity checks ...
	g_return_if_fail(Attached());
	
	gtk_window_set_transient_for(GTK_WINDOW(m_Object), Window);
}



