
// SDPTypes Library
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
// 
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


/** \file
		\brief Implements useful odds-and-ends
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpcompiler.h>
#include <sdputility.h>

#if defined SDPWIN32
#include <windows.h>
#else	// SDPWIN32
#include <sys/time.h>
#endif // SDPWIN32

#include <math.h>
#include <iostream>

void sdpSleep(const gulong Milliseconds)
{
#if defined SDPWIN32
	Sleep(Milliseconds);
#else
	timeval duration;
	duration.tv_sec = gulong(floor(Milliseconds / 1000.0));
	duration.tv_usec = 1000 * (Milliseconds - (1000 * duration.tv_sec));
	select(0, 0, 0, 0, &duration);
#endif
}

class sdpEndianTest
{
public:
	sdpEndianTest()
	{
		m_Union.Long = 1;
	}
	
	bool BigEndian()
	{
		return 1 != m_Union.Char[0];
	}
	
	bool LittleEndian()
	{
		return 1 == m_Union.Char[0];
	}
	
private:
	union
	{
		long Long;
		char Char[sizeof(long)];
	} m_Union;
};

bool sdpBigEndian()
{
	static sdpEndianTest test;
	return test.BigEndian();
}

bool sdpLittleEndian()
{
	static sdpEndianTest test;
	return test.LittleEndian();
}


