#ifndef SDPXMLATTRIBUTE_H
#define SDPXMLATTRIBUTE_H

// SDPXML Library
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


/** \file
		\brief Declares the sdpXmlAttribute class, which implements sdpXmlIAttribute
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpxmliattribute.h>
#include <sdpuserdata.h>

///////////////////////////////////////////////////////////////////////////////
// sdpXmlAttribute

/// Implements sdpXmlIAttribute, to encapsulate an XML attribute
class sdpXmlAttribute : public sdpXmlIAttribute,
												public sdpUserData
{
public:
	sdpXmlAttribute(sdpXmlIElement* ParentElement, SDPCSTRING Name, SDPCSTRING Value, const gulong Line, const gulong Column);
	virtual ~sdpXmlAttribute() {}

	// sdpIUnknown overrides ...
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface) { return SDP_E_NO_INTERFACE; }
	gulong AddRef() { return 0; }
	gulong Release() { return 0; }

	// sdpXmlIAttribute overrides ...
	sdpXmlIElement* ParentElement() const;
	SDPCSTRING Name() const;
	sdpResult SetName(SDPCSTRING Name);
	SDPCSTRING Value() const;
	sdpResult SetValue(SDPCSTRING Value);
	gulong Line() const;
	gulong Column() const;
	sdpIUserData* UserData();

protected:
	/// Stores the parent element
	sdpXmlIElement* m_ParentElement;
	/// Stores the attribute name
	sdpString m_Name;
	/// Stores the attribute value
	sdpString m_Value;
	/// Stores the attribute line number
	gulong m_Line;
	/// Stores the attribute column number
	gulong m_Column;
};

#endif SDPXMLATTRIBUTE_H
