#ifndef SDPXMLDOCUMENT_H
#define SDPXMLDOCUMENT_H

// SDPXML Library
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


/** \file
		\brief Declares the sdpXmlDocument class, which implements sdpXmlIDocument
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpcompiler.h>
#include <sdpuserdata.h>

#include <sdpxmlidocument.h>
#include <sdpxmlsax.h>

#include <iostream>
#include <vector>
#include <stack>

// Forward class declarations ...
class sdpXmlElement;
class sdpXmlIElement;

///////////////////////////////////////////////////////////////////////////////
// sdpXmlDocument

/// Implements sdpXmlIDocument, to encapsulate an XML document
class sdpXmlDocument : public sdpXmlIDocument,
											 public sdpXmlISAXHandler,
											 public sdpUserData
{
public:
	sdpXmlDocument();
	virtual ~sdpXmlDocument();

	/// Clears the document contents
	void Clear();

	// Loads and parses an XML document ...
	bool LoadFile(SDPCSTRING FilePath);
	bool Load(sdpString& Buffer, SDPCSTRING FilePath);
	bool Load(std::istream& Stream, SDPCSTRING FilePath);
	
	// Saves an XML document ...
	bool SaveFile(SDPCSTRING FilePath) const;
	bool Save(std::ostream& Stream) const;

	// sdpIUnknown overrides ...
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface) { return SDP_E_NO_INTERFACE; }
	gulong AddRef() { return 0; }
	gulong Release() { return 0; }

	// sdpXmlIDocument overrides ...
	SDPCSTRING FilePath() const;
	sdpXmlIElement* GetRootElement() const;
	sdpXmlIElement* CreateRootElement(SDPCSTRING Name);
	sdpResult DestroyRootElement();
	sdpResult SetIndentation(SDPCSTRING Indentation);
	SDPCSTRING GetIndentation() const;
	sdpIUserData* UserData();

	// sdpXmlISAXHandler overrides ...
	void StartDocument();
	void EndDocument();
	void ProcessingInstruction(SDPCSTRING Target, SDPCSTRING Data);
	void StartElement(SDPCSTRING Name, const sdpStringArray& Attributes);
	void EndElement();
	void Characters(SDPCSTRING Text);
	void CDATABlock(SDPCSTRING Value);
	void Warning(SDPCSTRING Message);
	void Error(SDPCSTRING Message);
	void FatalError(SDPCSTRING Message);

protected:
	// Helper functions for writing documents ...
	void Save(std::ostream& Stream, gulong Level, sdpXmlIElement* Element) const;
	void Save(std::ostream& Stream, sdpXmlIAttribute* Attribute) const;
	sdpString Indentation(gulong Level) const;
	
	// Parsing state ...
//	gulong CurrentLine() const;
//	gulong CurrentColumn() const;
	
	// Create a stack for keeping track of the "current" element ...
	typedef std::stack<sdpXmlElement*> ElementStack;
	ElementStack m_ElementStack;

	// The "current" element ...
	sdpXmlElement* CurrentElement() const;

	sdpString m_FilePath;
	sdpXmlElement* m_RootElement;
	sdpString m_Indentation;
	
//	void* m_ParserContext;
};

/// Writes an XML document to a stream
std::ostream& operator << (std::ostream& Stream, const sdpXmlDocument& Document);
/// Writes an XML element to a stream (recursive, but without fancy formatting)
std::ostream& operator << (std::ostream& Stream, const sdpXmlIElement* const Element);

#endif SDPXMLDOCUMENT_H
