#ifndef SDPXMLELEMENT_H
#define SDPXMLELEMENT_H

// SDPXML Library
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


/** \file
		\brief Declares the sdpXmlElement class, which implements sdpXmlIElement
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpxmlielement.h>
#include <sdpuserdata.h>
#include <vector>

// Forward class references ...
class sdpXmlElement;
class sdpXmlAttribute;

///////////////////////////////////////////////////////////////////////////////
// sdpXmlElement

/// Implements sdpXmlIElement, to encapsulate an XML element
class sdpXmlElement : public sdpXmlIElement,
											public sdpUserData
{
friend class sdpXmlDocument;
public:
	sdpXmlElement(sdpXmlIDocument* Document, sdpXmlIElement* ParentElement, SDPCSTRING Name, const sdpStringArray& Attributes, const gulong Line, const gulong Column);
	virtual ~sdpXmlElement();

	// sdpIUnknown overrides ...
	sdpResult QueryInterface(const sdpIID& InterfaceID, void** Interface) { return SDP_E_NO_INTERFACE; }
	gulong AddRef() { return 0; }
	gulong Release() { return 0; }

	// sdpXmlIElement overrides ...
	sdpXmlIDocument* Document() const;
	sdpXmlIElement* ParentElement() const;
	SDPCSTRING Name() const;
	sdpResult SetName(SDPCSTRING Name);
	SDPCSTRING Text() const;
	sdpResult SetText(SDPCSTRING Text);
	gulong Line() const;
	gulong Column() const;
	SDPXMLATTRIBUTEHANDLE GetFirstAttribute() const;
	SDPXMLATTRIBUTEHANDLE GetNextAttribute(SDPXMLATTRIBUTEHANDLE Attribute) const;
	sdpXmlIAttribute* GetAttribute(SDPXMLATTRIBUTEHANDLE Attribute) const;
	sdpXmlIAttribute* CreateAttribute(SDPCSTRING Name, SDPCSTRING Value);
	sdpResult DestroyAttribute(sdpXmlIAttribute* Attribute);
	SDPXMLELEMENTHANDLE GetFirstElement() const;
	SDPXMLELEMENTHANDLE GetNextElement(SDPXMLELEMENTHANDLE Element) const;
	sdpXmlIElement* GetElement(SDPXMLELEMENTHANDLE Element) const;
	sdpXmlIElement* CreateElement(SDPCSTRING Name);
	sdpResult DestroyElement(sdpXmlIElement* Element);
	sdpIUserData* UserData();

protected:
	/// The parent document
	sdpXmlIDocument* m_Document;
	/// The parent element
	sdpXmlIElement* m_ParentElement;
	/// Stores the element name
	sdpString m_Name;
	/// Stores the element text
	sdpString m_Text;
	/// Stores the XML file line number
	gulong m_Line;
	/// Stores the XML file column number
	gulong m_Column;

	/// Stores child elements
	typedef std::vector<sdpXmlElement*> Elements;
	typedef Elements::iterator ElementIterator;
	typedef Elements::const_iterator ConstElementIterator;
	Elements m_Elements;

	/// Stores attributes
	typedef std::vector<sdpXmlAttribute*> Attributes;
	typedef Attributes::iterator AttributeIterator;
	typedef Attributes::const_iterator ConstAttributeIterator;
	Attributes m_Attributes;
};

#endif SDPXMLELEMENT_H
