#ifndef SDPXMLIDOCUMENT_H
#define SDPXMLIDOCUMENT_H

// SDPXML Library
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


/** \file
		\brief Declares the sdpXmlIDocument interface
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpxmlielement.h>

/// Interface to an XML document
class sdpXmlIDocument
{
public:
	/// Returns the filepath to the XML document
	virtual SDPCSTRING FilePath() const = 0;
	/// Returns the top-level element in the XML document
	virtual sdpXmlIElement* GetRootElement() const = 0;
	/// Creates a new root-level element
	virtual sdpXmlIElement* CreateRootElement(SDPCSTRING Name) = 0;
	/// Destroys the root element (equivalent to clearing the document)
	virtual sdpResult DestroyRootElement() = 0;
	/// Sets the char(acters) that will be used to indent any output text
	virtual sdpResult SetIndentation(SDPCSTRING Indentation) = 0;
	/// Returns the char(acters) that will be used to indent any output text
	virtual SDPCSTRING GetIndentation() const = 0;
	/// Returns an sdpIUserData interface to the underlying XML document
	virtual sdpIUserData* UserData() = 0;
};

#endif SDPXMLIDOCUMENT_H
