#ifndef SDPXMLIELEMENT_H
#define SDPXMLIELEMENT_H

// SDPXML Library
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


/** \file
		\brief Declares the sdpXmlIElement interface
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpxmliattribute.h>

#include <vector>
#include <list>
#include <set>

// Forward class declarations
class sdpXmlIDocument;
class sdpXmlIElement;

/// Opaque handle for working with XML elements
SDP_DECLARE_HANDLE(SDPXMLELEMENTHANDLE);

/// sdpCOM interface for an XML element
class sdpXmlIElement
{
public:
	/// Returns the owning document
	virtual sdpXmlIDocument* Document() const = 0;
	/// Returns the parent element
	virtual sdpXmlIElement* ParentElement() const = 0;
	/// Returns the element name
	virtual SDPCSTRING Name() const = 0;
	/// Sets the element name
	virtual sdpResult SetName(SDPCSTRING Name) = 0;
	/// Returns the element text
	virtual SDPCSTRING Text() const = 0;
	/// Sets the element text
	virtual sdpResult SetText(SDPCSTRING Text) = 0;
	/// Returns the XML file line number of the element
	virtual gulong Line() const = 0;
	/// Returns the XML file column of element
	virtual gulong Column() const = 0;
	/// Returns an opaque handle to the first attribute in this element
	virtual SDPXMLATTRIBUTEHANDLE GetFirstAttribute() const = 0;
	/// Returns an opaque handle to the next attribute in the element
	virtual SDPXMLATTRIBUTEHANDLE GetNextAttribute(SDPXMLATTRIBUTEHANDLE Attribute) const = 0;
	/// Converts an attribute handle to an sdpXmlIAttribute interface
	virtual sdpXmlIAttribute* GetAttribute(SDPXMLATTRIBUTEHANDLE Attribute) const = 0;
	/// Creates a new attribute
	virtual sdpXmlIAttribute* CreateAttribute(SDPCSTRING Name, SDPCSTRING Value) = 0;
	/// Destroys an attribute
	virtual sdpResult DestroyAttribute(sdpXmlIAttribute* Element) = 0;
	/// Returns an opaque handle to the first child of this element
	virtual SDPXMLELEMENTHANDLE GetFirstElement() const = 0;
	/// Returns an opaque handle to the next child of this element
	virtual SDPXMLELEMENTHANDLE GetNextElement(SDPXMLELEMENTHANDLE Element) const = 0;
	/// Converts an element handle to an sdpXmlIElement interface
	virtual sdpXmlIElement* GetElement(SDPXMLELEMENTHANDLE Element) const = 0;
	/// Creates a new child element
	virtual sdpXmlIElement* CreateElement(SDPCSTRING Name) = 0;
	/// Destroys a child element
	virtual sdpResult DestroyElement(sdpXmlIElement* Element) = 0;
	/// Returns an sdpIUserData interface to the underlying element
	virtual sdpIUserData* UserData() = 0;
};

/// Contains an array of elements
typedef std::vector<sdpXmlIElement*> sdpXmlElementArray;
/// Contains a linked-list of elements
typedef std::list<sdpXmlIElement*> sdpXmlElementList;
/// Contains a set of elements
typedef std::set<sdpXmlIElement*> sdpXmlElementSet;

#endif SDPXMLIELEMENT_H
