// SDPXML
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the xpathtest command-line application, which can query data from an XML file using an XPath specification
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <sdpxml.h>
#include <iostream>

int main(int argc, char* argv[])
{
	// Test our arguments ...
	if(argc != 3)
		{
			std::cout << "Usage: xpathtest [xml file] [xpath specification]" << std::endl;
			return 1;
		}
	
	// Load the XML document ...
	sdpXmlDocument doc;
	g_return_val_if_fail(doc.LoadFile(argv[1]), 1);
	
	// Perform an XPath selection ...
	sdpXmlElementArray results = sdpXmlXPathSelect(&doc, argv[2]);
	
	// Print the results ...
	for(int i = 0; i < results.size(); i++)
		std::cout << results[i];
		
	return 0;
}
